#!/bin/sh 
# #!/bin/sh -xv  
# Use -xv option to debug
#   
# $Id: install.sh,v 1.111 2012/04/18 21:58:45 root Exp root $
#
# The following code is Confidential and is covered by the installation license
# Copyright 2004-2012 by Midrange Performance Group
#
#&& Installs the Power Navigator Data Collector
#
#&% install.sh and mpgpn.tar must be a matched set of files
#&%
#&% install.sh version must match version of manifest
#&%
#&% Script must stay posix compliant
#&%
#&% To execute install.sh script type:
#&%    chmod 540 install.sh  # Makes the install.sh executable
#&%    ./install.sh   
#
#& Using ./install.sh without option will install in /usr/local/mpg
#& This is the recommended installation directory
#
#&@ mpgpn.tar is also required. 
#
install_sh_ver=`grep "\$\I\d\:" ${0} | awk '{print $3,$4,$5,$6}' ` 
echo "Running $install_sh_ver"
#
copyright="
# The following code is Confidential and is covered by the installation license
# Copyright 2004-2012 by Midrange Performance Group
# "

installer_version=`grep "\$\I\d\:" ${0} | awk '{print $4}'` 
startdir=`pwd`
tarfile="mpgpn.tar" 
def_tarfile="mpgpn.tar" 
jdate=`date +%Y%m%d_%H%M`
# debug="T"

## Randy Watson 3/5/2009
# 	Updated to add nmon rhel5 and sles10 for mainframe
#	Add vcx support for power (3Com)
#	Add check to install nmon 12f for 6.1.2
#
## Jack Miller 4/13/2009
# 	Updated to add nmon sles11 for mainframe
## Jack Miller 4/16/2009
#       Updated functions to sh/bourne standard syntax
#       These do not use the function keyword
## Jack Miller 4/22/2009
#       Finished changes for 12a binaries
#       Updated sum/sizes for 3.3.0 mpgpn.tar
#	Added chk_ksh function to create soft link if /bin/ksh does not exist
## Jack Miller 6/25/2009
#       Significant changes to several scripts to handle HP and Solaris
## Jack Miller 1/19/2010
#       Added in nmon 12f compiles on several versions of linux
#       Numberous script changes to archive data, reduced reduncy, 
#       and added configuration controls with comments
## Jack Miller 2/10/2010
#       Added nmon merge for ia64 and ppc64
#       Script changes to rackdata.sh, and added nmonsource for nmonmerge 
#       and nmon. Added dmidecode for ia64.

##### :! csum mpgpn.tar; cksum mpgpn.tar; sum mpgpn.tar; ls -l mpgpn.tar ####
##### begin package values
package_md5="ca0ec34fedd5cb08debec49f06ee695b  mpgpn.tar"
package_cksum="2395656155 29982720 mpgpn.tar"
package_sum="00049"
package_size="29982720"
##### end package values

# Create a soft link for ksh at /bin/ksh if it does not exist
chk_ksh () {
kshlocation=`type /bin/ksh  2>/dev/null | sed -e 's,^.*\ \/,\/,' | grep -v "not found"`
# echo "(${kshlocation})"
if [ "x${kshlocation}x" = "xx" ]; then
       # /bin/ksh was not found, lQQk for ksh
       kshlocation=`type ksh  2>/dev/null | sed -e 's,^.*\ \/,\/,' | grep -v "not found"`
       if [ "x${kshlocation}x" != "xx" ]; then
          # if an alternate location to ksh exists create a soft link
          ln -s ${kshlocation} /bin/ksh
       else
          # if an alternate location to ksh does not exists, create a soft link to bash
          kshlocation=`type bash  2>/dev/null | sed -e 's,^.*\ \/,\/,' | grep -v "not found"`
          if [ "x${kshlocation}x" != "xx" ]; then
             ln -s ${kshlocation} /bin/ksh
          else
             echo "Problems identifying a location to create a soft link from for /bin/ksh"
             exit 1
          fi
       fi  
       echo "created a soft link from ${kshlocation} to /bin/ksh using the following command:"
       echo ln -s ${kshlocation} /bin/ksh
   else
       echo "/bin/ksh was found"
fi
}
# Run function chk_ksh to see if there is a /bin/ksh link or file
# and create a soft link to ksh or bash if it does not
chk_ksh


# Locate applications using normal pathing.
locate_app () {
   # Identify locations of applications
   # Check normal bin sbin directories for application
   for jdirs in /bin/ /sbin/ /usr/bin/ /usr/sbin/ /usr/local/bin/ /usr/local/sbin/ /usr/contrib/bin/ /usr/contrib/sbin/ /opt/hpvm/bin/
   do
      if [ -x ${jdirs}${1} ]; then
         jloc="${jdirs}${1}"
         break
      fi
   done
   if [ "x${jloc}x" != "xx" ]; then
      export jloc
      return 0
   else
      # HP-UX  required the grep "^no "
      jloc=`which $1 2>/dev/null | grep -v "^no "`
      if [ "x${jloc}x" != "xx" ]; then
         export jloc
         return 0
      else
         return 1
      fi
   fi
}


dmi_lookup () {
   if [ "x${1}x" = "xx" ] ; then
      echo "dmi_lookup must have and item to lookup"
      return
   fi
   if [ -s /usr/sbin/dmidecode ] ; then
      DMI="/usr/sbin/dmidecode"         
   else                                    
      DMI="${nmondir}/pntestdmi"        
   fi
   # echo "$DMI -s $1" # for tesing only command is placed in pn.config 
   jtest=`$DMI -s $1`                            
      echo $jtest | grep "sorry" > /dev/null 
   jgreptest=$?                              
   if [ ${jgreptest} -eq 0 ]                 
   then                                      
      echo $jtest                            
      echo "### This may be an old server ###"
   fi      
      $DMI -s $1 
}

usage () {
echo "
USAGE: install.sh [options]
where options may be any of:
  -i <install_dir>	install directory
  -o <collected_data>	full path to file that will contain the collected data
  -s <sar_directory>	full path to the directory containing sar data files
  -t <PowerNav tarfile>	full path to the Power Navigator package tar file
  -c <config_file>	full path to configuration file
  -S			install the Power Navigator sar data collector
  -R			re-install (replace data collector and cron entries)
  -f			force install when prereqs not satisfied - may not work!
  -H <shellpath>	full path to shell to use for installed scripts (ksh or bash)
  -r			do not reduce data (normally eliminates some extraneous data)
  -z			do not compress nmon daily files
  -Z			do not compress data file
  -v			verbose output
  -D			Debug output (even more verbose)
  -V			show installer Version
"
}

print_greeting () {
  echo "MPG Power Navigator data collector installer $installer_version"
}

chk_tar () {
tarname=$1
echo "tarname=(${tarname})"
# Confirm tarname and return if exists
# If tarname does not exist, see if a gzipped or compressed version exists.
# If a gzip version exists, gunzip it.
# If a compressed version exists, uncompress it.
# 
targzipped="${tarname}.gz"
tarcompressed="${tarname}.Z"
if [ -f ${tarname} ] ; then
   return 
elif [ -f ${targzipped} ]; then 
   echo $targzipped exists, gunzip $targzipped
   gunzip $targzipped
   return
elif [ -f ${tarcompressed} ]; then 
   echo $tarcompressed exists, uncompress $tarcompressed
   uncompress $tarcompressed
   return
else
   echo Could not find $tarname, $targzipped, or $tarcompressed
   echo Please ensure one of these files exists
   exit 1
fi
}

locate_app uname
juname="$jloc"

get_ostype () {
 ostype=`$juname -s`
 case $ostype in
   "AIX")	idcmd="id" ; whichcmd="which" ;;
   "Linux")	idcmd="id" ; whichcmd="which" ;;
   "HP-UX")	idcmd="id" ; whichcmd="fixwhich" ;;
   "SunOS")     idcmd="/usr/xpg4/bin/id" ; whichcmd="fixwhich" ;;
    *)		idcmd="id" ; whichcmd="fixwhich" ;;
 esac
}

fixwhich () {
# a "which" function to replace the ones created by (sun,hpux)
# that always return 0 (success), even on a failure...
  what=$1
  ret=`which $what`
  if [ -z "$ret" ] ; then  return 1 ;
  else
    if [ -f "$ret" ] ; then return 0 ;
    else   return 1;
    fi
  fi
}

get_manifest () {
 tar xvf $tarfile manifest 
 rc=$?
	if [ "$debug" = "T" ] ; then echo "rc from tar (untar manifest) : $retcode" ; fi
 if [ "$rc" != "0" ] ; then
   echo "ERROR ($rc) trying to untar manifest - aborting" ;
   exit 1;
 fi
 . ./manifest
 # if [ "$installer_version" != "$manifest_version" ] ; then
 #   echo "ERROR - files in mpgpn.tar do not match installer version " ;
 #   echo "  Please contact MPG Inc. for correct installation package " ;
 #   exit 1;
 # fi
}

verify_pkg () {
 if [ ! -z "$just_verify_pkg" ] ; then save_verbose=$verbose; verbose="T"; fi
 
 get_manifest

 #####  Need to check if md5sum, csum, or sum is available?
 #####  First check if md5sum is available?
 $whichcmd md5sum > /dev/null 2>&1 
 chk_md5sum=$?
 #####  Second check if csum is available?
 $whichcmd csum > /dev/null 2>&1 
 chk_csum=$?

 if [ "$chk_md5sum" = "0" ] ; then
   # The awk statement below is required, because some versions of md5sum
   # place one space instead of two in the output.  This will ensure it is 
   # always two spaces between the md5sum and the filename.
   test_md5=`md5sum mpgpn.tar|awk '{print $1"  "$2}'` 
   if [ "x${package_md5}x" != "x${test_md5}x" ] ; then
     echo "ERROR - package mpgpn.tar failed md5sum check "
     echo "  Please contact MPG Inc. for correct installation package " ;
     exit 1
   else 
     if [ ! -z "$verbose" ] ; then echo "package md5sum OK" ; fi 
   fi
 elif [ "$chk_csum" = "0" ] ; then 
   # csum output is the same as md5sum output.
   csum_package_chk=`csum mpgpn.tar`
   if [ "x${package_md5}x" != "x${csum_package_chk}x" ] ; then
     echo "ERROR - package mpgpn.tar failed csum check "
     echo "  Please contact MPG Inc. for correct installation package " ;
     exit 1
   else
     if [ ! -z "$verbose" ] ; then echo "package md5sum from csum OK" ; fi
   fi
 else
   if [ ! -z "$verbose" ] ; then echo "md5sum and csum are not available, trying sum" ; fi
   $whichcmd sum > /dev/null 2>&1
   rc=$?
   if [ "$rc" = "0" ] ; then
     ## have to use "sum -r" - solaris uses old sum 
     checkpkgsum=`sum -r mpgpn.tar | awk '{print $1}'`
     if [ "$package_sum" != "$checkpkgsum" ] ; then
       echo "ERROR - package mpgpn.tar failed checksum "
       echo "  Please contact MPG Inc. for correct installation package " ;
       exit 1 
     else
       if [ ! -z "$verbose" ] ; then echo "package chksum OK" ; fi
     fi
   else
     # md5sum, csum and sum are not available, check package size.
     if [ ! -z "$verbose" ] ; then 
       echo "md5sum, csum, and sum are not available, checking size" 
     fi
     checkpkgsize=`wc -c mpgpn.tar | awk '{print $1}'`
     if [ "$package_size" != "$checkpkgsize" ] ; then
       echo "ERROR - package mpgpn.tar wrong size "
       echo "  Please contact MPG Inc. for correct installation package " ;
       exit 1
     else
       if [ ! -z "$verbose" ] ; then echo "package size OK" ; fi
     fi
   fi
 fi
 if [ ! -z "$just_verify_pkg" ] ; then
   verbose=$save_verbose;
   exit 0;
 fi
}


while getopts "xVDvo:s:i:t:c:l:SRrzZfH:" opt ; do
  case $opt in
	x)	testplatform="T" ;;	##### remove for production !!! #####
	V)	just_verify_pkg="T" ;;
	D)	debug="T" ;;
	v)	verbose="T" ;;
	o)      opt_datafile=$OPTARG ;;
	l)      opt_datacur=$OPTARG ;;
	s)      opt_sardir=$OPTARG ;;
	i)      opt_instdir=$OPTARG ;;
	t)      opt_tarfile=$OPTARG ;;
	c)      opt_pnconfig=$OPTARG ;;
	S)	install_sar="T" ;;
	R)	reinstall="T" ;;
	f)	force_install="T" ;;
	H)      opt_shelltouse=$OPTARG ;;
	r)	reduce="T" ;;
	z)	compressnmon="T" ;;
	Z)	compressdatafile="T" ;;
	?)	usage; exit ;;
  esac
done
shift `expr $OPTIND - 1`
if [ "$debug" = "T" ] ; then verbose="T" ; fi
## if force, set opt_sardir to something - checks will assume correct and move on
if [ "$force_install" = "T" ] ; then
  if [ -z "$opt_sardir" ] ; then
    $opt_sardir="/var/log/sardir"
  fi
fi

check_for_root () {
## check that we are running as root
runningas=`$idcmd -u`
runningasname=`$idcmd -u -n`
if [ "$runningas" != "0" ] ; then
  echo "ERROR - install must be run as root - current userid is $runningas($runningasname)"
	if [ "$testplatform" = "T" ] ; then
		echo "testplatform set - continuing anyway - install may fail"
	else
  exit 1
	fi
fi
}

set_default_variables () {
 ## default and os specific default settings
 ## note that the order of some of these is important - ie $instdir must be set before $sar_cron_entry, etc.
 ## default mpgdata file location
 defdir="/usr/local/mpg"
 defconfig="pn.config"
 if [ "$install_sar" = "T" ] ; then
	defmpgfile=mpgdata ;	# default for sar
	defmpgcur=mpgdata_current ;	# default for sar
 else
	defmpgfile=mpgdata2 ;	# default for nmon
	defmpgcur=mpgdata2_current ;	# default for nmon
 fi
 if [ "X$opt_instdir" = "X" ] ; then
	instdir=$defdir;
 else
	instdir=`echo $opt_instdir | sed s,\/$,,`;
 fi
 if [ "X$opt_pnconfig" = "X" ] ; then
	pnconfig=$instdir/$defconfig ;
 else
	pnconfig=$opt_pnconfig ;
 fi

 ### regex's to find old installations
 nmonregex_old="$instdir"';.*nmon -F.*date +mpg'
 nmonregex="$instdir"'.*/start_nmon_now -f'	## gets both cd <dir> and direct versions
 cendregex="$instdir"'.*/cend.sh -c'
 pndcmregex="$instdir"'.*/pndcm.sh -c'
 sdcmregex="$instdir"'.*/sdcm.sh -c'
 rackregex="$instdir"'.*/rackdata.sh'
 truncregex="$instdir"'.*/pndc_truncate.sh'
 pndchkregex="$instdir"'.*/pndchk.sh'
 pndchksarregex="$instdir"'.*/pndchk_sar.sh'
 ## command to see if power navigator (pngetinfo) is already in cron
 pngetinforegex="$instdir"'.*/pngetinfo.sh -q'
 ## command to see if power navigator (pndcm) is already in cron
 pndcmrunning_AIX="crontab -l | grep -v '^#' | grep \"$pndcmregex\""
 pndcmrunning_linux="crontab -l | grep -v '^#' | grep \"$pndcmregex\""
 pndcmrunning_generic="crontab -l | grep -v '^#' | grep \"$pndcmregex\""
 # pndcmrunning_generic="crontab -l | grep -v '^#' | grep pndcm"

 ## command to see if nmon is configured to run in default config
 nmonrunning_AIX_old="crontab -l | grep -v '^#' | grep \"$nmonregex_old\""
 nmonrunning_linux_old="crontab -l | grep -v '^#' | grep \"$nmonregex_old\""
 nmonrunning_generic_old="crontab -l | grep -v '^#' | grep \"$nmonregex_old\""
 nmonrunning_AIX="crontab -l | grep -v '^#' | grep \"$nmonregex\""
 nmonrunning_linux="crontab -l | grep -v '^#' | grep \"$nmonregex\""
 nmonrunning_generic="crontab -l | grep -v '^#' | grep \"$nmonregex\""

 ## Power Navigator nmon data collector crontab entries
 pndcm_cron_entry="25 4 * * * cd $instdir; $instdir/cend.sh -c $pnconfig"
 #OLD# nmon_cron_entry="1 0 * * * cd $instdir; $instdir/start_nmon_now -f"
 nmon_cron_entry="1,16,31,46 * * * * cd $instdir; $instdir/start_nmon_now -f -q"
 pndcm_clean_cron_entry="35 4 * * * cd $instdir; $instdir/pndcm.sh -c $pnconfig clean"
 ## pndchk.sh will run monthly on the 2nd 
 pndchk_cron_entry="30 5 2 * * cd $instdir; $instdir/pndchk.sh " 

 ## pndchk_sar.sh will run monthly on the 2nd 
 pndchk_sar_cron_entry="30 5 2 * * cd $instdir; $instdir/pndchk_sar.sh " 

 ### sar specific default values
 ## defsardir = default sar data location
 defsardir_AIX="/var/adm/sa" ;
 defsardir_HPUX="/var/adm/sa" ;
 defsardir_SunOS="/var/adm/sa" ;
 defsardir_linux="/var/log/sa" ;
 defsardir_linux_debian="/var/log/sysstat" ;
 defsardir_generic="/var/adm/sa" ;

 ## sar package command
 sarq_AIX="lslpp -qLc bos.acct" ;
 sarq_linux="rpm -q sysstat" ;
 #####	same on Suse ???
 sarq_linux_debian="dpkg -s sysstat"
 sarq_generic="$whichcmd sar"

 ## command to see if sar is configured to run in default config
 sarrunning_AIX='grep -s sa1 /var/spool/cron/crontabs/* /var/spool/cron/tabs/* /var/spool/cron/*'
 sarrunning_SunOS='grep -s sa1 /var/spool/cron/crontabs/* /var/spool/cron/tabs/* /var/spool/cron/*'
 sarrunning_HPUX='grep -s sa1 /var/spool/cron/crontabs/* /var/spool/cron/tabs/* /var/spool/cron/*'
 sarrunning_linux='grep -s sa1 /var/spool/cron/crontabs/* /var/spool/cron/tabs/* /var/spool/cron/*'
 sarrunning_linux_debian='grep -s sa1 /var/spool/cron/crontabs/* /var/spool/cron/tabs/* /var/spool/cron/*'
 sarrunning_generic='grep -s sa1 /var/spool/cron/crontabs/* /var/spool/cron/tabs/* /var/spool/cron/*'


 ## command to see if power navigator (sdcm) is already in cron
 sdcmrunning_AIX="crontab -l | grep -v '^#' | grep \"$sdcmregex\""
 sdcmrunning_SunOS="crontab -l | grep -v '^#' | grep \"$sdcmregex\""
 sdcmrunning_HPUX="crontab -l | grep -v '^#' | grep \"$sdcmregex\""
 sdcmrunning_linux="crontab -l | grep -v '^#' | grep \"$sdcmregex\""
 sdcmrunning_linux_debian="crontab -l | grep -v '^#' | grep \"$sdcmregex\""
 sdcmrunning_generic="crontab -l | grep -v '^#' | grep \"$sdcmregex\""

 ## command to see if power navigator (pngetinfo -q) is already in cron
 pngetinforunning_AIX="crontab -l | grep -v '^#' | grep \"$pngetinforegex\""
 pngetinforunning_SunOS="crontab -l | grep -v '^#' | grep \"$pngetinforegex\""
 pngetinforunning_HPUX="crontab -l | grep -v '^#' | grep \"$pngetinforegex\""
 pngetinforunning_linux="crontab -l | grep -v '^#' | grep \"$pngetinforegex\""
 pngetinforunning_linux_debian="crontab -l | grep -v '^#' | grep \"$pngetinforegex\""
 pngetinforunning_generic="crontab -l | grep -v '^#' | grep \"$pngetinforegex\""


 ## sar crontab entries
 sar_cron_entry_AIX='0,5,10,15,20,25,30,35,40,45,50,55 * * * * /usr/lib/sa/sa1 &'
 sar_cron_entry_SunOS='0,5,10,15,20,25,30,35,40,45,50,55 * * * * /usr/lib/sa/sa1 &'
 sar_cron_entry_HPUX='0,5,10,15,20,25,30,35,40,45,50,55 * * * * /usr/lib/sa/sa1 &'
 sar_cron_entry_linux='0,5,10,15,20,25,30,35,40,45,50,55 * * * * /usr/lib/sa/sa1 &'
 sar_cron_entry_linux_debian='0,5,10,15,20,25,30,35,40,45,50,55 * * * * /usr/lib/sysstat/sa1 &'
 sar_cron_entry_generic='0,5,10,15,20,25,30,35,40,45,50,55 * * * * /usr/lib/sa/sa1 &'

 ## Power Navigator sar data collector (sdcm) crontab entries
 sdcm_cron_entry_AIX="35 4 * * * cd $instdir; ./sdcm.sh -c $pnconfig collect all"
 sdcm_cron_entry_HPUX="35 4 * * * cd $instdir; ./sdcm.sh -c $pnconfig collect all"
 sdcm_cron_entry_SunOS="35 4 * * * cd $instdir; ./sdcm.sh -c $pnconfig collect all"
 sdcm_cron_entry_linux="35 4 * * * cd $instdir; ./sdcm.sh -c $pnconfig collect all"
 sdcm_cron_entry_linux_debian="35 4 * * * cd $instdir; ./sdcm.sh -c $pnconfig collect all"
 sdcm_cron_entry_generic="35 4 * * * cd $instdir; ./sdcm.sh -c $pnconfig collect all"

 ## Power Navigator pngetinfo quietly get mpg data files, pn.config, and rackdata file
 pngetinfo_cron_entry_AIX="0 5 * * * cd $instdir; ./pngetinfo.sh -qa"
 pngetinfo_cron_entry_HPUX="0 5 * * * cd $instdir; ./pngetinfo.sh -qa"
 pngetinfo_cron_entry_SunOS="0 5 * * * cd $instdir; ./pngetinfo.sh -qa"
 pngetinfo_cron_entry_linux="0 5 * * * cd $instdir; ./pngetinfo.sh -qa"
 pngetinfo_cron_entry_linux_debian="0 5 * * * cd $instdir; ./pngetinfo.sh -qa"
 pngetinfo_cron_entry_generic="0 5 * * * cd $instdir; ./pngetinfo.sh -qa"

 ## command to see if rack config is already in cron
 rackrunning_AIX="crontab -l | grep -v '^#' | grep \"$rackregex\""
 rackrunning_SunOS="crontab -l | grep -v '^#' | grep \"$rackregex\""
 rackrunning_HPUX="crontab -l | grep -v '^#' | grep \"$rackregex\""
 rackrunning_linux="crontab -l | grep -v '^#' | grep \"$rackregex\""
 rackrunning_linux_debian="crontab -l | grep -v '^#' | grep \"$rackregex\""
 rackrunning_generic="crontab -l | grep -v '^#' | grep \"$rackregex\""
 # rackrunning_generic="crontab -l | grep -v '^#' | grep rackdata"

 ## rack config crontab entry
 rack_cron_entry="30 4 * * * cd $instdir; $instdir/rackdata.sh -c $pnconfig"

 chk_tar $tarfile

 verify_pkg

 files_for_all="$pkg_rack_files $pkg_info_files $pkg_install_files"
 files_for_nmon="$pkg_pndcm_files $pkg_nmon_utils $pkg_pndcm_utils" ;
 # nmonver, nmonmergever, reducepatt are added later
 files_for_sar="$pkg_sdcm_files"
 ## generic cron comments
 raw_cron_header="# MPG Power Navigator data collector Version"
 cron_header="# MPG Power Navigator data collector Version $installer_version"
 # cron_trailer="#"
 ## temporary file that crontab will be stored in
 crontab_file_AIX="crontab.root"
 crontab_file_linux="crontab.root"
 crontab_file_generic="crontab.root"
 ## command to get crontab to edit
 cron_get_AIX="crontab -l > $crontab_file_AIX"
 ##### this removes the header lines on linux - new headers are added when crontab installs
 #####	not really safe (just rip out 3 lines) - should pattern match 'em
# cron_get_linux="crontab -l | sed -e '1,3d' > $crontab_file_linux"
########### removed the rip 3 lines for now - need to test...
 cron_get_linux="crontab -l > $crontab_file_linux"
 cron_get_generic="crontab -l > $crontab_file_generic"
 ## command to install new crontab
 cron_install_AIX='su -c "crontab $crontab_file_AIX"'
 cron_install_linux="crontab $crontab_file_linux"
 cron_install_generic="crontab $crontab_file_generic"

 ## get the machine name
 get_machine_name_AIX="$juname -n"
 get_machine_name_linux="$juname -n"
 get_machine_name_generic="$juname -n"
 get_revision_num="$juname -r"
 
 ## get the serial number
 ##### AIX serial num, can also use: 'lsattr -E -l sys0 | grep "systemid" | awk \'{print $2}\''
 get_serial_num_AIX="$juname -u"
 get_serial_num_HPUX="/usr/contrib/bin/machinfo | grep \"ine serial\" | awk '{print \$NF}'"
 get_serial_num_SunOS='hostid'
 get_serial_num_linux='dmi_lookup system-serial-number'	
 get_serial_num_generic='echo "serial number not found"'
}

set_AIX_defaults () {
 oslevel=`/usr/bin/oslevel`
 case $oslevel in
	[789]*)	nmonver=nmon12e_aix61 ;; ## Will actually use topas_nmon
	6*)    ## check for 6.1.2 ;; ## nmon ??
		ml=`/usr/bin/oslevel -r | cut -c 6,7`
		case $ml in
			02)		nmonver=nmon12f_aix612 ;;
			*)		nmonver=nmon12e_aix61 ;; ## for 6.1.1 and earlier
		esac
	    ;;
	5.3*)	## nmonver=nmon_aix53 ;; ##nmon11
		ml=`/usr/bin/oslevel -r | cut -c 6,7`
		case $ml in
			0[0123])	nmonver=nmon12e_aix530 ;;
			04)		nmonver=nmon12e_aix534 ;;
			05)		nmonver=nmon12e_aix535 ;;
			06)		nmonver=nmon12e_aix536 ;;
			*)		nmonver=nmon12e_aix537 ;; ## for 5.3.07 and anything later
		esac
		;;
	5.2*)	## nmonver=nmon_aix52 ;; ## bugs in various ML's mean different nmons needed
		ml=`/usr/bin/oslevel -r | cut -c 6,7`
		case $ml in
			0[56789])	nmonver=nmon12e_aix527 ;;
			0[012)		nmonver=nmon12e_aix522 ;;
			0[34])		nmonver=nmon12e_aix51 ;;
			*)		nmonver=nmon12e_aix527 ;;
		esac
		;;
	5.1*)	nmonver=nmon12e_aix51 ;;
	4.3.2*)	nmonver=nmon9f_aix432 ;;
	4.3*)	nmonver=nmon_aix433 ;;
	4.2*)	nmonver=nmon_aix420 ;;
	4.1*)	nmonver=nmon_aix415 ;;
	*)      echo "Unable to determine AIX version - installing nmon12e for AIX 6.1";
                echo "Please contact MPG Inc. for help determining proper nmon package." ;
		nmonver=nmon12e_aix61 ;;
 esac

 nmonmergever="nmonmerge_aix" ;
 reducepatt="reduce.patt.aix" ;
 pndcmrunning=$pndcmrunning_AIX ;
 nmonrunning=$nmonrunning_AIX ;
 defsardir=$defsardir_AIX ;
 sarq=$sarq_AIX ;
 sarrunning=$sarrunning_AIX ;
 sdcmrunning=$sdcmrunning_AIX ;
 pngetinforunning=$pngetinforunning_AIX ;
 rackrunning=$rackrunning_AIX ;
 sdcm_cron_entry=$sdcm_cron_entry_AIX ;
 pngetinfo_cron_entry=$pngetinfo_cron_entry_AIX ;
 cron_get=$cron_get_AIX ;
 crontab_file=$crontab_file_AIX ;
 cron_install=$cron_install_AIX ;
 get_machine_name=$get_machine_name_AIX ;
 get_serial_num=$get_serial_num_AIX ;
}

set_Linux_defaults () {
# get machine architecture (known: ix86 power s390 i386 i486 i586 i686 x86_64)
 osarch=`$juname -m`	# uname -p on AIX, ?? on linux-power, ix86 on pentium
#	if [ "$testplatform" = "T" ] ; then osarch="s390"; fi
##### ? if unknown achitecture install sar collector?
##  Next line is null if it actually matches the pattern.
 osarchx86=${osarch%%i?86}	# see if x86

 defsardir=$defsardir_linux ;
 sarq=$sarq_linux
 # os=`cat /etc/issue`
 os=`cat /etc/issue 2> /dev/null | awk '{printf $0}' | sed -e 's,\\\,/,g'`
 # os=`printf "%s\n" "$os"`
 os=`printf "%s" "$os"`
 nmonver=no_nmon_found

# ----- mainframe s390 architecture 32 bit -----
 if [ "$osarch" = "s390" ] ; then
    plat=mainframe_32
    nmonmergever="nmonmerge_mainframe_32" # defaults to a 32 bit version
    # pndmi is blank then nothing specified
    pndmi=""
    case $os in
	*Red\ Hat*release\ 4*)	nmonver=nmon_"$plat"_rhel4 ; 
                                nmonmergever="nmonmerge_mainframe_32_rhel4" ;;
	*Red\ Hat*release\ 5.5*)	nmonver=nmon_"$plat"_rhel55 ;
                                nmonmergever="nmonmerge_mainframe_32_rhel5" ;;
	*Red\ Hat*release\ 5*)	nmonver=nmon_"$plat"_rhel5 ;
                                nmonmergever="nmonmerge_mainframe_32_rhel5" ;;
##	*SUSE*9*)				nmonver=nmon_"$plat"_rhel4 ;; ## Needs recompile
##	*SUSE*10*)				nmonver=nmon_"$plat"_sles10 ;; ## Needs recompile
##	*SUSE*11*)				nmonver=nmon_"$plat"_sles11 ;; ## Needs recompile
	*)	nmonver=nmon_linux_"$plat" ;;   ## fallback
    esac
# ----- mainframe s390x architecture 64 bit -----
 elif [ "$osarch" = "s390x" ] ; then
    plat=mainframe_64
    nmonmergever="nmonmerge_mainframe_64" # defaults to a 64 bit version
    # pndmi is blank then nothing specified
    pndmi=""
    case $os in
	*Red\ Hat*release\ 4*)	nmonver=nmon_"$plat"_rhel4 ; 
                                nmonmergever="nmonmerge_mainframe_64_rhel4" ;;
	*Red\ Hat*release\ 5.5*)	nmonver=nmon_"$plat"_rhel55 ;
                                nmonmergever="nmonmerge_mainframe_64_rhel5" ;;
	*Red\ Hat*release\ 5*)	nmonver=nmon_"$plat"_rhel5 ;
                                nmonmergever="nmonmerge_mainframe_64_rhel5" ;;
##	*SUSE*9*)				nmonver=nmon_"$plat"_rhel4 ;; ## Needs recompile
	*SUSE*10*)				nmonver=nmon_"$plat"_sles10 ;; ## Needs recompile
	*SUSE*11*)				nmonver=nmon_"$plat"_sles11 ;; ## Needs recompile
	*)	nmonver=nmon_linux_"$plat" ;;   ## fallback nmon_linux_mainframe_64
    esac
# ----- x86_64 architecture -----
 elif [ "oo${osarch}oo" = "oox86_64oo" ] ; then	
    plat=x86_64
    nmonmergever="nmonmerge_x86_64" 
    pndmi="dmidecode_x86_64" 
    case $os in
	*Red\ Hat*release\ 4.5*)	nmonver=nmon_"$plat"_rhel45 ;;	#OK
	*Red\ Hat*release\ 4*)	nmonver=nmon_"$plat"_rhel4 ;;
	*Red\ Hat*release\ 5.4*)	nmonver=nmon_"$plat"_rhel54 ;;
	*Red\ Hat*release\ 5*)	nmonver=nmon_"$plat"_rhel5 ;;
	*Red\ Hat*release\ 6*)	nmonver=nmonmerge_"$plat"_rhel6 ;
				nmonver=nmon_"$plat"_rhel6 ;;
	*Mint\ 7*)	nmonver=nmon_"$plat"_mint7 ;;
	*Mint\ 8*)	nmonver=nmon_"$plat"_mint8 ;;
	*Mint\ 12*)	nmonver=nmon_"$plat"_mint12 ;;
	*Mint\ *)	nmonver=nmon_"$plat"_mint12 ;;
	*openSUSE*11*)	nmonver=nmon_"$plat"_opensuse11 ;;
	*openSUSE*12*)	nmonver=nmon_"$plat"_opensuse12 ;;
	*openSUSE*)	nmonver=nmon_"$plat"_opensuse12 ;;
	*Ubuntu*8.1*)	nmonver=nmon_"$plat"_ubuntu810 ;;
	*Ubuntu*9.[0-9]*)	nmonver=nmon_"$plat"_ubuntu910 ;;
	*Ubuntu*10.[0-9]*)	nmonver=nmon_"$plat"_ubuntu1004 ;;
	*Ubuntu*11.[0-9]*)	nmonver=nmon_"$plat"_ubuntu1104 ;;
	*CentOS*release\ 5*)	nmonver=nmon_"$plat"_centos5 ;;
	*CentOS*release\ 6*)	nmonver=nmon_"$plat"_centos6 ;;
	*Debian*5.[0-9]*)	nmonver=nmon_"$plat"_debian50 ;;
	*Debian*6.[0-9]*)	nmonver=nmon_"$plat"_debian60 ;;
	*Debian*)	nmonver=nmon_"$plat"_debian50 ;;
	*Fedora*release\ 10*)	nmonver=nmon_"$plat"_fedora10 ;;
	*Fedora*release\ 11*)	nmonver=nmon_"$plat"_fedora11 ;;
	*Fedora*release\ 12*)	nmonver=nmon_"$plat"_fedora12 ;;
	*Fedora*release\ 13*)	nmonver=nmon_"$plat"_fedora13 ;;
	*Fedora*release\ 14*)	nmonver=nmon_"$plat"_fedora14 ;;
	*Fedora*release\ 16*)	nmonver=nmon_"$plat"_fedora16 ;;
	*Fedora*release\ *)	nmonver=nmon_"$plat"_fedora16 ;;
#  	*SUSE*Server\ 8*)	nmonver=nmon_"$plat"_sles8 ;; # not tested (not in 11f)
#	*SUSE*Server\ 9*)	nmonver=nmon_"$plat"_sles9 ;; # need compile
	*SUSE*Server\ 10*)	nmonver=nmon_"$plat"_sles10 ;; 
	*SUSE*Server\ 11*)	nmonver=nmon_"$plat"_sles11 ;;
	*SUSE*Server\ 12*)	nmonver=nmon_"$plat"_opensuse12 ;;
	*)	nmonver=nmon_linux_"$plat" ;;   ## fallback to debian 50 if not available
    esac
 # ----- power 32 architecture -----
 elif [ "oo${osarch}oo" = "ooppcoo" ] || [ "oo${osarch}oo" = "ooppc64oo" ] ; then	
    # Even if the architecture is 64 bit, the kernel may be 32 bit
    getconf_long_bit=`getconf LONG_BIT`
    if [ "xx${getconf_long_bit}xx" = "xx32xx" ]; then 
        # ----- power 32 architecture -----
        plat=power_32
        nmonmergever="nmonmerge_linux_power_32" 
        pndmi="dmidecode_power_32" 
        case $os in
           *Red\ Hat*release\ 5*)	nmonver=nmon_"$plat"_rhel5 ;;
	   *Red\ Hat*release\ 4*)	nmonver=nmon_"$plat"_rhel4 ;;   #OK
	   *SUSE*9*)	nmonver=nmon_"$plat"_sles9 ;;   #OK
	   *SUSE*10*)	nmonver=nmon_"$plat"_sles10 ;;
	   *SUSE*11*)	nmonver=nmon_"$plat"_sles11 ;; #OK
	   *Ubuntu*)	nmonver=nmon_"$plat"_ubuntu1004 ;; #compiled on apple, but works on IBM power systems
	   *VCX*)		nmonver=nmon_"$plat"_rhel4 ;;  #3Com VCX is based on RetHat
           *)	nmonver=nmon_linux_"$plat" ;;   ## fallback
       esac
    else
       # ----- power 64 architecture -----
       plat=power_64
       nmonmergever="nmonmerge_linux_power_64" 
       pndmi="dmidecode_power_64" 
       case $os in
          *Red\ Hat*release\ 5*)	nmonver=nmon_"$plat"_rhel5 ;;
          *Red\ Hat*release\ 6.2*)	nmonver=nmon_"$plat"_rhel62 ;;
          *Red\ Hat*release\ 6*)	nmonver=nmon_"$plat"_rhel6 ;;
          *SUSE*11*)	nmonver=nmon_"$plat"_sles11 ;; #OK
          *)	nmonver=nmon_linux_"$plat" ;;   ## fallback to Redhat 5 copy under nmon_linux_power_64 name
       esac
    fi
# ----- itanium architecture -----
 elif [ "oo${osarch}oo" = "ooia64oo" ] ; then	
    plat=ia64
    nmonmergever="nmonmerge_ia64" 
    pndmi="dmidecode_ia64" 
    case $os in
	*Red\ Hat*release\ 52*)	nmonver=nmon_"$plat"_rhel4 ;;
	*Red\ Hat*release\ 44*)	nmonver=nmon_"$plat"_rhel4 ;; 
	*Red\ Hat*release\ 4*)	nmonver=nmon_"$plat"_rhel4 ;;   #OK
#	*SUSE*9*)	nmonver=nmon_"$plat"_sles9 ;;   #OK
	*SUSE*10*)	nmonver=nmon_"$plat"_sles10 ;;
	*SUSE*11*)	nmonver=nmon_"$plat"_sles10 ;; #OK
	*VCX*)		nmonver=nmon_"$plat"_rhel4 ;;  #3Com VCX is based on RedHat
	*)	nmonver=nmon_linux_"$plat" ;;   ## 4now always. OK RedHat and SuSe 
    esac
# ----- x86 architecture -----
 else	## must be ix86, i386, i486, i586, or i686 # 
    plat=x86
    nmonmergever="nmonmerge_linux" 
    pndmi="dmidecode_x86" 
    case $os in
	*Ubuntu*8.[1-9]*)	nmonver=nmon_"$plat"_ubuntu810 ;;
	*Ubuntu*9.[0-9]*)	nmonver=nmon_"$plat"_ubuntu910 ;;
	*Ubuntu*10.[0-9]*)	nmonver=nmon_"$plat"_ubuntu1004 ;;
	*Ubuntu*11.[0-9]*)	nmonver=nmon_"$plat"_ubuntu1104 ;;
	*Ubuntu*[0-7].*)	nmonver=nmon_"$plat"_knoppix5 ;
			sarq=$sarq_linux_debian ;
			defsardir=$defsardir_linux_debian ;;
	*Debian*5.[0-9]*)	nmonver=nmon_"$plat"_debian50 ; 
			sarq=$sarq_linux_debian ;
			defsardir=$defsardir_linux_debian ;;
	*Debian*6.[0-9]*)	nmonver=nmon_"$plat"_debian60 ; 
			sarq=$sarq_linux_debian ;
			defsardir=$defsardir_linux_debian ;;
	*Debian*)	nmonver=nmon_"$plat"_debian50 ; 
			sarq=$sarq_linux_debian ;
			defsardir=$defsardir_linux_debian ;;
	*Fedora\ Core*release\ 4*)	nmonver=nmon_"$plat"_fedoracore4 ;;  
	*Fedora*release\ 10*)	nmonver=nmon_"$plat"_fedora10 ;;  
	*Fedora*release\ 11*)	nmonver=nmon_"$plat"_fedora11 ;;  
	*Fedora*release\ 12*)	nmonver=nmon_"$plat"_fedora12 ;;  
	*Fedora*release\ 13*)	nmonver=nmon_"$plat"_fedora13 ;;  
	*Fedora*release\ 14*)	nmonver=nmon_"$plat"_fedora14 ;;  
	*Fedora*release\ 16*)	nmonver=nmon_"$plat"_fedora16 ;;  
	*Fedora*)	nmonver=nmon_"$plat"_fedora16 ;; # not tested
	*noppix*4*)	nmonver=nmon_"$plat"_knoppix4 ;;  
	*noppix*5*)	nmonver=nmon_"$plat"_knoppix5 ;;  
	*noppix*6*)	nmonver=nmon_"$plat"_knoppix6 ;;  
  	*Shrike*)	nmonver=nmon_"$plat"_redhat9 ;;	# OK (not in 11f)
	*Enigma*)	nmonver=nmon_linux_"$plat" ;;   # fallback to nmon10p   #OK
	*Red\ Hat*release\ 2*)	nmonver=nmon_"$plat"_rhel2 ;;
	*Red\ Hat*release\ 3*)	nmonver=nmon_"$plat"_rhel3 ;;
	*Red\ Hat*release\ 4*)	nmonver=nmon_"$plat"_rhel4 ;;	#OK
	*Red\ Hat*release\ 5*4*)	nmonver=nmon_"$plat"_rhel54 ;;
	*Red\ Hat*release\ 5*2*)	nmonver=nmon_"$plat"_rhel52 ;;
	*Red\ Hat*release\ 5*)	nmonver=nmon_"$plat"_rhel5 ;;
	*Red\ Hat*release\ 6*)	nmonver=nmon_"$plat"_rhel6 ;;
	*CentOS*release\ 4*)	nmonver=nmon_"$plat"_rhel4 ;;	#OK
	*CentOS*release\ 5*)	nmonver=nmon_"$plat"_centos5 ;;
	*CentOS*release\ 6*)	nmonver=nmon_"$plat"_centos6 ;;
	*Mint\ 7*)	nmonver=nmon_"$plat"_mint7 ;;
	*Mint\ 8*)	nmonver=nmon_"$plat"_mint8 ;;
	*Mint\ 12*)	nmonver=nmon_"$plat"_mint12 ;;
	*Mint\ *)	nmonver=nmon_"$plat"_mint12 ;;
	*openSUSE*10*)	nmonver=nmon_"$plat"_opensuse10 ;;  # Must be before other SUSE entries
	*openSUSE*11*)	nmonver=nmon_"$plat"_opensuse11 ;;  # Must be before other SUSE entries
	*openSUSE*12*)	nmonver=nmon_"$plat"_opensuse12 ;;  # Must be before other SUSE entries
	*SUSE*12*)	nmonver=nmon_linux_"$plat" ;;
	*SUSE*11*)	nmonver=nmon_"$plat"_opensuse11 ;;
	*SUSE*10*)	nmonver=nmon_linux_"$plat" ;;
	*SUSE*9*)	nmonver=nmon_linux_"$plat" ;;	#OK
  	*SUSE*8*)	nmonver=nmon_linux_"$plat" ;; # not tested (not in 11f)
	*)	nmonver=nmon_linux_"$plat" ;;	## fallback
    esac
 fi
	if [ "$debug" = "T" ] ; then
		echo "osarch=$osarch, osarchx86=$osarchx86" 
		echo "nmonver=$nmonver" 
	fi
 files_for_nmon="$files_for_nmon $pndme" # add nmon ,linux  
 files_for_sar="$files_for_sar " # add sar, linux
 reducepatt="reduce.patt.linux" ;
 pndcmrunning=$pndcmrunning_linux ;
 nmonrunning=$nmonrunning_linux ;
 sarrunning=$sarrunning_linux ;
 sdcmrunning=$sdcmrunning_linux ;
 pngetinforunning=$pngetinforunning_linux ;
 rackrunning=$rackrunning_linux ;
 sdcm_cron_entry=$sdcm_cron_entry_linux ;
 pngetinfo_cron_entry=$pngetinfo_cron_entry_linux ;
 cron_get=$cron_get_linux ;
 crontab_file=$crontab_file_linux ;
 cron_install=$cron_install_linux ;
 get_machine_name=$get_machine_name_linux ;
 get_serial_num="$get_serial_num_linux" ;
}

set_HPUX_defaults () {
   defmpgfile=mpgdata ;	# default for sar
   defmpgcurr=mpgdata_current ;	# default for sar
   install_sar="T";	## force sar install on HP-UX - no nmon available
   sar_cron_entry='0,5,10,15,20,25,30,35,40,45,50,55 * * * * /usr/lib/sa/sa1 &'
   if [ "$testplatform" = "T" ] ; then
      if [ "$install_sar" != "T" ] ; then
	 print "installing on HP-UX (must be a test install) allowing nmon install for testplatform";
      fi
   else 
      install_sar="T";	## force sar install on HP-UX - no nmon available
      echo "installing on HP-UX using sar (no nmon available on HP-UX)";
   fi
   if [ "$install_sar" = "T" ] ; then
      defsardir=$defsardir_generic ;
      sarq=$sarq_generic ;
      sarrunning=$sarrunning_generic ;
      sdcmrunning=$sdcmrunning_generic ;
      pngetinforunning=$pngetinforunning_generic ;
   else
      ## nmon may be available later...
      nmonver="nmon64" ;
      pndcmrunning=$pndcmrunning_generic ;
      nmonrunning=$nmonrunning_generic ;
      pngetinforunning=$pngetinforunning_generic ;
   fi
   rackrunning=$rackrunning_generic ;
   sdcm_cron_entry=$sdcm_cron_entry_generic ;
   pngetinfo_cron_entry=$pngetinfo_cron_entry_generic ;
   cron_get=$cron_get_generic ;
   crontab_file=$crontab_file_generic ;
   cron_install=$cron_install_generic ;
   get_machine_name=$get_machine_name_generic ;
   sarrunning=$sarrunning_HPUX ;
   get_serial_num=$get_serial_num_HPUX ;
}

set_solaris_defaults () {
   defmpgfile=mpgdata ;	# default for sar
   defmpgcur=mpgdata_current ;	# default for sar
   install_sar="T";	## force sar install on solaris - no nmon available
   sar_cron_entry='0,5,10,15,20,25,30,35,40,45,50,55 * * * * /usr/lib/sa/sa1 &'
   if [ "$testplatform" = "T" ] ; then
      if [ "$install_sar" != "T" ] ; then
         print "installing on Solaris (must be a test install) allowing nmon install for testplatform";
      fi
   else
      install_sar="T";	## force sar install on Solaris - no nmon available
      echo "installing on Solaris using sar (no nmon available on Solaris)";
   fi
   if [ "$install_sar" = "T" ] ; then
      defsardir=$defsardir_generic ;
      sarq=$sarq_generic ;
      sarrunning=$sarrunning_SunOS ;
      sdcmrunning=$sdcmrunning_generic ;
      pngetinforunning=$pngetinforunning_generic ;
   # else
      ## nmon may be available later...
      ## nmonver="nmon64" ;
      ## pndcmrunning=$pndcmrunning_generic ;
      ## nmonrunning=$nmonrunning_generic ;
      ## pngetinforunning=$pngetinforunning_generic ;
   fi
   rackrunning=$rackrunning_generic ;
   sdcm_cron_entry=$sdcm_cron_entry_generic ;
   pngetinfo_cron_entry=$pngetinfo_cron_entry_generic ;
   cron_get=$cron_get_generic ;
   crontab_file=$crontab_file_generic ;
   cron_install=$cron_install_generic ;
   get_machine_name=$get_machine_name_generic ;
   get_serial_num=$get_serial_num_SunOS ;
}

set_defaults () {
#determine OS and set OS-specific defaults
 set_default_variables
 ostype=`$juname -s`
#	if [ "$testplatform" = "T" ] ; then ostype="Linux"; fi
 case $ostype in
   "AIX")	set_AIX_defaults ;;
   "Linux")	set_Linux_defaults ;;
   "HP-UX")	set_HPUX_defaults ;;
   "SunOS")	set_solaris_defaults ;;
    *)
	echo "unknown OS type";
	if [ "$testplatform" != "T" ] ; then install_sar="T"; fi ##### allow nmon install for testing
	if [ "$install_sar" = "T" ] ; then
		echo " - using sar with mixed generic defaults";
	else
	  if [ "$testplatform" = "T" ] ; then
		nmonver="nmon64" ;
		pndcmrunning=$pndcmrunning_generic ;
		nmonrunning=$nmonrunning_generic ;
		pngetinforunning=$pngetinforunning_generic ;
	  else
		if [ "$force_install" = "T" ] ; then
		  nmonver="nmon_linux_x86"	## pick a basic default - tell the admin
		  echo "##########"
		  echo "  Forced install, but unable to determine nmon version to use:"
		  echo "  please extract and link the correct nmon version"
		  echo "  or use a SAR install (-S)"
		  echo "  note: defaulted to $nmonver which may not work on your platform"
		  echo "##########"
		else
		  echo "unable to determine nmon version to use" ; exit ;
		fi
	  fi
	fi
	defsardir=$defsardir_generic ;			# generic Unix ?
	sarq=$sarq_generic ;				# useless
	sarrunning=$sarrunning_generic ;		# assumes sar runs as user "adm"
	sdcmrunning=$sdcmrunning_generic ;		# checks root crontab
        pngetinforunning=$pngetinforunning_generic ;              # checks root crontab
	rackrunning=$rackrunning_generic ;		# checks root crontab
	sdcm_cron_entry=$sdcm_cron_entry_generic ;	# all sdcm cron entries same
        pngetinfo_cron_entry=$pngetinfo_cron_entry_generic ;      # all pngetinfo cron entries same
	cron_get=$cron_get_generic ;			# should work for most...
	crontab_file=$crontab_file_generic ;		# used for all now
	cron_install=$cron_install_generic ;		# should work if cron_get works...
	get_machine_name=$get_machine_name_generic ;	# used for all now
	get_serial_num=$get_serial_num_generic ;	# useless
	;;
 esac

 if [ "X$opt_sardir" = "X" ] ; then
	sardir=$defsardir;
 else
	sardir=$opt_sardir;
 fi
 
 if [ "X$opt_tarfile" = "X" ] ; then
	tarfile=$def_tarfile;
 else
	tarfile=$opt_tarfile;
 fi
 
 if [ "$install_sar" = "T" ] ; then
    if [ "$ostype" = "HP-UX" ] ; then
      files_to_untar="$files_for_all $files_for_sar $files_for_hpux"
    elif [ "$ostype" = "AIX" ] ; then
      files_to_untar="$files_for_all $files_for_sar $files_for_aix "
    else
      files_to_untar="$files_for_all $files_for_sar"
    fi
 else
   if [ -s /bin/nmon ] ; then
      grep topas /bin/nmon > /dev/null
      jtopas=$?
      if [ $jtopas -eq 0 ]; then
         if [ "$ostype" = "AIX" ] ; then
            files_to_untar="$files_for_all $files_for_nmon $pndmi $nmonmergever $reducepatt $files_for_aix "
         else
            files_to_untar="$files_for_all $files_for_nmon $pndmi $nmonmergever $reducepatt"
         fi
         nmonver='/bin/nmon'
      else
         if [ "$ostype" = "AIX" ] ; then
            files_to_untar="$files_for_all $files_for_nmon $pndmi $nmonver $nmonmergever $reducepatt $files_for_aix "
         else
            files_to_untar="$files_for_all $files_for_nmon $pndmi $nmonver $nmonmergever $reducepatt "
         fi
      fi
   else
      if [ "$ostype" = "AIX" ] ; then
         files_to_untar="$files_for_all $files_for_nmon $pndmi $nmonver $nmonmergever $reducepatt $files_for_aix "
      else
         files_to_untar="$files_for_all $files_for_nmon $pndmi $nmonver $nmonmergever $reducepatt"
      fi
   fi
 fi

 ## where will we store mpg datafile?
 if [ "X$opt_datafile" = "X" ] ; then
	defmpgdata=$instdir/$defmpgfile
	defmpgcur=$instdir/$defmpgcur
 else
	defmpgdata=$opt_datafile
	defmpgcur=$opt_datacur
 fi
 sysname=`eval $get_machine_name`
	if [ ! -z "$debug" ] ; then echo "sysname=$sysname" ; fi
 defmpgdata="${defmpgdata}.$sysname"
 defmpgcur="${defmpgcur}.$sysname"
 mpgdata=$defmpgdata ;
 mpgcur=$defmpgcur ;
 if [ ! -z "$verbose" ] ; then
	echo "datafile = $mpgdata"
	echo "datacur = $mpgcur"
 fi

 if [ "$debug" = "T" ] ; then
 ## show ostype and derived values
	echo "ostype = $ostype" ;
   if [ "$install_sar" = "T" ] ; then
	echo "sardir = $sardir" ;
	echo "sarq = $sarq" ;
	echo "sarrunning = $sarrunning" ;
	echo "sdcmrunning = $sdcmrunning" ;
	echo "sar_cron_entry = $sar_cron_entry"
	echo "sdcm_cron_entry = $sdcm_cron_entry" ;
   else
	echo "nmonver = $nmonver" ;
	echo "nmonmergever = $nmonmergever" ;
	echo "reducepatt = $reducepatt" ;
	echo "pndcmrunning = $pndcmrunning" ;
	echo "nmonrunning = $nmonrunning" ;
	echo "nmon_cron_entry = $nmon_cron_entry" ;
	echo "pndcm_cron_entry = $pndcm_cron_entry" ;
   fi
   echo "pngetinforunning = $pngetinforunning" ;
   echo "pngetinfo_cron_entry = $pngetinfo_cron_entry" ;
   echo "rackrunning = $rackrunning" ;
   echo "rack_cron_entry = $rack_cron_entry" ;
   echo "cron_get = $cron_get" ;
   echo "crontab_file = $crontab_file" ;
   echo "cron_install = $cron_install" ;
   echo "instdir = $instdir" ;
   echo "tarfile = $tarfile" ;
   echo "configfile = $pnconfig" ;
   echo "files_to_untar=$files_to_untar" ;
 fi
}

fixshells () {
   ## change the #!... line in .sh files for SunOS (or others that need it)
   cd $instdir
   hasbash=`which bash`
   hasksh=`which ksh`
   if [ ! -z "$opt_shelltouse" ] ; then
      shelltouse=$opt_shelltouse ;
      echo "  using shell: $shelltouse  (from commandline) "
   else
      if [ -z "$hasbash" ] ; then
         # no bash, try ksh
         if [ -z "$hasksh" ] ; then
            #cannot find any usable shell - punt unless forced
            echo "cannot find an appropriate shell"
            if [ "$force_install" = "T" ] ; then
	       useksh="TRUE"
	       shelltouse="/usr/bin/ksh"
	    echo " ----- Forced install, defaulted to /usr/bin/ksh for scripts "
	    echo " -----  if this fails, reinstall with the  -H shellpath  option"
         else
	    exit 1
         fi
      else
         useksh="TRUE"
         shelltouse=$hasksh
      fi
   else
      usebash="TRUE"
      shelltouse=$hasbash
   fi
fi
 
for i in *.sh
do
   cp $i fixshell.tmp
   cat fixshell.tmp | sed -e "1d" > fixshell.noshell
   echo "#!$shelltouse" > $i
   cat fixshell.noshell >> $i
   rm -f fixshell.noshell fixshell.tmp
done
cd $startdir 
}

write_config_file () {
   nmondir=$instdir
   sc=${1:-"unknown"}	# should be called with "success" or a failure message
   echo "installation disposition: $sc"
   if [ "$debug" = "T" ] ; then echo "current working dir: $PWD" ; fi
   if [ "$debug" = "T" ] ; then echo "writing config file: $pnconfig" ; fi
   instdate=`date`
   serial_num=`eval $get_serial_num`
   revision_num=`$get_revision_num`
   ##### could just skip these line on reinstall
   echo "$copyright" > $pnconfig
   echo "# MPG Power Navigator data collector configuration" >> $pnconfig
   echo "# $install_sh_ver" >> $pnconfig
   ##### end reinstall skip
   echo "# Power Navigator installer version $installer_version" >> $pnconfig
   echo "# install date: $instdate" >> $pnconfig
   echo "# installation for: $sysname" >> $pnconfig
   if [ "x${ostype}x" != "xx" ] ; then
      echo "# ostype:           $ostype" >> $pnconfig
   fi
   if [ "x${oslevel}x" != "xx" ] ; then
      echo "# oslevel:          $oslevel" >> $pnconfig
   fi
   if [ "x${os}x" != "xx" ] ; then
      echo "# os:               $os" >> $pnconfig
   fi
   if [ "x${plat}x" != "xx" ] ; then
      echo "# plat:             $plat" >> $pnconfig
   fi
   if [ "x${nmonver}x" != "xx" ] ; then
      echo "# nmonver:          $nmonver" >> $pnconfig
   fi
   if [ "$serial_num" != "serial number not found" ] ; then
      echo "#	serial number: $serial_num" >> $pnconfig
   fi
   echo "#     revision number: $revision_num" >> $pnconfig
   echo "# install disposition: $sc" >> $pnconfig
   if [ "$install_sar" = "T" ] ; then
      echo '#' >> $pnconfig
      echo '# admin_data_collect variable is used to turn off/on admin data collection' >> $pnconfig
      echo '# when rackdata.sh runs. ' >> $pnconfig
      echo '# admin_data_collect="F" # turn off the collection.' >> $pnconfig
      echo '# admin_data_collect="T" # turn on the collection.' >> $pnconfig
      echo 'admin_data_collect="F"' >> $pnconfig
      echo '# ' >> $pnconfig
      echo '# Variable truncate_days is used by pndchk_sar.sh to determine how many days' >> $pnconfig
      echo '# should be retained. Data prior to that period is appended into:' >> $pnconfig
      echo '# mpgdata.hostname_archive.gz - An empty truncate_days defaults to 366.' >> $pnconfig
      echo '# If a value is specified less than 30 days, pndchk.sh will not truncate.' >> $pnconfig
      echo 'truncate_days=""' >> $pnconfig
      echo '#' >> $pnconfig
      if [ "$sardir" != "$defsardir" ] ; then
         echo "sardir=$sardir" >> $pnconfig
      fi
   else
      nmonverstring="# using $nmonver"
      nmon_pkg_ver=`grep $nmonver manifest | awk -F= '{print $1}'`
      if [ ! -z "$nmon_pkg_ver" ] ; then
         nmonverstring="$nmonverstring from $nmon_pkg_ver package"
      fi
      echo $nmonverstring >> $pnconfig
      echo "nmondir=$nmondir" >> $pnconfig
      echo '# Use variable nmonopts to add additional options to the start' >> $pnconfig
      echo '# of nmon (i.e. -t gathers top on each snap if greater than 10%)' >> $pnconfig
      echo '#         (     -t -I 0.02 top on each snap if greater than 2%)' >> $pnconfig
      echo '#         (     -d includes disk service times on each snap)' >> $pnconfig
      echo '#         (     "nmon -help" for other options)' >> $pnconfig
      echo 'nmonopts=""' >> $pnconfig
      echo '#' >> $pnconfig
      echo '# ext_data_collect used to turn off/on external data collection.' >> $pnconfig
      echo '# ext_data_collect="F" off the next day, or when nmon restarts.' >> $pnconfig
      echo '# ext_data_collect="T" on the next day, or when nmon restarts.' >> $pnconfig
      echo 'ext_data_collect="F"' >> $pnconfig
      echo '#' >> $pnconfig
      echo '# admin_data_collect variable is used to turn off/on admin data collection' >> $pnconfig
      echo '# when rackdata.sh runs.'   >> $pnconfig
      echo '# admin_data_collect="F" # turn off the collection.' >> $pnconfig
      echo '# admin_data_collect="T" # turn on the collection.' >> $pnconfig
      echo 'admin_data_collect="F"' >> $pnconfig
      echo '# ' >> $pnconfig
      echo '# transfer_script_cmd is the command line for transferring nmon data' >> $pnconfig
      echo '# to an alternate location using the following files:' >> $pnconfig
      echo '# mpgdata2.hostname mpgdata2_current.hostname rackdata.hostname' >> $pnconfig
      echo '# An example: "/usr/local/mpg/transfer_script.sh -c /usr/local/mpg/transfer_cp.config"' >> $pnconfig
      echo 'transfer_script_cmd=""' >> $pnconfig
      echo '#' >> $pnconfig
      echo '# transfer_current_cmd is the command line for transferring nmon update' >> $pnconfig
      echo '# data to an alternate location using the following files:' >> $pnconfig
      echo '# mpgdata2_current.hostname' >> $pnconfig
      echo '# An example: "/usr/local/mpg/transfer_current.sh -c /usr/local/mpg/transfer_cp.config"' >> $pnconfig
      echo 'transfer_current_cmd=""' >> $pnconfig
      echo '#' >> $pnconfig
      echo '# Variable truncate_days is used by pndchk.sh to determine how many days' >> $pnconfig
      echo '# should be retained. Data prior to that period is appended into:' >> $pnconfig
      echo '# mpgdata2.hostname_archive.gz - An empty truncate_days defaults to 366.' >> $pnconfig
      echo '# If a value is specified less than 30 days, pndchk.sh will not truncate.' >> $pnconfig
      echo 'truncate_days=""' >> $pnconfig
      echo '#' >> $pnconfig
      echo '# Variable nmondaystokeep is used by pndcm.sh to determine how many nmon daily' >> $pnconfig
      echo '# data files to retain.  Minimum value is 7 days.  This can be set higher.' >> $pnconfig
      echo 'nmondaystokeep="7"' >> $pnconfig
      echo '#' >> $pnconfig
      echo '# compressdatafile is used to turn off making a compressed copy of the nmon' >> $pnconfig
      echo '# data file. If it is set to "nocopy" there will not be a copy made.' >> $pnconfig
      echo '# There is no need for the compressed copy, pngetinfo.sh gathers same info.' >> $pnconfig
      echo 'compressdatafile="nocopy"' >> $pnconfig
      echo '#' >> $pnconfig

      ## always need nmondir for rackdata
      ## if [ "$instdir" != "$defdir" ] ; then
      ##   fi
   fi
   # Create an mpgdata or mpgdata2 data file if it does not exist on install
   # depending on nmon or sar 
   if [ ! -s $mpgdata ] ; then
      touch $mpgdata
   fi
   echo "datafile=$mpgdata" >> $pnconfig
   echo "datacur=$mpgcur" >> $pnconfig
   echo "sysname=$sysname" >> $pnconfig
   ## add rackfile var for transfer_script
   echo "rackfile=$instdir/rackdata.$sysname" >> $pnconfig
   if [ ! -z "$reduce" ] ; then
      echo "reduce=F" >> $pnconfig
   fi
   if [ ! -z "$compressnmon" ] ; then
      echo "compressnmonfiles=none" >> $pnconfig
   fi
   if [ ! -z "$compressdatafile" ] ; then
      echo "compressdatafile=none" >> $pnconfig
   fi
}

add_to_etc_conf () {
  if [ -f /etc/pn.conf ] ; then
     cp -p /etc/pn.conf /etc/pn.conf.${jdate}
     cp -p /etc/pn.conf /etc/pn.conf.org
  else
     touch /etc/pn.conf
  fi
  grep $instdir /etc/pn.conf
  inconf=$?
  if [ "$inconf" = "0" ] ; then
     echo "found an instance already in /etc/pn.conf ???"
  else
     echo "nminst=$instdir" >> /etc/pn.conf
  fi
}

install_inits () {
 if [ -d /etc/init.d ] ; then
   initbasedir="/etc"
 elif [ -d /etc/rc.d/init.d ] ; then
   initbasedir="/etc/rc.d" 
 fi
 cp pnndc.rc $initbasedir/init.d/pnndc
##### set rcdirs (aix: rc2.d only?, linux how to determine which dirs to set up?)
 rlstring=`who -r 2>/dev/null | grep run-level`
 rlknown=$?
 if [ "$rlknown" != "0" ] ; then
	echo "runlevel undetermined - please create init script links"
	echo "init script is in $initbasedir/init.d/pnndc"
 else
   if [ "$ostype" = "AIX" ] ; then
     currl=`who -r | cut -c23 2>/dev/null`
     krl=$currl		# whatever...
   else
     currl=`who -r | cut -c20 2>/dev/null`
     krl="0"
   fi
#   currlok=${currl/[235]/OK}
#   if [ "$currlok" = "OK" ] ; then
   currlok=${currl%%[235]}
   if [ "$currlok" = "" ] ; then
     echo "creating links to start PowerNavigator data collector at runlevel $currl"
     ln -s $initbasedir/init.d/pnndc $initbasedir/rc${currl}.d/S99pnndc
     ln -s $initbasedir/init.d/pnndc $initbasedir/rc${krl}.d/K01pnndc
   else
	echo "currl=$currl, rlstring=$rlstring"
	echo "runlevel undetermined - please create init script links"
	echo "init script is in $initbasedir/init.d/pnndc"
   fi
 fi
}

incron_message () {
 found_message=$1;
 existing_entry=${2:-"none"};
 rc_prog=${3:-"none"};
 example_entry=${4:-"none"};
   echo
   echo "##########"
   echo "$found_message"
   if [ "$existing_entry" != "none" ] ; then
	echo "existing crontab entry:"
	echo "$existing_entry"
   fi
   if [ "$rc_prog" != "none" ] ; then
	echo "----- instruct admin to re-configure $rc_prog in crontab if needed -----"
   fi
   if [ "$example_entry" != "none" ] ; then
	echo "example:"
	echo "$example_entry"
   fi
   echo "##########"
   echo
}

create_install_dir () {
## make $instdir and parents if needed
 if [ ! -d "$instdir" ] ; then
	if [ "$debug" = "T" ] ; then echo "need to create $instdir" ; fi
   id=$instdir
   tomake="$id"
   idp=`dirname $id`
   while [ "$idp" != "$id" ] ; do
     if [ ! -d "$idp" ] ; then
	if [ "$debug" = "T" ] ; then echo "need to create $idp" ; fi
	tomake="$idp $tomake"
	id=$idp
	idp=`dirname $id`
     else
	id=$idp
     fi
   done
   for dir in $tomake
   do
     mkdir $dir
     retcode=$?
     if [ "$retcode" != "0" ] ; then
	echo "ERROR ($retcode) unable to create $dir - aborting"
	exit 1
     fi
     chmod 755 $dir
     retcode=$?
     if [ "$retcode" != "0" ] ; then
	echo "ERROR ($retcode) unable to set permissions on $dir - aborting"
	exit 1
     fi
   done
 fi
 chmod 755 $instdir
}

untar_powernav_pkg () {
## untar the powernavigator package
	if [ ! -z "$verbose" ] ; then echo "copying files to $instdir" ; fi
 cp $tarfile $instdir
 retcode=$?
	if [ "$debug" = "T" ] ; then echo "retcode from cp : $retcode" ; fi
 if [ "$retcode" != "0" ] ; then
   echo "ERROR ($retcode) trying to copy files to $instdir - aborting" ;
   exit 1;
 fi
 cd $instdir
 ps -ef | grep "${instdir}/nmon" | grep -v grep | tee /tmp/nmon_running.tmp
 if [ -s /tmp/nmon_running.tmp ] ; then
    echo "*** nmon process still running ***"
    echo "You may need to run the following commmand:"
    cat /tmp/nmon_running.tmp | awk '{print "# kill ",$2 }'
 fi
 if [ ! -z "$verbose" ] ; then
   tar xvf $tarfile $files_to_untar
 else
   tar xvf $tarfile $files_to_untar > /dev/null 2>&1
 fi
 retcode=$?
 # Ensure .sh files are set to execute for root.
 chmod 740 *.sh
 # Ensure nmon_* files are set to execute for root.
 if [ -s /usr/bin/topas_nmon ]; then
   echo "### Not installing AIX nmon files, will be using /usr/bin/topas_nmon ### "
 fi
 if [ -s nmon_* ] ; then
   # Added in the 2>&1 so sun and hp will not display no nmon exists
   chmod 740 nmon_* 2>&1 > /dev/null
 fi
 if [ "$debug" = "T" ] ; then echo "retcode from tar : $retcode" ; fi
 if [ "$retcode" != "0" ] ; then
   echo "ERROR ($retcode) trying to untar files in $instdir - aborting" ;
   exit 1;
 fi
}

check_sar_installed () {
## is sar installed?
  if [ ! -z "$verbose" ] ; then
    eval $sarq
  else
    eval $sarq > /dev/null
  fi
  if [ "$?" != "0" ] ; then
    if [ "X$opt_sardir" != "X" ] ; then
	echo
	echo "##########"
	echo "sar package not found, however sardir set to non-default location $sardir"
	echo "continuing as though sar were installed!"
	echo "##########"
	sc="unknown, sar in non-default location?"
    else
	echo "apparently sar is not installed"
	echo
	echo "##########"
	echo "----- instruct admin to install sar package and configure it to run -----"
	echo "##########"
	echo
	write_config_file "failed - sar not installed"
	exit 1
    fi
  else
    if [ ! -z "$verbose" ] ; then
	echo "found sar installed"
    fi
  fi
}

check_sar_in_crontab () {
## is sar configured and running?
  sar_in_cron="`$sarrunning`"
  # Detection of existing sa1 entries 
  if [ "x${sar_in_cron}x" = "xx" ]; then
     found_sar=1
  else
     found_sar=0
  fi    
  if [ "$debug" = "T" ] ; then
        echo "found_sar=$found_sar";
        echo "sar_in_cron=$sar_in_cron";
  fi
  if [ "$found_sar" != "0" ] ; then
    if [ "X$opt_sardir" != "X" ] ; then
	echo
	echo "##########"
	echo "sar configuration not found, however sardir set to non-default location $sardir"
	echo "continuing as though sar were configured!"
	echo "example sar crontab entry: "
	echo "$sar_cron_entry"
	if [ "$ostype" = "AIX" ] ; then
	  echo "NOTE: please verify that sar is configured in the adm crontab"
	else
	  echo "NOTE: please verify that sar is configured in cron"
	fi
	echo "##########"
	sc="$sc - unknown sar, not configured or in non-default location?"
    else
	# echo
	if [ ! -z "$verbose" ] ; then
		echo "apparently sar is not configured - will add sar entry to cron" ;
	fi
	echo
	if [ "$ostype" = "AIX" ] ; then
	  add_sar_to_AIX_cron="T";  add_header_to_cron="T";
	else
	  add_sar_to_cron="T";  add_header_to_cron="T";
	fi
    fi
  else
    #if [ ! -z "$verbose" ] ; then
	m1="Found an existing crontab entry for sar"
	incron_message "$m1" "$sar_in_cron" "sar" "$sar_cron_entry"
    #fi
  fi
}

#####  extra check - not finished - probably not needed
## where are sar files?
# ? are the files in the default sar dir? or the dir specified on commandline (opt_sardir)
##	??? use the default search pattern ???
#	ls -al $defsardir/*

check_devnull () {
   ## Confirm permission on /dev/null are 666 or 777
   ## That is read write for owner group everyone
   find /dev/null -perm 666 > /tmp/devnull_perm_check.tmp
   find /dev/null -perm 777 >> /tmp/devnull_perm_check.tmp
   if [ ! -s /tmp/devnull_perm_check.tmp ] ; then
      ls -al /dev/null
      echo "Please correct the permissions of /dev/null"
      echo "Permissions should be 666 or 777; the syntax to correct this is on the next line:"
      echo ""
      echo "#    chmod 666 /dev/null"
      echo "#    or"
      echo "#    chmod 777 /dev/null"
      echo ""
      echo "Exiting, please correct the permissions of /dev/null."
      echo "Then run the install.sh script again."
      exit 1
   fi
}

check_sdcm_in_crontab () {
## add sdcm to crontab if needed
  sdcm_in_cron=`eval $sdcmrunning 2>&1`
  found_sdcm=$?
  if [ "$debug" = "T" ] ; then
	echo "found_sdcm=$found_sdcm";
	echo "sdcm_in_cron=$sdcm_in_cron";
  fi
  if [ "$found_sdcm" != "0" ] ; then
	# echo
	if [ ! -z "$verbose" ] ; then
		echo 'cannot find data collector (sdcm.sh) configured in crontab' ;
	fi
		echo "will add Power Navigator Data Collector for sar (sdcm) to cron" ;
	echo
	add_sdcm_to_cron="T"; add_header_to_cron="T"; add_pndchk_sar_to_cron="T";
  else
	m1='Found an existing crontab entry for Power Navigator data collector (sdcm.sh)'
	incron_message "$m1" "$sdcm_in_cron" "sdcm.sh" "$sdcm_cron_entry"
  fi
}

check_pngetinfo_in_crontab () {
## add pngetinfo to crontab if needed
  pngetinfo_in_cron=`eval $pngetinforunning 2>&1`
  found_pngetinfo=$?

  if [ "$debug" = "T" ] ; then
        echo "found_pngetinfo=$found_pngetinfo";
        echo "pngetinfo_in_cron=$pngetinfo_in_cron";
  fi
  if [ "$found_pngetinfo" != "0" ] ; then
	# echo
	if [ ! -z "$verbose" ] ; then
           echo 'cannot find data collector (pngetinfo.sh) configured in crontab' ;
	fi
		echo "will add pngetinfo to cron" ;
	echo
        add_pngetinfo_to_cron="T";  add_header_to_cron="T";
  else
        m1='Found an existing crontab entry for Power Navigator data collector (pngetinfo.sh)'
        incron_message "$m1" "$pngetinfo_in_cron" "pngetinfo.sh" "$pngetinfo_cron_entry"
  fi
}

check_nmon_in_crontab () {
## add nmon to crontab if needed
  nmon_in_cron=`eval $nmonrunning 2>&1`
  found_nmon=$?
  if [ "$debug" = "T" ] ; then
	echo "found_nmon=$found_nmon";
	echo "nmon_in_cron=$nmon_in_cron";
  fi
  if [ "$found_nmon" != "0" ] ; then
	# echo
	if [ ! -z "$verbose" ] ; then
		echo "apparently nmon is not configured - will add nmon entry to cron" ;
	fi
	echo
	add_nmon_to_cron="T" ; add_header_to_cron="T";
  else
    if [ ! -z "$verbose" ] ; then
	m1="Found an existing crontab entry for nmon"
	incron_message "$m1" "$nmon_in_cron" "nmon" "$nmon_cron_entry"
    fi
  fi
}

check_pndcm_in_crontab () {
## add pndcm to crontab if needed
  pndcm_in_cron=`eval $pndcmrunning 2>&1`
  found_pndcm=$?
  if [ "$debug" = "T" ] ; then
	echo "found_pndcm=$found_pndcm";
	echo "pndcm_in_cron=$pndcm_in_cron";
  fi
  if [ "$found_pndcm" != "0" ] ; then
	# echo
	if [ ! -z "$verbose" ] ; then
		echo 'cannot find data collector (pndcm.sh) configured in crontab' ;
	fi
	echo "will add Power Navigator Data Collector (pndcm.sh) to cron" ;
	echo
	add_pndcm_to_cron="T" ; add_header_to_cron="T";
	add_pndcm_clean_to_cron="T" ;
	add_pndchk_to_cron="T" ;
 	pndchk_cron_entry="$pndchk_cron_entry"
  else
	m1='Found an existing crontab entry for Power Navigator data collector (pndcm.sh)'
	incron_message "$m1" "$pndcm_in_cron" "pndcm.sh" "$pndcm_cron_entry"
  fi
}

create_nmon_symlinks () {
  cd $instdir	# should already have been done...
  if [ "$debug" = "T" ] ; then echo "creating symbolic link $nmonver -> nmon" ; fi
  ln -s $nmonver nmon
  if [ "$debug" = "T" ] ; then echo "creating symbolic link $nmonmergever -> nmonmerge" ; fi
  if  [ "oo${ostype}oo" = "ooLinuxoo" ] && [ "oo${pndmi}oo" != "oooo" ] ; then
     ln -s $pndmi pntestdmi 
     if [ "$debug" = "T" ] ; then echo "creating symbolic link $pndmi -> pntestdmi" ; fi
  fi
  ln -s $nmonmergever nmonmerge
  if [ "$debug" = "T" ] ; then echo "creating symbolic link $reducepatt -> reduce.patt" ; fi
  ln -s $reducepatt reduce.patt
}

check_rackdata_in_crontab () {
## check if rackdata is in crontab
 rack_in_cron=`eval $rackrunning 2>&1`
 found_rack=$?
 if [ "$debug" = "T" ] ; then
	echo "found_rack=$found_rack";
	echo "rack_in_cron=$rack_in_cron";
 fi
 if [ "$found_rack" != "0" ] ; then
	if [ ! -z "$verbose" ] ; then echo "will add rack config entry to crontab" ; fi
	echo
	add_rack_to_cron="T" ;  add_header_to_cron="T";
 else
	m1='Found an existing crontab entry for rack config'
	incron_message "$m1" "$rack_in_cron" "rackdata.sh" "$rack_cron_entry"
 fi
}

force_testplatform_settings () {
##### this section only for testing on unsupported systems #####
 # force attempt to add cron entries - so they will be printed...
 if [ "$testplatform" = "T" ] ; then
	if [ "$install_sar" = "T" ] ; then
		add_sar_to_cron="T";  add_header_to_cron="T";
	else
		add_nmon_to_cron="T";  add_header_to_cron="T"; 
		add_pndcm_to_cron="T";  add_header_to_cron="T"; 
		add_pndcm_clean_to_cron="T" ;
	fi
	add_rack_to_cron="T";  add_header_to_cron="T"
	add_pngetinfo_to_cron="T";
 fi	##### end testplatform section
}

create_crontab_entries () {
   ## create crontab entries as needed
   if [ "$add_header_to_cron" = "T" ] ; then
      echo "creating Power Navigator crontab entries"
      ##### debian specific
      CRONTAB_NOHEADER="N"
      export CRONTAB_NOHEADER
      ##### end debian specific
      eval $cron_get
      cp $crontab_file ${crontab_file}.save
      # Placed add sar to cron above the header intentionally.
      # Otherwise, it will change position on subsequent installs.
      if [ "$add_sar_to_cron" = "T" ] ; then echo "$sar_cron_entry" >> $crontab_file ; fi
      echo $cron_header >> $crontab_file ;
      if [ "$add_sdcm_to_cron" = "T" ] ; then echo "$sdcm_cron_entry" >> $crontab_file ; fi
      if [ "$add_nmon_to_cron" = "T" ] ; then echo "$nmon_cron_entry" >> $crontab_file ; fi
      if [ "$add_pndcm_to_cron" = "T" ] ; then echo "$pndcm_cron_entry" >> $crontab_file ; fi
      if [ "$add_pndcm_clean_to_cron" = "T" ] ; then echo "$pndcm_clean_cron_entry" >> $crontab_file ; fi
      if [ "$add_rack_to_cron" = "T" ] ; then echo "$rack_cron_entry" >> $crontab_file ; fi
      if [ "$add_pndchk_to_cron" = "T" ] ; then echo "$pndchk_cron_entry" >> $crontab_file ; fi
      if [ "$add_pndchk_sar_to_cron" = "T" ] ; then echo "$pndchk_sar_cron_entry" >> $crontab_file ; fi
      if [ "$add_pngetinfo_to_cron" = "T" ] ; then echo "$pngetinfo_cron_entry" >> $crontab_file ; fi
      # echo $cron_trailer >> $crontab_file ;
      if [ "$testplatform" = "T" ] ; then cron_install="false" ; fi ##### remove for production !!!
      eval $cron_install
      sic=$?
      if [ "$debug" = "T" ] ; then echo "add to cron: $sic" ; fi
      if [ "$sic" != "0" ] ; then
         sc="failed - unable to create cron entries; $sc"
         echo
         echo "##########"
         echo "unable to create crontab entries -- please have admin set them up"
         echo " desired crontab entry:"
         echo
         echo $cron_header ;
         if [ "$add_sar_to_cron" = "T" ] ; then echo "$sar_cron_entry" ; fi
         if [ "$add_sdcm_to_cron" = "T" ] ; then echo "$sdcm_cron_entry" ; fi
         if [ "$add_nmon_to_cron" = "T" ] ; then echo "$nmon_cron_entry" ; fi
         if [ "$add_pndcm_to_cron" = "T" ] ; then echo "$pndcm_cron_entry" ; fi
         if [ "$add_pndcm_clean_to_cron" = "T" ] ; then echo "$pndcm_clean_cron_entry" ; fi
         if [ "$add_rack_to_cron" = "T" ] ; then echo "$rack_cron_entry" ; fi
         if [ "$add_pndchk_to_cron" = "T" ] ; then echo "$pndchk_cron_entry" ; fi
         if [ "$add_pndchk_sar_to_cron" = "T" ] ; then echo "$pndchk_sar_cron_entry" ; fi
         if [ "$add_pngetinfo_to_cron" = "T" ] ; then echo "$pngetinfo_cron_entry" ; fi
         echo $cron_trailer ;
         echo "##########"
      fi
      if [ "$add_sar_to_AIX_cron" = "T" ] ; then
	  crontab -l adm > crontab.adm
	  cp crontab.adm crontab.adm.save
	  echo $cron_header >> crontab.adm
	  echo "$sar_cron_entry" >> crontab.adm
	  echo $cron_trailer >> crontab.adm
	  su adm -c "crontab crontab.adm"
	  sica=$?
	  if [ "$sica" != "0" ] ; then
	     sc="failed - unable to create adm cron entry for sar; $sc"
	     echo "##########"
	     echo "Unable to create crontab entry for sar"
	     echo "Please add the following to the adm crontab:"
	     echo "$sar_cron_entry"
	     echo "##########"
	  fi
      fi
   else
      echo "nothing to add to cron";
   fi
}

do_os_specific_setup () {
## do remaining os-specific setup stuff
 if [ "$ostype" = "AIX" ] ; then
   fileowner="root:system" ;
   oslevel=`/usr/bin/oslevel`
   if [[ "$oslevel" = 4.* ]] ; then
     chmod ugo+r /dev/kmem ;	## only needed for nmon 9 on AIX 4.x
   fi
   chdev -l sys0 -a iostat=true ;
 elif [ "$ostype" = "Linux" ] ; then
   fileowner="root:root" ;
 elif [ "$ostype" = "HP-UX" ] ; then
   fileowner="root:root" ;
 fi
}

set_file_ownerships () {
## set ownership of installed files
# NOTE: directory permissions set when created (above)
#	file permissions set by the untar process
   if [ "X$fileowner" != "X" ] ; then
      if [ "$testplatform" != "T" ] ; then	# do not set file perms on testplatform
         if [ ! -z "$verbose" ] ; then
            echo "setting file and directory ownerships and permissions" ;
         fi
         chown -R $fileowner $instdir
         if [ "$install_sar" != "T" ] ; then
            chown -h $fileowner $instdir/nmon	## chown the link, not the target
         fi
      fi  # testplatform fi
   fi
}

save_rack_config () {
## save the rack configuration
 if [ ! -z "$verbose" ] ; then echo "saving rack configuration" ; fi
 ${instdir}/rackdata.sh -c $pnconfig
 rd=$?
 if [ "$rd" != "0" ] ; then
	sc="rackdata failed; $sc"
	echo
	echo "rackdata failed - please verify installation"
	echo
 fi
}

check_reinstall_needed () {
 if [ -z "$reinstall" ] ; then
	if [ ! -z "$verbose" ] ; then echo "----- reinstall not set on commandline" ; fi
   if [ -d "$instdir" ] ; then
	if [ ! -z "$verbose" ] ; then echo "----- $instdir exists" ; fi
     if [ -s "$pnconfig" ] || [ -s "$mpgdata" ] || [ -s "pndcm.sh" ] || [ -s "pndcm.sh" ] ; then
#	if [ ! -z "$verbose" ] ; then
	   echo "----- previous install found at $instdir : setting reinstall"
#	fi
	reinstall="T"
     fi
   fi
 fi
}

prep_reinstall_pn () {
 	if [ "$debug" = "T" ] ; then echo "preparing for re-install" ; fi
##CONF	remove instance from /etc/pn.conf
  if [ -f /etc/pn.conf ] ; then
	cp /etc/pn.conf /etc/pn.conf.org
	grep $instdir /etc/pn.conf
	inconf=$?
	if [ "$inconf" = "0" ] ; then
		grep -v $instdir /etc/pn.conf.org > /etc/pn.conf
	fi
  fi
##### need to read config file and preserve options - ??? check for changed options...
##### quickie - just preserve the config file as .org
  cp -p $instdir/pn.config $instdir/pn.config.${jdate}
  cp -p $instdir/pn.config $instdir/pn.config.org
  cd $instdir
## get installer version of current installation
## use it to create "old_cron_header" line to be removed from cron
  old_pnversion=`grep 'Power Navigator' pn.config.org | grep -i version | awk '{print $NF}'` 
 	if [ "$debug" = "T" ] ; then echo "	old PNVersion: $old_pnversion" ; fi
  old_cron_header="$raw_cron_header $old_pnversion"
 	if [ "$debug" = "T" ] ; then echo "	old cron header: $old_cron_header" ; fi

 	if [ "$debug" = "T" ] ; then echo "	removing old files" ; fi
# remove old files that need to be replaced
  rm -f $instdir/mpgpn.tar $instdir/rackdata.sh 
  rm -f $instdir/rackd.sh $instdir/pngetinfo.sh 
  if [ "$install_sar" != "T" ] ; then
    rm -f $instdir/pndcm.sh $instdir/pndcutils.sh $instdir/pndchk.sh 
    rm -f $instdir/pntestdmi $instdir/dmi* 
    rm -f $instdir/pndc_truncate.sh $instdir/trunc.sh $instdir/nmon
    rm -f $instdir/nmon_* $instdir/nmonmerge* $instdir/reduce.patt* $instdir/pngetinfo.sh
  else
    rm -f $instdir/sdcm.sh $instdir/sar.sh $instdir/sar.awk $instdir/pngetinfo.sh $instdir/pndchk_sar.sh 
  fi
# clean out cron entries (will replace)
 	if [ "$debug" = "T" ] ; then echo "	cleaning out cron entries" ; fi
  ##### debian specific
  CRONTAB_NOHEADER="N"
  export CRONTAB_NOHEADER
  ##### end debian specific
  eval $cron_get
  cp $crontab_file ${crontab_file}.pre_remove
  grep -v "$cendregex" $crontab_file > ${crontab_file}.0
  grep -v "$nmonregex" ${crontab_file}.0 > ${crontab_file}.1
  grep -v "$nmonregex_old" ${crontab_file}.1 > ${crontab_file}.2
  grep -v "$pndcmregex" ${crontab_file}.2 > ${crontab_file}.3
  grep -v "$rackregex" ${crontab_file}.3 > ${crontab_file}.4
  grep -v "$sdcmregex" ${crontab_file}.4 > ${crontab_file}.5
  grep -v "$truncregex" ${crontab_file}.5 > ${crontab_file}.6
  grep -v "$pngetinforegex" ${crontab_file}.6 > ${crontab_file}.7
  grep -v "^${old_cron_header}$" ${crontab_file}.7 > ${crontab_file}.8
  # Next line removes erroneous headers from crontab in RedHat linux versions
  grep -v "DO NOT EDIT THIS FILE|.*crontab.*installed on|Cron version -" ${crontab_file}.8 > ${crontab_file}.9
  grep -v "$pndchkregex" ${crontab_file}.9 > ${crontab_file}.a
  grep -v "$pndchksarregex" ${crontab_file}.a > ${crontab_file}.b
  mv ${crontab_file}.b $crontab_file
  eval $cron_install
  # remove temporary files
  rm -f ${crontab_file}.0 ${crontab_file}.1 ${crontab_file}.2 
  rm -f ${crontab_file}.3 ${crontab_file}.4 ${crontab_file}.5 
  rm -f ${crontab_file}.6 ${crontab_file}.7 ${crontab_file}.8 
  rm -f ${crontab_file}.9 ${crontab_file}.a
  cd $startdir
}

## "main"
 print_greeting
 get_ostype
 check_for_root
 check_devnull
 set_defaults
 check_reinstall_needed
 create_install_dir
 if [ "$reinstall" = "T" ] ; then 
    prep_reinstall_pn
 fi
 untar_powernav_pkg
 if [ "$install_sar" = "T" ] ; then	## sar specific section
    if [ "$debug" = "T" ] ; then echo "installing sar data collector " ; fi
    check_sar_installed
    check_sar_in_crontab
    check_sdcm_in_crontab
    if [ -d $sardir ]; then
       echo "The directory $sardir exists." 
    else
       echo "The directory $sardir missing. Creating $sardir" 
       mkdir -p $sardir 
    fi
 else	# end sar specific section 
    if [ "$debug" = "T" ] ; then echo "installing nmon data collector " ; fi
    check_nmon_in_crontab
    check_pndcm_in_crontab
    create_nmon_symlinks
 fi  # end nmon specific setup
 check_pngetinfo_in_crontab
 check_rackdata_in_crontab
 force_testplatform_settings
 create_crontab_entries
 do_os_specific_setup
#	 install_inits	# AIX ignores these
 add_to_etc_conf
 set_file_ownerships
 if [ "$install_sar" != "T" ] ; then	## nmon (not sar) specific section
    $instdir/start_nmon_now -f -v
 fi

## SunOS default shell non-posix (or ksh or bash) - get a useful one
 if [ "$ostype" = "SunOS" ] || [ ! -z "$opt_shelltouse" ] ; then
    fixshells
 fi
 sc=${sc:-"success"} ; # no news is good news
 write_config_file "$sc"
 save_rack_config

 if [ "$sc" = "success" ] ; then
    exit 0;
 else
    exit 1;
 fi
## "End of main"
