#!/bin/ksh
##!/bin/ksh -xv
#
# $Id: pnadcp.sh,v 1.11 2012/04/18 02:44:07 root Exp $ 
#
# The following code is Confidential and is covered by the installation license
# Copyright 2004-2012 by Midrange Performance Group
#
#&& Power Navigator Admin Data Collection Process (pnadcp_start.sh)
#& This script runs from rackdata.sh in the background.
#& .
#& Details on variable names 
#& MPG ID time/date/system_name
#& DskVars Disk Variables
#& SWSB System Wide Sticky Bits
#& DskDF ID File Systems
#& DFCnt Directory File Counts
#& DSpc Directory Space
#& BigFile Largest Files
#& .
#& admin_data_collect variable is used to turn off/on admin data collection 
#& and is entered in the pn.config file gathered when rackdata.sh runs.
#& admin_data_collect="F" off the next day.
#& admin_data_collect="T" on the next day.
#

# Since it is running in the background, check and see if it is already running.
ps -ef | grep -v grep | grep "pnadcp.sh" > /tmp/pnadcp.running
jchk=`cat /tmp/pnadcp.running | awk '{if( $3 == 1 ) print $0}' | tee /tmp/jhold | wc -l | awk '{print $1}'` 
if [ $jchk -gt 1 ]; then
   echo "Running pnadcp.sh already ($jchk), exiting!"
   exit
fi

invocdir=`dirname $0`
sysname=`uname -n`

# Some Linuxs and others do not accept a +n option. This helps correct it.
# RedHat is known to only accept -k# option.
jsorttest=`echo "1 3" | sort +1 2>/dev/null`
## echo "${jsorttest}"
## If jsorttest is not empty, then....
if [ ! -z "${jsorttest}" ] ; then
   jsortmod="+2" 
else
   jsortmod="-k3" 
fi
## echo ${jsortmod}
## echo "Remove Later" ; exit 1

# Load the Power Navigator Admin Data Collection configuration options
# if available
if [ -s ${invocdir}/pnadcp.config ]; then
   . $invocdir/pnadcp.config 
fi

# Load the Power Navigator configuration options if available
if [ -s ${invocdir}/pn.config ]; then
   . $invocdir/pn.config 
fi

# Set defaults if not read from pndcutils.sh, pnadcp.config, or pn.config 
if [ "x${TopDFCnt}" = "x" ] ; then
   TopDFCnt=20
fi
if [ "x${TopDSpc}" = "x" ] ; then
   TopDSpc=20
fi
if [ "x${TopBigFile}" = "x" ] ; then
   TopBigFile=20
fi
if [ "x${MinFSz}" = "x" ] ; then
   MinFSz=50000000
fi
if [ "x${MinFCnt}" = "x" ] ; then
   MinFCnt=100
fi
if [ "x${MinDSpc}" = "x" ] ; then
   MinDSpc=10000 
fi

# If not specified externally set pnadcp output file
if [ "x${pnadcp_out}" = "x" ] ; then
   pnadcp_out="${invocdir}/mpgdata4.$sysname"
fi

# Create the mpg file and comment with company name if it does not exist
if [ ! -s ${pnadcp_out} ]; then
   echo "# This file used by Midrange Performance Group, Inc.(MPG) for Power Navigator" >> ${pnadcp_out} 
   echo "# for the admin data collection process (pnadcp)." >> ${pnadcp_out} 
   echo "# For questions, please call 303-939-9648." >> ${pnadcp_out} 
   echo "#" >> ${pnadcp_out} 
   echo "# Details on variable names" >> ${pnadcp_out} 
   echo "# MPG - ID time/date/system_name" >> ${pnadcp_out} 
   echo "# DskVars - Disk Variables" >> ${pnadcp_out} 
   echo "# SWSB - System Wide Sticky Bits" >> ${pnadcp_out} 
   echo "# DskDF - ID File Systems" >> ${pnadcp_out} 
   echo "# DFCnt - Directory File Counts" >> ${pnadcp_out} 
   echo "# DSpc - Directory Space" >> ${pnadcp_out} 
   echo "# BigFile - Largest Files" >> ${pnadcp_out} 
fi

if [ "x${admin_data_collect}" != "xT" ] ; then
   exit 0
fi

# Determine the system os to be used below and determine df or bdf command
# syntax
jOS=`uname -s`
if [ "x${jOS}" = "xSunOS" ] ; then
   # Tested with SunOS 8,9
   jdskcmd="df -k"
elif [ "x${jOS}" = "HP-UX" ] ; then
   jdskcmd="bdf"
else
   # Tested with AIX, SCO_SV, and severa Linux versions
   jdskcmd="df -Pk"
fi

# Use either gawk, nawk, or awk in that order for AWK.
# User the first one found.  gawk has more options than nawk which has
# more options than awk.
for jawk in /bin/gawk /usr/bin/gawk /bin/nawk /usr/bin/nawk /bin/awk /usr/bin/awk
do
   jtest=`which $jawk 2>&1 | egrep -v "annot| no | not "`
   jchk=$?
   if [ $jchk -eq 0 ]; then
      AWK="$jtest"
   break
   fi
done

if [ ! -z "$debug" ] ; then
   echo $AWK
fi

### echo "(${jdskcmd}); Remove Later"
# Add in date, time, and host (Will normally be ran from rackdata if available)
echo "MPG,date,`date +%d-%b-%Y | sed -e 'y/abcdefghijklmnopqrstuvwxyz/ABCDEFGHIJKLMNOPQRSTUVWXYZ/'`" >> ${pnadcp_out} 
echo "MPG,time,`date +%H:%M`" >> ${pnadcp_out} 
echo "MPG,host,${sysname}" >> ${pnadcp_out} 

# Add in Disk Variables used to adjust output
# DskVars,TopBigFile,TopDFCnt,TopDSpc,MinFSz,MinFCnt,MinDSpc
# DskVars,${TopBigFile},${TopDFCnt},${TopDSpc},${MinFSz},${MinFCnt},${MinDSpc}
# Defaults # DskVars,20,20,20,50000000,100,10000
echo "DskVars,TopBigFile,TopDFCnt,TopDSpc,MinFSz,MinFCnt,MinDSpc" >> ${pnadcp_out} 
echo "DskVars,${TopBigFile},${TopDFCnt},${TopDSpc},${MinFSz},${MinFCnt},${MinDSpc}" >> ${pnadcp_out} 

# Counts of system wide sticky bit 4000 perms, 2000 perms, 1000 perms, and
# scripts with sticky bits.
# SWSB,SB4000,SB2000,SB1000,SBScr
echo "SWSB,SB4000,SB2000,SB1000,SBScr" >> ${pnadcp_out} 
# Locate all sticky bit files that are not "directories/modules/empty/character special"
# The intent is to locate files that are scripts or command files, since this may be a problem on some unix/linux systems. 
SBScr=`find /  -perm -2000 -o -perm -1000  -o -perm -4000  2>/dev/null | xargs file /tmp/mpgmpg 2>/dev/null | egrep -v "statically|dynamically|directory|module|empty|character special" | tee /tmp/sticky_scripts.mpginc | wc -l | $AWK '{print $1}'`
# Locate all owner sticky bit files"
SB4000=`find /  -perm -4000 2>/dev/null | tee /tmp/sticky_owner.mpginc | wc -l | $AWK '{print $1}'` 
# Locate all group sticky bit files"
SB2000=`find /  -perm -2000 2>/dev/null | tee /tmp/sticky_group.mpginc | wc -l | $AWK '{print $1}'`
# Locate all everyone sticky bit files"
SB1000=`find /  -perm -1000 2>/dev/null | tee /tmp/sticky_everyone.mpginc | wc -l | $AWK '{print $1}'`
echo "SWSB,${SB4000},${SB2000},${SB1000},${SBScr}" >> ${pnadcp_out} 

### First identify filesystem and mountpoints using system df command ### 
echo "DskDF,FSName,FSDev,FSSize,FSUsed,FSMntPoint" >> ${pnadcp_out} 
${jdskcmd} \
  | grep '%' \
  | egrep "/dev/" \
  | egrep -vi "available|free|/dev/fd" \
  | $AWK '{print $1,$1,$2,$3,$NF}' \
  | sed -e 's,^/dev/dsk/,,' \
  | sed -e 's,^/dev/,,' \
  | $AWK '{print "0"$1,$0}' \
  | sed -e 's,^0[a-z][a-z]*,0,' \
  | sort -n \
  | $AWK '{print $2,$3,$4,$5,$6}' > /tmp/df_k.tmp 
cat /tmp/df_k.tmp | $AWK '{print "DskDF,"$1","$2","$3","$4","$5}' >> ${pnadcp_out} 
#----------------------------#

## echo "Remove later"; exit 1

### Use filesystems and mountpoints to find directories and get count ###
### of files in each directory and display top 20 counts of each fs.  ###
echo "DFCnt,FSName,FCnt,DirName" >> ${pnadcp_out} 

while
read j_fs j_dev j_size j_used j_mp
do
   for j_dir in `find ${j_mp} -xdev ! -fstype nfs -type d 2>/dev/null`
   do
      j_cnt=`ls -1 ${j_dir} 2>/dev/null |wc -l |$AWK '{print $1}'`
      echo "DFCnt,${j_fs},${j_cnt},${j_dir}" \
       | $AWK -F, -v minfcnt=${MinFCnt} '{if($3>=minfcnt) print $0}'
   done | sort -t, -nr ${jsortmod} | head -${TopDFCnt}
done < /tmp/df_k.tmp >> ${pnadcp_out} 
#----------------------------#

### Use filesystems and mountpoints to find directories and ###
### get space used by files in each directory and display top 20 counts of each fs.   ###
echo "DSpc,FSName,DirSpcK,DirName" >> ${pnadcp_out} 

while
read j_fs j_dev j_size j_used j_mp
do
   for j_dir in `find ${j_mp} -xdev ! -fstype nfs -type d 2>/dev/null`
   do
      j_cnt=`ls -1s ${j_dir} 2>/dev/null | grep "^total" |$AWK '{print $2}'`
      echo "DSpc,${j_fs},${j_cnt},${j_dir}" \
       | $AWK -F, -v mindspc=$MinDSpc '{if($3>=mindspc) print $0}' 
   done | sort -t, -nr ${jsortmod} | head -${TopDSpc}
done < /tmp/df_k.tmp >> ${pnadcp_out} 
#----------------------------#

### Use filesystems and mountpoints to find files over 50 megs in size  ###

### display top 20 files by size.                                       ###
echo "BigFile,FSName,KBytes,Perm,Links,Own,Grp,Bytes,M,D,Yr_Time,Path_Name" >> ${pnadcp_out} 

while
read j_fs j_dev j_size j_used j_mp
do
   # The /tmp/mpgmpg was required for Suse, because the xargs was activating 
   # with no data came to it.
   find ${j_mp}  -xdev -type f -size +${MinFSz}c 2>/dev/null \
    | xargs ls -als /tmp/mpgmpg 2>/dev/null  \
    | sort -nr  \
    | head -${TopBigFile} \
    | sed -e 's/\ \ */\,/g' \
    | $AWK -v jfs=$j_fs -v jdev=$j_dev -v jmp=$j_mp '{print "BigFile,"jfs","$0}'
done < /tmp/df_k.tmp >> ${pnadcp_out} 
#----------------------------#

echo "MPG,done,`date +%d-%b-%Y | sed -e 'y/abcdefghijklmnopqrstuvwxyz/ABCDEFGHIJKLMNOPQRSTUVWXYZ/'`,`date +%H:%M`" >> ${pnadcp_out} 

