#!/bin/ksh 
#
# $Id: pnbmpgd2.sh,v 1.11 2012/04/10 18:13:53 root Exp $
#
# The following code is Confidential and is covered by the installation license
# Copyright 2004-2012 by Midrange Performance Group
#
#&& Power Navigator Build mpgdata2.hostname file (pnbmpgd2.sh) 
#&
#& Will build an mpgdata2 file from existing nmon files that were
#& not created by the Power Navigator consolidation process.
#& It is intended to be used on existing nmon data and will build an
#& mpgdata2.hostname file on an AIX system.
#& 
#& This will allow existing data to be used with the Power Navigator 
#& consolidation process.
#
#&@ Requires files of the nmon data collected from an existing nmon 
#&@ installation. 
#&@
#&@ Will also run "pnbrdf.sh" if it is the same directory as pnbmpgd2.sh 
#&
#& All nmon files in the current directory will be processed except mpgdata2 
#& files
#&
#& If a mpgdata2.hostname file alread exists, it will create a 
#& mpgdata2.hostname_date_hourmin file.
#&

#& To create the mpgdata2 manually do the following:
#& 
#& First test (ls -1tr *nmon*) and confirm that only nmon files are listed.
#& If not, replace the *nmon* wildcard statement with one that lists nmon files.
#& 
#& 	ls -1tr *nmon*  | xargs cat /tmp/mpgmpg 2>/dev/null >> mpgdata2.hostname
# 
echo "Running `grep "\$\I\d\:" ${0} | awk '{print $3,$4,$5,$6}' ` "
#
# invocation dir (default for temp/data files, utilities)
invocdir=`dirname $0` 
#

# Start analyzing the files in the current directory to create 
echo ""
echo Analyzing the nmon files in the current directory 

jdate=`date '+%Y%m%d_%H%M'`
grep "AAA\,host\," * | sed s/:/,/ | awk -F, '{print $1,$4}'| egrep -v "^mpgdata2|temp$|tmp$|^pnbmpgd2|^pnbrdf" | sed s,
,,g | uniq > files_host.tmp
while 
read jfile jhost
do
   jjtest=`grep "AAA\,host\," $jfile | sort | uniq | tee filetest.tmp |  wc -l | awk '{print $1}'`
   # echo $jjtest
   if [ $jjtest -eq 1 ]; then
      echo "$jhost" >> hosts_processed.tmp 
      cat $jfile | sed s,
,,g >> mpgdata2.${jhost}_${jdate} 
   else
      echo ""
      echo "Ignoring file $jfile, because more than one host was in the data."
      cat filetest.tmp  | awk '{print "     "$0}'
   fi
done < files_host.tmp


cat hosts_processed.tmp | sort | uniq > hosts_processed_uniq.tmp
cat  hosts_processed_uniq.tmp \
   | awk '{print "mpgdata2."$1"_"Jdate}' Jdate=$jdate > files_created.tmp
while
read filename
do
  jfiletest=`echo "$filename" | awk -F_ '{print $1}'`
  if [ -s ${jfiletest} ]; then
     echo $filename >> files_created.tmp2
  else 
     mv $filename $jfiletest
     echo $jfiletest >> files_created.tmp2
  fi
done < files_created.tmp
mv files_created.tmp2 files_created.tmp

echo ""
echo "Files created were: "
cat files_created.tmp | awk '{print "     "$0}'

while
read joutfile
do
   if [ -s ${invocdir}/pnbrdf.sh ]; then 
      echo ""
      echo "Running (${invocdir}/pnbrdf.sh $joutfile) to build rackdata.${jhostname}"
      ${invocdir}/pnbrdf.sh $joutfile
   else
      echo "Did not find (${invocdir}/pnbrdf.sh $joutfile)."
      echo "Please run (pnbrdf.sh $joutfile) seperately."
   fi
done <  files_created.tmp

rm -f  files_created.tmp files_created.tmp2 files_host.tmp filetest.tmp hosts_processed.tmp hosts_processed_uniq.tmp

exit


#& Sample run:
#&
#& Running pnbmpgd2.sh,v #.# YYYY/MM/DD hh:mm:ss
#&
#& Analyzing the nmon files in the current directory
#&
#& Ignoring file badfile_20080601, because more than one host was in the data.
#&      AAA,host,bigserve
#&      AAA,host,orarack
#&
#& Ignoring file badfile_20080601, because more than one host was in the data.
#&      AAA,host,bigserve
#&      AAA,host,orarack
#&
#& Files created were:
#&      mpgdata2.bigserve
#&      mpgdata2.orarack
#&
#& Running (/home/pn/hold4build/pnbrdf.sh mpgdata2.bigserve) to build rackdata.
#&     rackdata.bigserve was created from mpgdata2.bigserve
#&
#& Running (/home/pn/hold4build/pnbrdf.sh mpgdata2.orarack) to build rackdata.
#&     rackdata.orarack was created from mpgdata2.orarack

