#!/bin/ksh 
#
# $Id: pnbrdf.sh,v 1.13 2012/04/10 18:13:53 root Exp $
#
# The following code is Confidential and is covered by the installation license
# Copyright 2004-2012 by Midrange Performance Group
#
#&& Power Navigator Build Rack Data File (pnbrdf.sh) 
#&
#& This script will build a rackdata output file with out the df -k outputs 
#& It is intended to be used if existing nmon data was used to build an
#& mpgdata2.hostname file on an AIX system.
#
#&@ Requires a file input of the data collected from an existing nmon 
#&@ installation. If nofile is specified, a syntax message will be presented.  
#&
#& If no mpgdata2.hostname file exists, to build an mpgdata2.hostname file:
#& First test (ls -1tr *nmon*) and confirm that only nmon files are listed.
#& If not, replace the *nmon* wildcard statement with one that lists nmon files.
#& 
#& 	ls -1tr *nmon*  | xargs cat /tmp/mpgmpg 2>/dev/null >> mpgdata2.hostname
# 
#
# invocation dir (default for temp/data files, utilities)
invocdir=`dirname $0` 

jsyntax () {
   echo "Error: $1" 
   echo "     pnbrdf.sh filename"
   echo "     (filename must be a valid mpgdata2.hostname file)"
   exit 1 
}
if [ "x${1}x" = "xx" ] ; then
   jsyntax "No nmon file specified!"
else
   jmainfile=${1}
fi
if [ -s ${jmainfile} ] ; then
   # Confirm that the file is an nmon file.
   # Test that this is an nmon file.
   jhostname=`grep ",host,"  ${jmainfile} | head -1 | awk -F, '{print $3}'`
   # echo "hostname is $jhostname!"
   if [ "x${jhostname}x" = "xx" ] ; then
      jsyntax "File ${jmainfile} is not an nmon file!"
   fi
else
   jsyntax "File ${jmainfile} is empty or does not exist"
fi

# Get the last nmon section for most current details and build a file from it. 
lastnmon=`grep -n 'AAA,progname' ${jmainfile} | tail -1 | awk -F: '{print $1}'`
jfile="./nmon.$jhostname.temp" # tempfile for most nmon recent data
tail -n +${lastnmon} ${jmainfile} > ${jfile}

#
# Start analyzing the file to create rackdata.hostname_YYYYMMDD_hhmm
jdate=`date '+%Y%m%d_%H%M'`
# If rackdata.hostname exists, create rackdata.hostname_datestamp 
if [ -s  rackdata.${jhostname} ] ; then
   joutfile="rackdata.${jhostname}_${jdate}"
else
   joutfile="rackdata.${jhostname}"
fi

#######
Is_AIX (){
   echo "# Autogenerated from $jmainfile"
   echo rackdata for $jhostname 
   nn=$jhostname
   nnd="Machine Node Name"
   printf "%22s = %s\n" "$nnd" "$nn"
   tempfile="${invocdir}/mpg.temp" # default tempfile

   josinfo=`grep 'AAA,AIX,' ${jfile} | sed -e 's/\./\,/g' | head -1`
   # echo $josinfo Remove later
   
   os=`echo $josinfo | awk -F',' '{print $2}'` 
   osd="Operating System Name"
   printf "%22s = %s\n" "$osd" "$os"

   osv=`echo $josinfo | awk -F',' '{print $3}'` 
   osvd="OS Version"
   printf "%22s = %s\n" "$osvd" "$osv"

   osr=`echo $josinfo | awk -F',' '{print $4}'` 
   osrd="OS Release"
   printf "%22s = %s\n" "$osrd" "$osr"
   
   osid="hostid"
   osi="Unknown"
   if [ "x${osi}x" != "xx" ]; then
      printf "%22s = %s\n" "$osid" "$osi"
   fi

   jproctype=`grep  'Processor Type:' ${jfile} | sed -e 's/\"//g' | head -1 | awk '{print $NF}'`
   jtitle="Processor Type"
   if [ "x${jproctype}x" != "xx" ]; then
      printf "%22s = %s\n" "$jtitle" "$jproctype"
   fi

#   jserial=`grep 'AAA,SerialNumber,' ${jfile} | head -1 | awk -F',' '{print $3}'`
   jserial=`grep 'Machine Serial Number' ${jfile} | head -1 | sed -e 's/"//g' | awk '{print $NF}'` 
#   echo $jserial
#   echo "Remove Later"; exit 1

   if [ $jserial ] 
   then 
      memd="Serial"
      printf "%22s = %s\n" "$memd" "$jserial"
   fi
   

   # Query for AIX TL level
   joslevel=`grep oslevel ${jfile} |  grep "\-[0-9][0-9]\-" | sed -e 's/"//g' | head -1 | awk -F',' '{print $4}'`
   OSLV=`echo $joslevel | cut -c1-2`
   if [ "x${OSLV}x" = "xx" ]
   then
      OSLV="${osv}${osr}"
   fi 
   if (( OSLV >= 53 ))
   then
      tl=`echo $joslevel | cut -c9-10`
      if [ "x${tl}x" = "xx" ]
      then
         tl="Unknown"
      fi
      tld="Technology Level"
      printf "%22s = %s\n" "$tld" "$tl"
   fi
   
   isa="Unknown"
   isad="Instruction set architecture"
   printf "%40s = %s\n" "$isad" "$isa"

   mt_mn=`grep 'System Model:' ${jfile} | sed -e 's/"//g' | head -1 | awk '{print $NF}'`
   mt_mnd="Machine Type and Model Name"
   printf "%40s = %s\n" "$mt_mnd" "$mt_mn"

   pc_mi=`grep -i 'systemid' ${jfile} | awk '{print $4}' | head -1`
   pc_mid="Plant Code and Machine ID"
   printf "%40s = %s\n" "$pc_mid" "$pc_mi"

   ## Memory info
   mem=`grep -i 'realmem' ${jfile} | awk '{print $4}' | head -1`
   memd="Amount of usable physical memory in K"
   printf "%40s = %s\n" "$memd" "$mem"


   if (( OSLV >= 53 )); then
   # if [ "$osv" = "5" ] && [ "$osr" -gt "2" ] ; then
      ## LPAR specific (V5R3)

      # Next line did not work for older versions of nmon
      # lpnn=`grep -i 'LPARNumberName' ${jfile} | head -1 | awk -F',' '{print $3" "$4}'`  
      lpnn=`egrep -i "Partition N" ${jfile} | sed -e 's/"//g'  | awk '{print $NF}' | paste - - | awk '{print $2,$1}'`
      lpnnd="LPAR Number and LPAR Name"
      printf "%40s = %s\n" "$lpnnd" "$lpnn"

      lpcap=`grep -i 'Partition is capped' ${jfile} | awk '{print $4}' | head -1`
      lpcapd="Partition is capped"
      printf "%40s = %s\n" "$lpcapd" "$lpcap"

      lpded=`grep -i 'Partition is dedicated' ${jfile} | awk '{print $4}' | head -1`
      lpdedd="Partition is dedicated"
      printf "%40s = %s\n" "$lpdedd" "$lpded"

      lpentcap=`grep -i 'ent_capacity' ${jfile} | awk '{print $4}' | head -1`
      lpentcapd="Entitled processor capacity"
      printf "%40s = %s\n" "$lpentcapd" "$lpentcap"

      lpmincap=`grep -i 'min_capacity' ${jfile} | awk '{print $4}' | head -1`
      lpmincapd="Minimum potential processor capacity"
      printf "%40s = %s\n" "$lpmincapd" "$lpmincap"

      lpmaxcap=`grep -i 'max_capacity' ${jfile} | awk '{print $4}' | head -1`
      lpmaxcapd="Maximum potential processor capacity"
      printf "%40s = %s\n" "$lpmaxcapd" "$lpmaxcap"
   fi
   
   ## get disk info (number, name, type, and sizes) 
   grep 'BBBB' ${jfile}   \
      | egrep -v "unknown|,cd|BBBB,0000" \
      | sort \
      | uniq \
      | awk -F',' '{print "",$3,":","type:",$5,":","Size in MB:", 1000*$4}' > ${tempfile}
   jdcnt=`wc -l ${tempfile} | awk '{print $1}'`
   echo ; echo "found $jdcnt disk devices :"
   cat ${tempfile}
   if [ -s ${tempfile} ] ; then
       rm -f ${tempfile} 
       # echo ${tempfile} 
   fi 
   echo "###########"
   echo "No df output available"
   echo "###########"
}

grep 'AAA,AIX,' ${jfile} > /dev/null
jchk=$?
if [ $jchk -eq 0 ] ; then
   Is_AIX > ${joutfile} 
   echo "    ${joutfile} was created from ${jmainfile}"
   # Is_AIX 
fi
# Remove temporary ${jfile}
if [ -s ${jfile} ] ; then
    rm -f ${jfile} 
   # echo $jfile
fi

exit

###### 
###### 
###### 
All of the lines below this are for possible use with Linux, Solaris, or HPUX
###### 
###### 
###### 

# get rack config info 

rac_generic () {
   # generic rack info (should work on any system)
   ##### see about using a parm for width (%40s or %30s or %25s...)



   os=`uname -s`
   osd="Operating System Name"
   printf "%22s = %s\n" "$osd" "$os"

   osv=`uname -v`
   osvd="OS Version"
   printf "%22s = %s\n" "$osvd" "$osv"

   osr=`uname -r`
   osrd="OS Release"
   printf "%22s = %s\n" "$osrd" "$osr"
   
   osid="hostid"
   if [ "x${osi}x" != "xx" ]; then
      printf "%22s = %s\n" "$osid" "$osi"
   fi

   if [ `uname -s` = "AIX" ] && [ "x${jserial}x" = "xx" ] ; then 
     jserial=`uname -m`
   fi

   if [ $jserial ] 
   then 
      memd="Serial"
      printf "%22s = %s\n" "$memd" "$jserial"
   fi
}

rac_aix () {
   # collect "rack config" on AIX systems
   #  this only works completely on AIX 5.3 and later

   rac_generic
   ##### AIX specific
   get_tl_level

   isa=`uname -p`
   isad="Instruction set architecture"
   printf "%40s = %s\n" "$isad" "$isa"

   mt_mn=`uname -M`
   mt_mnd="Machine Type and Model Name"
   printf "%40s = %s\n" "$mt_mnd" "$mt_mn"

   pc_mi=`uname -u`
   pc_mid="Plant Code and Machine ID"
   printf "%40s = %s\n" "$pc_mid" "$pc_mi"

   ## Memory info

   mem=`lsattr -El sys0 -a realmem -F value`
   memd="Amount of usable physical memory in K"
   printf "%40s = %s\n" "$memd" "$mem"

   if [ "$osv" = "5" ] && [ "$osr" -gt "2" ] ; then
      ## LPAR specific (V5R3)

      lpnn=`uname -L`
      lpnnd="LPAR Number and LPAR Name"
      printf "%40s = %s\n" "$lpnnd" "$lpnn"

      lpcap=`lsattr -El sys0 -a capped -F value`
      lpcapd="Partition is capped"
      printf "%40s = %s\n" "$lpcapd" "$lpcap"

      lpded=`lsattr -El sys0 -a dedicated -F value`
      lpdedd="Partition is dedicated"
      printf "%40s = %s\n" "$lpdedd" "$lpded"

      lpentcap=`lsattr -El sys0 -a ent_capacity -F value`
      lpentcapd="Entitled processor capacity"
      printf "%40s = %s\n" "$lpentcapd" "$lpentcap"

      lpmincap=`lsattr -El sys0 -a min_capacity -F value`
      lpmincapd="Minimum potential processor capacity"
      printf "%40s = %s\n" "$lpmincapd" "$lpmincap"

      lpmaxcap=`lsattr -El sys0 -a max_capacity -F value`
      lpmaxcapd="Maximum potential processor capacity"
      printf "%40s = %s\n" "$lpmaxcapd" "$lpmaxcap"
   fi

      disksizedesc[$i]="Size in MB"
      grep 'BBBB' mpgdata2.core03  | egrep -v "unknown|,cd|BBBB,0000" | sort | uniq | awk -F',' '{print $3,$4,$5}'      
      nd=$i
      ## report disk info
      echo " ${diskname[$i]} : type: ${disktype[$i]} ${disp_disksize[$i]}"
}


   ## Mainframe LPAR data
   if [ "$osm" = "s390x" ] ; then 
      lpnn=`cat /proc/sysinfo | grep 'LPAR Number:' | awk '{print $3}'`
      lpnnd="LPAR Number"
      printf "%22s = %sb\n" "$lpnnd" "$lpnn"

      lpchar=`cat /proc/sysinfo | grep 'LPAR Characteristics:' | awk '{print $3}'`
      lpchard="LPAR Characteristics"
      printf "%22s = %s\n" "$lpchard" "$lpchar"

      lpvmtot=`cat /proc/sysinfo | grep 'VM.. CPUs Total:' | awk '{print $4}'`
      lpvmtotd="VM Total CPUs"
      printf "%22s = %s\n" "$lpvmtotd" "$lpvmtot"
 
      lpvmcfg=`cat /proc/sysinfo | grep 'VM.. CPUs Configured:' | awk '{print $4}'`
      lpvmcfgd="VM Configured CPUs"
      printf "%22s = %s\n" "$lpvmcfgd" "$lpvmcfg"
   fi
 

if [ "$OStype" = "AIX" ] ; then
   rac_aix >> $outfile 
fi

# cat $dffile >> $outfile
