#!/bin/ksh   
#
# $Id: pngetinfo.sh,v 1.27 2012/04/09 23:41:19 root Exp root $
#
# The following code is Confidential and is covered by the installation license
# Copyright 2004-2012 by Midrange Performance Group
#
#&& Power Navigator Get Information (pngetinfo.sh)
#
#& This script gets the information that is used by Midrange Performance Group
#& and puts them in one file. (pninfo.hostname.systype.tar.gz) 
#&   Options are:
#& 	-a auto gzip of tar file (Does not build a tar file first)
#& 	-q quite compression
#& 	-c current data only (7 days)(large files not included)
#&      -n # number of days to gather (most recent)(large files not included)
#&         if no -n specified, will default to everyday
#&         unless there is a -c specified
#& 	-h help/usage message
#&   Can gather large files: mpgdata, mpgdata1, mpgdata2, mpgdata3, mpgdata4 
#&   Always gathers current, last# (days), config files, rackdata outputs,
#&   and *.mpginc  
#
#&@ pndcutils.sh is also required
#
#&% Ran daily in crontab
#
# Added a -q for quiet operation (Nothing echoes to the screen for cron)
# Will replace the current compression option to create the tar file 
# of all data files.

invocdir=`dirname $0`
cd $invocdir
nmondir=$invocdir
if [ -s $invocdir/pndcutils.sh ] ; then 
   . $invocdir/pndcutils.sh
fi
if [ -s $invocdir/pn.config ] ; then 
   . $invocdir/pn.config
fi

# Remove files that have _last# in the name

rm -f mpgdata*_last[1-9]* 2>&1 > /dev/null

PATH=$PATH:/usr/contrib/bin:/usr/bin

function usage {
echo "usage: $0 [options]
  Options may be any of:
   	-a auto gzip of tar file (Does not build a tar file first)
   	-q quite compression
   	-c current data only (7 days)(large files not included)
        -n # number of days to gather (most recent)(large files not included)
           if no -n specified, will default to everyday
           unless there is a -c specified
        -h help/usage message
"
}

#& This script gets the information that is used by Midrange Performance Group
#& and puts them in one file. (pninfo.hostname.systype.tar.gz) 
#&   Options are:
#& 	-a auto gzip of tar file (Does not build a tar file first)
#& 	-q quite compression
#& 	-c current data only (7 days)(large files not included)
#&      -n # number of days to gather (most recent)(large files not included)
#&         if no -n specified, will default to everyday
#&         unless there is a -c specified
#& 	-h help/usage message
#&   Can gather large files: mpgdata, mpgdata1, mpgdata2, mpgdata3 
#&   Always gathers current, last# (days), config files, rackdata outputs,
#&   and *.mpginc  

jthissystem="T"

pnhost=`uname -n`
if [ -s "./rackdata.${pnhost}" ] ; then 
   pnsys=`uname -s`
else
   echo "Looks like this is being ran on data that is not from this system."
   pnhost=`grep "Machine Node Name" rackdata.* | head -1 |  awk '{print $NF}'`
   pnsys=`grep "Operating System Name" rackdata.* | head -1 | awk '{print $NF}'`
   echo "Running on files from ${pnhost} on a ${pnsys} system."
   jthissystem="F"
fi

jautogzip="F"
jcurrent="F"
jquiet="F"
jcntchange="F" 

while getopts ":achqn:" opt ; do
   case $opt in
	  a)      jautogzip="T" ;;
	  c)      jcurrent="T" ;;
	  q)      jquiet="T" ;;
	  n)      jcntchange="T" 
			jcurrent="T" 
			jdays="$OPTARG";;
	  h)      usage;  exit ;;
	  ?)      usage;  exit ;;
   esac
done

# If no count change is specified, default to 7 days
if [ "$jcurrent" = "T" ] && [ "$jcntchange" = "F" ] ; then
   jdays="7"
fi
# If no number of days is not still specified, default to 7 days
if [ "x${jdays}x" = "xx" ] ; then
   jdays="7"
fi


if [ "x${jcurrent}x" = "xTx" ] ; then
   echo No Big files
   jbigfiles=""
else
   # echo With Big files
   jbigfiles="mpgdata.${pnhost} mpgdata1.${pnhost} mpgdata2.${pnhost} mpgdata3.${pnhost} mpgdata4.${pnhost}   "
fi

chosen_files="${jbigfiles} mpgdata4_last${jdays}.${pnhost} \
mpgdata3_last${jdays}.${pnhost} mpgdata2_current.${pnhost} \
mpgdata2_last${jdays}.${pnhost} mpgdata1_last${jdays}.${pnhost} \
mpgdata_current.${pnhost} mpgdata_last${jdays}.${pnhost} rackdata.${pnhost} \
rdhist.${pnhost} df.${pnhost} pn.config.${pnhost} etc_pn.conf.${pnhost} \
pn.config *.mpg *.mpginc *.inf"

# echo $chosen_files

# tail command different on SunOS systems
jtailchk=`uname -s`
if [ "x${jtailchk}x" = "xSunOSx" ] ; then
   jtailmod=''
else
   jtailmod='-n'
fi

# Not currently gathering mpgdata1.hostname, but placed for future
if [ -s mpgdata1.${pnhost} ] ; then
   ## To gather a last 7 day file for current data gathering mpgdata1
   ## This will aid in troubleshooting with customers on recent data
   j7day=`egrep -n "ZZZZ,T0001,00"  mpgdata1.${pnhost} | tail ${jtailmod} -${jdays} | head -1 | awk -F: '{print $1}'`
   tail ${jtailmod} +${j7day} mpgdata1.${pnhost} > mpgdata1_last${jdays}.${pnhost}
fi

if [ -s mpgdata2.${pnhost} ] ; then
   ## To gather a last 7 day file for current data gathering mpgdata2
   ## This will aid in troubleshooting with customers on recent data
   # Find the find the location of the oldest of the last 7 dates in mpgdata3
   j7day=`egrep -n "AAA,progname|AAA,date"  mpgdata2.${pnhost} | paste - - | tail ${jtailmod} -${jdays} | head -1 | awk -F: '{print $1}'`
   # Tail mpgdata2.hostname jloc2 as the 1st line of mpgdata2_last7.hostname
   tail ${jtailmod} +${j7day} mpgdata2.${pnhost} > mpgdata2_last${jdays}.${pnhost}
fi

if [ -s mpgdata3.${pnhost} ] ; then
   ## To gather a last 7 day file for current data gathering mpgdata3
   ## This will aid in troubleshooting with customers on recent data
   # Find the last 7 dates in mpgdata3
   jdate=`egrep ",date,"  mpgdata3.${pnhost} | uniq | tail ${jtailmod} -${jdays} | head -1 | awk -F: '{print $1}'`
   if [ "x${jdate}x" = "xx" ] ; then
      # if date is blank create a blank mpgdata3_last7.${pnhost}
      touch mpgdata3_last${jdays}.${pnhost}
   else
      # Find the find the location of the oldest of the last 7 dates in mpgdata3
      jloc=`egrep -n "$jdate"  mpgdata3.${pnhost} | head -1 | awk -F: '{print $1}'`
      # Adjust the location by two lines backwards
      jloc2=$(expr $jloc - 2)
      # Tail mpgdata3.hostname jloc2 as the 1st line of mpgdata3_last${jdays}.hostname
      tail ${jtailmod} +${jloc2} mpgdata3.${pnhost} > mpgdata3_last${jdays}.${pnhost}
   fi
fi

# pnadcm.sh creates mpgdata4.hostname and is common to all unix/linux
# to gather admin data and is keyed by rackdata.sh script
if [ -s mpgdata4.${pnhost} ] ; then
   ## To gather a last 7 day file for current data gathering mpgdata4
   ## This will aid in troubleshooting with customers on recent data
   j7daypoint=`egrep "MPG,date"  mpgdata4.${pnhost} | uniq | tail ${jtailmod} -${jdays} | head -1 | awk -F, '{print $3}'`
   j7day=`egrep -n "${j7daypoint}"  mpgdata4.${pnhost} | head -1 | awk -F: '{print $1}'`
   tail ${jtailmod} +${j7day} mpgdata4.${pnhost} > mpgdata4_last${jdays}.${pnhost}
fi

if [ -s mpgdata.${pnhost} ] ; then
   ## To gather a last 7 day file for current data gathering mpgdata
   ## This will aid in troubleshooting with customers on recent data
   jfirstdate=`grep ^2 mpgdata.${pnhost} | cut -c1-40 | awk -F\; '{print substr($1,1,8)}' | uniq | tail ${jtailmod} -${jdays} | head -1`
   jloc=`grep -n ${jfirstdate} mpgdata.${pnhost} | head -1 | awk -F: '{print $1}'`
   jloc2=$(expr $jloc - 1)
   tail ${jtailmod} +${jloc2} mpgdata.${pnhost} > mpgdata_last${jdays}.${pnhost}
fi

## if there is no nmon then we are on a Sun/HP-UX/SCO system or one running
## sar.
jchk=`find nmon 2>/dev/null`
if [ "xx" = "x${jchk}x" ] && [ -s ./sar.sh ] ; then
   ## Gather the todays mpgdata_current file
   ./sar.sh > mpgdata_current.${pnhost}
fi

if [ "x${jthissystem}x" = "T" ]; then
  cp -p /etc/pn.conf etc_pn.conf.${pnhost}
  cp -p pn.config pn.config.${pnhost}
fi

# If transfer_script_cmd variable assigned:
# If there is a transfer_script_cmd variable, and it has transfer_script.sh
# in the variable, then run the transfer_script_cmd commmand.
if [ "x${transfer_script_cmd}x" != "xx" ] ; then
   echo ${transfer_script_cmd} | grep transfer_script.sh > /dev/null
   jchk=$?
   if [ ${jchk} -eq 0 ] ; then
      ${transfer_script_cmd}
   fi
fi

if [ "x${jquiet}x" = "xTx" ] ; then
   if [ "x${jautogzip}x" = "xTx" ] ; then
      tar -cvf - ${chosen_files} 2> /dev/null | gzip -c > pninfo.${pnhost}.${pnsys}.tar.gz
   else
      tar -cvf pninfo.${pnhost}.${pnsys}.tar ${chosen_files} 2> /dev/null  > /dev/null
      gzip -f pninfo.${pnhost}.${pnsys}.tar
   fi
else
   if [ "x${jautogzip}x" = "xTx" ] ; then
      tar -cvf - ${chosen_files} 2> /dev/null | gzip -c > pninfo.${pnhost}.${pnsys}.tar.gz
   else
      tar -cvf pninfo.${pnhost}.${pnsys}.tar ${chosen_files} 2> /dev/null 
      gzip -f pninfo.${pnhost}.${pnsys}.tar
   fi
   echo "Please send file pninfo.${pnhost}.${pnsys}.tar.gz for Performance Navigator"
   echo "to Midrange Performance Group (MPG)" 
fi

