#!/bin/ksh
#!/bin/ksh -xv    
#
# $Id: rackdata.sh,v 1.64 2012/04/12 03:48:18 root Exp $
#
# The following code is Confidential and is covered by the installation license
# Copyright 2004-2012 by Midrange Performance Group
#
#&& rackdata.sh creates a file rackdata.(uname -n) with system information
#& A continuous df -k or bdf of disk data is gathered daily or whenever it it ran.
#&
#& -D option will add outputs to the screen
#
#&@ df.hostname is used to gather history of disk space  
#
#&% If the df.(uname -n) file does not exist, only the present data is gathered 
#&% Ran daily in crontab
#
# Randy Watson - 3/05/2009
#    Enhance to support all OS types for file systems data
#    Collect disk info for Linux
#    Add unique Mainframe rackdata info
#    Add date to DF command
#    Append Df results daily for history
#    Need to add purge job for df.hostname file - keep one year
#
## Jack Miller 4/18/2009
#    Added shell of ksh, not sh compliant.
#
## Jack Miller 6/11/2009
#    Is now fully sh compliant.
#    Added significant data gathering enhancments for all versions of linux
#    and Unix.  Changed the data for the df command to output in comma
#    delimited format.
#
## Jack Miller 6/23/2009
#    Significant changes to handle data gathering on virtual machines for both
#    sun and hp.


PATH=$PATH:/sbin

dateformat=`date '+%d-%b-%C%y'`
jdate=`date +%Y%m%d_%H%M`

# /tmp/rackdata_debug.tmp used to temporarily store debug messages for the screen.
rm -f /tmp/rackdata_debug.tmp
touch /tmp/rackdata_debug.tmp

locate_app () {
   # Identify locations of applications 
   # Matt identified need for next line; Thanks!
   jloc=''  
   # HP-UX  required the grep "^no "
   # Check normal bin sbin directories for application
   for jdirs in /bin/ /sbin/ /usr/bin/ /usr/sbin/ /usr/local/bin/ /usr/local/sbin/ /usr/contrib/bin/ /usr/contrib/sbin/ /opt/hpvm/bin/ ${jsolaris_sbin}
   do 
      if [ -x ${jdirs}${1} ]; then
         jloc="${jdirs}${1}"
         break
      fi 
   done
   if [ "x${jloc}x" != "xx" ]; then
      export jloc  
      return 0
   else
      jloc=`which $1 2>/dev/null | grep -v "^no "`
      if [ "x${jloc}x" != "xx" ]; then
         export jloc  
         return 0
      else
         export jloc  
         return 1
      fi
   fi
}

disk_info () {
   ### Beginning of disk gathering information
   ### Disk information originally for AIX. 
   ### Setting up for linux solaris hpux 
   ### get disk info (number, name, type, and sizes)
   locate_app powermt
   jpowermt=$jloc
   if [ "xx" != "x${jpowermt}x" ] ; then
      if [ `$juname -s` = "AIX" ] ; then
         # ###############
         # AIX - hdiskpower#
         # ###############
         $jpowermt display dev=all 2>/dev/null \
            | egrep -v "unknown|dead" \
            | sed '/./,/^$/!d' \
            | sed "s/=/ /"  \
            | awk '/^Pseudo/ {printf substr($3,11)" "$3" "} /active/{printf $3" "}/^$/' > /tmp/rackdata_powermt.tmp
         while
         read jline
         do
            jpseudo=`echo $jline | awk '{print $1}'`
            jpseudo2=`echo $jline | awk '{print $2}'`
            echo "$jline" \
               | cut -d" " -f3- \
               | tr " " "\n" \
               | sed "s/hdisk/_/g" \
               | sort -t_ -k1n -k2n -k3n -k4n \
               | sed "s/_/hdisk/" \
               | tr "\n" " " \
               | awk -v jpse=$jpseudo -v jpse2=$jpseudo2 '{print jpse, jpse2, $0}'
         done < /tmp/rackdata_powermt.tmp \
            | sort -n \
            | cut -d" " -f2- > ${nmondir}/powermt.mpginc
         cat ${nmondir}/powermt.mpginc 2>/dev/null \
            | awk '{if( NF != 0 )print $1, NF-1}' > ${nmondir}/powermt_paths.mpginc
      elif [ `$juname -s` = "HP-UX" ] ; then
         # ############
         # HPUX without pseudoname
         # ############
         $jpowermt display dev=all 2>/dev/null \
            | egrep -v "unknown|dead" \
            | sed '/./,/^$/!d' \
            | awk '/^$/ {print} /active/{printf " " $3}  ' > /tmp/rackdata_powermt.tmp
         while
         read jline
         do
           echo "$jline" \
             | tr " " "\n" \
             | sed "s/[a-z]/_/g" \
             | sort -t_ -k1n -k2n -k3n -k4n \
             | sed s/_/c/ \
             | sed s/_/t/ \
             | sed s/_/d/ \
             | tr "\n" " "
           echo ""
         done < /tmp/rackdata_powermt.tmp \
            | grep -v "^\ *$" \
            | awk '{print $0}' \
            | sed s,c,_, \
            | sed s,t,_, \
            | sed s,d,_, \
            | sort -t_ -k1n -k2n -k3n -k4n \
            | sed s,_,c, \
            | sed s,_,t, \
            | sed s,_,d, \
            | awk '{print $1,$0}' > ${nmondir}/powermt.mpginc
         cat ${nmondir}/powermt.mpginc 2>/dev/null \
            | awk '{if( NF != 0 )print "/dev/rmt/"$1, NF-1}' > ${nmondir}/powermt_paths.mpginc
	    # Added the /dev/rmt so diskinfo would work.

      elif [ `$juname -s` = "Linux" ]  ; then
         # ###############
         # Linux - emcdisk[a-z]
         # ###############
         $jpowermt display dev=all 2>/dev/null \
            | egrep -v "unknown|dead" \
            | sed '/./,/^$/!d' \
            | sed "s/=/ /"  \
            | awk '/^Pseudo/ {printf substr($3,9)" "$3" "} /active/{printf $(NF-6)" "}/^$/' > /tmp/rackdata_powermt.tmp
         while
         read jline
         do
            jpseudo=`echo $jline | awk '{print $1}'`
            jpseudo2=`echo $jline | awk '{print $2}'`
            echo "$jline" \
               | cut -d" " -f3- \
               | tr " " "\n" \
               | sed "s/hdisk/_/g" \
               | sort -t_ -k1n -k2n -k3n -k4n \
               | sed "s/_/hdisk/" \
               | tr "\n" " " \
               | awk -v jpse=$jpseudo -v jpse2=$jpseudo2 '{print jpse, jpse2, $0}'
         done < /tmp/rackdata_powermt.tmp \
            | sort -n \
            | cut -d" " -f2- > ${nmondir}/powermt.mpginc
         cat ${nmondir}/powermt.mpginc 2>/dev/null \
            | awk '{if( NF != 0 )print $1, NF-1}' > ${nmondir}/powermt_paths.mpginc
      elif [ `$juname -s` = "SunOS" ]  ; then
         # ########
         # solaris emcpower#a
         # ########
         $jpowermt display dev=all 2>/dev/null \
            | egrep -v "unknown|dead" \
            | sed '/./,/^$/!d' \
            | sed "s/=/ /"  \
            | awk '/^Pseudo/ {printf substr($3,9)" "$3" "} /active/{printf $3" "}/^$/' > /tmp/rackdata_powermt.tmp
         while
         read jline
         do
            jpseudo=`echo $jline | awk '{print $1}'`
            jpseudo2=`echo $jline | awk '{print $2}'`
            echo "$jline" \
               | cut -d" " -f3- \
               | tr " " "\n" \
               | sed "s/[a-z]/_/g" \
               | sort -t_ -k1n -k2n -k3n -k4n -k5n \
               | sed s,_,c, \
               | sed s,_,t, \
               | sed s,_,d, \
               | sed s,_,s, \
               | tr "\n" " " \
               | awk -v jpse=$jpseudo -v jpse2=$jpseudo2 '{print jpse, jpse2, $0}'
         done < /tmp/rackdata_powermt.tmp \
            | sort -n \
            | cut -d" " -f2- > ${nmondir}/powermt.mpginc
         cat ${nmondir}/powermt.mpginc 2>/dev/null \
            | awk '{if( NF != 0 )print $1, NF-1}' > ${nmondir}/powermt_paths.mpginc

      # For use with Unexpected OS ?? 
      # elif [ `$juname -s` = "NewOS" ]  ; then
      fi
   fi

   # # echo $OStype

   if [ "$OStype" = "HP-UX" ] ; then
      locate_app ioscan
      jioscan=$jloc
      ${jioscan} -funC disk \
         | awk '/rdsk/ {print $1"\n"$2"\n"$3}' \
         | grep rdsk \
         | grep -v "s[0-9]" \
         | while read vRdisk
           do
              print ${vRdisk} 1
           done > ${nmondir}/ioscan_rdsk_paths.mpginc
      # If both powermt and ioscan_rdsk_paths have space, create a uniq list 
      # of all paths from powermt.  Trim ioscan_rdsk_paths to not include items 
      # that are uniq and listed powermt.  powermt_paths will only list 
      # the first path in multipath situations and the path count.
      if [ -s ${nmondir}/powermt.mpginc ] && [ -s ${nmondir}/ioscan_rdsk_paths.mpginc ] ; then
	 # Remove items from ioscan_rdsk_paths that are already in powermt.
	 # This will only leave items that are not listed in the ioscan_rdsk_paths.
	 # This may mean that it is back to a zero byte file.
	 # ioscan_rdsk_paths_hold is the file before it is trimmmed.
	 for jpath in `cat ${nmondir}/powermt.mpginc `
	 do
	    echo $jpath
	 done | sort | uniq > ${nmondir}/powermt_uniq.mpginc
	 mv ${nmondir}/ioscan_rdsk_paths.mpginc ${nmondir}/ioscan_rdsk_paths_hold.mpginc 
         egrep -vw -f ${nmondir}/powermt_uniq.mpginc ${nmondir}/ioscan_rdsk_paths_hold.mpginc > ${nmondir}/ioscan_rdsk_paths.mpginc

         # If there are colons ":" in the data, 
         # the -f option in grep did not work.
         # So redo grep -vw one grep at a time.
         grep ':' ${nmondir}/ioscan_rdsk_paths.mpginc >/dev/null
         greptest=$?
         if [ ${greptest} -eq 0 ]; then
	    # First return ioscan_rdsk_paths_hold to ioscan_rdsk_paths
            cp ${nmondir}/ioscan_rdsk_paths_hold.mpginc ${nmondir}/ioscan_rdsk_paths.mpginc
            cat ${nmondir}/powermt_uniq.mpginc | while
            read jpath
	    do
               cp ${nmondir}/ioscan_rdsk_paths.mpginc ${nmondir}/ioscan_rdsk_paths_hold2.mpginc
               egrep -vw "${jpath}" ${nmondir}/ioscan_rdsk_paths_hold2.mpginc > ${nmondir}/ioscan_rdsk_paths.mpginc
            done
         fi
      fi
   fi
  
   if [ "$OStype" = "Linux" ] ; then
      /sbin/fdisk -l 2>/dev/null \
         | grep "^Disk" \
         | grep dev \
         | sed s/://g| sed s/,//g \
         | awk '{print $2,"1"}' > ${nmondir}/fdisk-l_path.mpginc
   fi
  
   if [ "$OStype" = "AIX" ] ; then
      # for AIX systems only
      # Changes to incorporate mpio
      touch ${nmondir}/lspath.mpginc 
      # lspath 2>/dev/null > ${nmondir}/lspath.mpginc # Original line
      locate_app lspath
      jlspath=$jloc
      if [ "xx" != "x${jlspath}x" ] ; then
         $jlspath 2>/dev/null \
            | awk '{if(substr($2,1,3) != "dac" && substr($2,1,3) != "ses")print $2,$1,$2,$3}' \
            | sed -e 's,^[a-z][a-z]*,,' \
            | sort -n  \
            | cut -d" " -f2-  > ${nmondir}/lspath.mpginc
      fi

      # for AIX systems only
      # Changes to incorporate mpio on non-powerpath drives
      # cat ${nmondir}/lspath.mpginc | awk '{print $2, "1"}' > ${nmondir}/lspath_paths.mpginc # Original line
      # Do not include Available paths that are not configured. 
      egrep -v "Available|Available" ${nmondir}/lspath.mpginc  \
         | awk '{print $2}' \
         | uniq -c \
         | awk '{print $2,$1}' > ${nmondir}/lspath_paths.mpginc 

      # for AIX systems only
      # Changes to incorporate lsdev
      touch ${nmondir}/lsdev.mpginc 
      locate_app lsdev
      jlsdev=$jloc
      if [ "xx" != "x${jlsdev}x" ] ; then
         $jlsdev -C 2>/dev/null > ${nmondir}/lsdev.mpginc
      fi
      # Only include hdisks that are Available 
      egrep "Available|Available" ${nmondir}/lsdev.mpginc  \
         | egrep "hdisk|hdisk" \
         | awk '{print $1}' \
         | uniq -c \
         | awk '{print $2,$1}' > ${nmondir}/lsdev_paths.mpginc
      # If both lspath_paths.mpginc and lsdev_paths.mpginc have space
      # trim lsdev_paths to not include items that are on lspath_paths
      if [ -s ${nmondir}/lsdev_paths.mpginc ] && [ -s ${nmondir}/lspath_paths.mpginc  ] ; then
	 # Remove items from lsdev_paths that are already in lspath_paths.
	 # This will only leave items that are not listed in the lspath_paths. 
	 # This may mean that it is back to a zero byte file.
	 # lsdev_paths_hold is the file before it is trimmmed. 
         mv ${nmondir}/lsdev_paths.mpginc  ${nmondir}/lsdev_paths_hold.mpginc 
         awk '{print $1}' ${nmondir}/lspath_paths.mpginc > ${nmondir}/lspath_disks.mpginc 
	 egrep -vw -f ${nmondir}/lspath_disks.mpginc ${nmondir}/lsdev_paths_hold.mpginc > ${nmondir}/lsdev_paths.mpginc
         if [ -s ${nmondir}/powermt.mpginc ]; then
            for jpath in `cat ${nmondir}/powermt.mpginc`
	    do
	       echo $jpath
            done | sort > ${nmondir}/powermt_uniq.mpginc
            mv ${nmondir}/lsdev_paths.mpginc ${nmondir}/lsdev_paths_hold2.mpginc
            grep -vw -f ${nmondir}/powermt_uniq.mpginc ${nmondir}/lsdev_paths_hold2.mpginc > ${nmondir}/lsdev_paths.mpginc
         fi
      fi
   fi

# This section gathers the disks and determines disk sizes 

# Disk information for gathering size	 
# AIX             bootinfo -s hdisk#  	
# FreeBSD         fdisk -v ad0 	
# HP-UX           diskinfo /dev/rdsk/c#t#d# 	
# Linux/RedHat    cat /proc/scsi/scsi0/sda/model 	
# SunOS           format -d c#t#d#
#                 format>current
#                 format>inquiry 	
# Tru64           file /dev/rrz0c

   # added limit of 1024 to all_paths
   cat ${nmondir}/lspath_paths.mpginc ${nmondir}/lsdev_paths.mpginc ${nmondir}/ioscan_rdsk_paths.mpginc ${nmondir}/powermt_paths.mpginc ${nmondir}/fdisk-l_path.mpginc  2>/dev/null | egrep -v "mapper" | head -1024 > ${nmondir}/all_paths.mpginc 
   i=0
   while
   read d jpaths
   do
      unset jdisktype
      unset jdisksize
      if [ "xx" = "x${jdisktype}x" ] && [ "$OStype" = "AIX" ] ; then
         jdisktype=`lsdev -Cc disk -l $d -F description 2>/dev/null | awk '{print $1}' `
      fi
      if [ "xx" = "x${jdisktype}x" ] && [ "$OStype" = "HP-UX" ] ; then
         jdisktype=`diskinfo -v ${d} 2>/dev/null | egrep "removable|vendor|product" | awk -F: '{print $2}' | sed s,^\ ,, | paste - - - | awk '{if($NF=="no") print $0}' | sed s,\	,\ ,g | sed s,\ \ *,\_,g | sed s,\_no$,,` 
      fi
      # echo "jdisksize=\`getconf DISK_SIZE /dev/${d} 2>/dev/null\`"
      # echo "jdisksize=`getconf DISK_SIZE /dev/${d} 2>/dev/null`"
      # echo "getconf disksize (${jdisksize})"
      jdisksize=`getconf DISK_SIZE /dev/${d} 2>/dev/null`

      if [ "xx" = "x${jdisksize}x" ] ; then
         if [ "$osv" = "5" ] && [ "$osr" -lt "3" ] ; then
            jdisksize=`lsattr -F value -l $d -a size_in_mb 2>/dev/null`
         else
            jdisksize=`lspv $d 2>/dev/null | grep 'TOTAL' | sed s/\(//g | awk '{print $4}' `
         fi
      fi
      if [ "xx" = "x${jdisksize}x" ] && [ "$OStype" = "HP-UX" ] ; then
         jdisksize=`diskinfo -b ${d} 2>/dev/null | awk '{ print $1/1024 }'`
      fi
      if [ "xx" = "x${jdisksize}x" ] && [ "$OStype" = "Linux" ] ; then
         jdisksize=`/sbin/fdisk -l ${d} 2>/dev/null \
            | grep "^Disk" \
            | grep dev \
            | sed s/://g| sed s/,//g \
            | awk '{print $(NF-1)/(1024*1024)}'`    
      fi
      if [ "xx" = "x${jdisktype}x" ] && [ "$OStype" = "Linux" ] ; then
         jdisktype=`/usr/sbin/smartctl -a "${d}" 2>/dev/null \
            | grep Device: \
            | awk -F: '{print $2}' \
            | cut -c2-  \
            | sed s,\ Version,, \
            | sed s,\ \ *,\_,g \
	    | sed s,\_$,,g`
            # echo "(${d}) # jack " 
            # echo "(${jdisktype}) # jack " 
      fi
      if [ "xx" = "x${jdisktype}x" ] ; then
         jdisktype="Unknown"
      fi
      # Added next group to ensure that disks that cannot be read for size
      # are still included.
      if [ "xx" = "x${jdisksize}x" ] ; then
	       jdisksize="Unknown"
      fi
      if [ "xx" != "x${jdisksize}x" ] ; then
         disksize[$i]=$jdisksize
         diskpath[$i]=$jpaths
         diskname[$i]=$d
         disktype[$i]=$jdisktype
         disksizedesc[$i]="Size in MB"
         disp_disksize[$i]=": ${disksizedesc[$i]}: ${disksize[$i]}"
         let i=$i+1
         ## The next line replaces the line above and will set i to 0 if i is not
         ## defined.
         # i=`expr "${i-0}" + 1`
      fi
   done < ${nmondir}/all_paths.mpginc
   nd=$i
   ## report disk info
   echo ; echo "found $nd disk devices :"
   i=0;
   while [ $i -lt $nd ]
   do
      echo " ${diskname[$i]} : type: ${disktype[$i]} ${disp_disksize[$i]} : paths : ${diskpath[$i]}"
      let i=$i+1
      # The next line replaces the line above and will set i to 0 if i is not
      # defined.
      # i=`expr "${i-0}" + 1`
   done
   ### End of disk gathering information
}

get_tl_level () {
   # Query for AIX TL level
   OSLV=`oslevel -r | cut -c1-2`
   if (( OSLV >= 53 ))
   then
      tl=`oslevel -s | cut -c9-10`
      tld="Technology Level"
      printf "%22s = %s\n" "$tld" "$tl"
   fi
}

get_lru_file_repage () {
   # Query for AIX TL level
   lrufr=`vmo -L lru_file_repage 2>/dev/null | grep lru_file_repage | awk '{print $2}'`
   if [ "x${lrufr}x" = "xx" ]
   then
      lrufr="NA"
   fi
   lrufrd="lru_file_repage"
   printf "%22s = %s\n" "$lrufrd" "$lrufr"
}

# get rack config info 

locate_app uname
juname="$jloc"

rac_generic () {
   # generic rack info (should work on any system)
   ##### see about using a parm for width (%40s or %30s or %25s...)

   nn=`$juname -n`
   nnd="Machine Node Name"
   printf "%22s = %s\n" "$nnd" "$nn"

      grep -is open /etc/*release* > /dev/null
      gchk=$?
      if [ $gchk -eq 0 ]
      then
         jopen="Open "
      else 
         jopen=""
      fi

   os=`$juname -s`
   if [ "$os" = "SunOS" ]; then
      case `$juname -r` in
        4*) os="${jopen}SunOS";;
        5*) os="${jopen}Solaris";;
        *) os="${jopen}Unknown";;
      esac
   fi
   osd="Operating System Name"
   printf "%22s = %s\n" "$osd" "$os"

   osv=`$juname -v`
   osvd="OS Version"
   printf "%22s = %s\n" "$osvd" "$osv"

   osr=`$juname -r`
   osrd="OS Release"
   printf "%22s = %s\n" "$osrd" "$osr"
   
   locate_app hostid
   jchk=$?
   if [ ${jchk} -eq 0 ]; then
      # osi=$jloc
      # echo "jloc($jloc)"
      osi=`$jloc`
   else
      osi="NA"
   fi
   osid="hostid"
   if [ "x${osi}x" != "xx" ]; then
      printf "%22s = %s\n" "$osid" "$osi"
   fi

   if [ "$OStype" != "HP-UX" ] ; then
      # uname -p does not work on HP-UX 
      if [ "$OStype" != "AIX" ] ; then
         jproctype=`$juname -p 2>/dev/null`
      else
         jproctype=`lsconf 2>/dev/null | tee /tmp/rackdata_lsconf.tmp | grep "Processor Type:" | awk '{print $NF}'`
      fi
      jtitle="Processor Type"
      if [ "x${jproctype}x" != "xx" ]; then
         printf "%22s = %s\n" "$jtitle" "$jproctype" 
      fi
   fi 
   if [ "$jproctype" != "ppc64" ] && [ `$juname -s` = "Linux" ] && [ "x${jserial}x" = "xx" ] && [ -s "${nmondir}/pntestdmi" ] ; then 
     jserial=`${nmondir}/pntestdmi -s system-serial-number | egrep -iv "dell|dell" 2>/dev/null | sed s,\ ,,g | awk '{print $1}'`
     if [ ! -z "$debug" ] ; then echo "#1 serial=$jserial" >> /tmp/rackdata_debug.tmp ; fi 
   fi
   
   # On some sparc systems, the system-board-serial# is populated.
   if [ "x${jproctype}x" = "xsparcx" ] && [ "x${jserial}x" = "xx" ] ; then 
     jserial=`eeprom | egrep "system-board-serial#=" | awk -F= '{print $2}'`
     if [ ! -z "$debug" ] ; then echo "#2 serial=$jserial" >> /tmp/rackdata_debug.tmp ; fi
   fi
   if [ "x${jproctype}x" = "xsparcx" ] && [ "x${jserial}x" = "xx" ] ; then 
     jserial=${osi}
     if [ ! -z "$debug" ] ; then echo "#3 serial=$jserial" >> /tmp/rackdata_debug.tmp ; fi
   fi
   if [ "$jproctype" = "ppc64" ] && [ `$juname -s` = "Linux" ] && [ "x${jserial}x" = "xx" ]  ; then 
     jserial=`grep -s serial_number /proc/ppc64/lparcfg | awk -F\= '{print $2}' | awk -F\, '{print $2}'`
     if [ ! -z "$debug" ] ; then echo "#4 serial=$jserial" >> /tmp/rackdata_debug.tmp ; fi
   fi 

   if [ `$juname -s` = "AIX" ] && [ "x${jserial}x" = "xx" ] ; then 
     # jserial=`$juname -m` ## This one does not truly give you serial number
     # jserial=`lsconf | grep "Machine Serial Number" | awk '{print $NF}'` ## This one is too slow
     jserial=`uname -u | awk '{print substr($1,length($1)-6)}'` ## This on is quick and valid
     if [ ! -z "$debug" ] ; then echo "#5 serial=$jserial" >> /tmp/rackdata_debug.tmp ; fi
   fi
   if [ `$juname -s` = "HP-UX" ] && [ "x${jserial}x" = "xx" ] ; then 
      locate_app machinfo
      jret=$?
      if [ ${jret} -eq 0 ]; then
         jmachinfo=$jloc   
      fi
      jserial=`$jmachinfo | grep "ine serial" | egrep -iv "available" | awk '{print $NF}'`
      if [ ! -z "$debug" ] ; then echo "#6 serial=$jserial" >> /tmp/rackdata_debug.tmp ; fi
   fi
   if [ "x${jserial}x" = "xx" ]; then
      if [ -s "${nmondir}/pntestdmi" ] ; then 
         jserial=`${nmondir}/pntestdmi -s system-serial-number | egrep -vi "dell" 2>/dev/null | awk '{print $1}'`
      if [ ! -z "$debug" ] ; then echo "#7 serial=$jserial" >> /tmp/rackdata_debug.tmp ; fi
      fi
   fi
   if [ "x${jserial}x" = "xx" ]; then
      if [ -s "/usr/sbin/smbios" ] ; then
         /usr/sbin/smbios 2>/dev/null > ./smbios.test 
         if [ -s ./smbios.test ] ; then
            jserial=`/usr/sbin/smbios -t SMB_TYPE_SYSTEM| grep Serial | awk '{print $NF}'`
         fi
      if [ ! -z "$debug" ] ; then echo "#8 serial=$jserial" >> /tmp/rackdata_debug.tmp ; fi
      fi
   fi
   if [ "x${jserial}x" = "xx" ]; then
      if [ -s "/usr/bin/getsn" ] ; then
         /usr/bin/getsn 2>/dev/null > ./getsn.test 
         if [ -s ./getsn.test ] ; then
            jserial=`/usr/bin/getsn | awk '{print $NF}' | egrep -iv "available"`
            if [ ! -z "$debug" ] ; then echo "#9 serial=$jserial" >> /tmp/rackdata_debug.tmp ; fi
         fi
      fi
   fi
   if [ "x${jserial}x" = "xx" ]; then 
      if [ -s "/usr/bin/getconf" ] ; then
         /usr/bin/getconf MACHINE_SERIAL 2>/dev/null > ./getconf.test 
         if [ -s ./getconf.test ] ; then
            jserial=`/usr/bin/getconf MACHINE_SERIAL | awk '{print $NF}'`
            if [ ! -z "$debug" ] ; then echo "#10 serial=$jserial" >> /tmp/rackdata_debug.tmp ; fi
         fi
      fi
   fi
   if [ "x${jserial}x" = "xx" ]; then
      if [ -s "/usr/sbin/cstm" ] ; then
         ( echo "selclass qualifier system;info;wait;infolog" | /usr/sbin/cstm ) | grep -i "System serial" 2>/dev/null > ./cstm.test 
         if [ -s ./cstm.test ] ; then
            jserial=`cat ./cstm.test | awk '{print $NF}'`
            if [ ! -z "$debug" ] ; then echo "#11 serial=$jserial" >> /tmp/rackdata_debug.tmp ; fi
         fi
      fi
   fi
   if [ "x${jserial}x" = "xx" ]; then
      locate_app ifconfig
      jret=$?
      if [ ${jret} -eq 0 ]; then
         ifc_loc=$jloc   
      fi
   fi
   if [ "x${jserial}x" = "xx" ] && [ "x${osi}x" != "xNAx" ] ; then
      jserial=$osi
   fi
   if [ "x${jserial}x" = "xx" ]; then
      jserial=`$ifc_loc -a 2>/dev/null \
         | grep HWaddr \
         | head -1 \
         | sed -e "s,:,,g" \
         | awk '{print "JQIX-"substr(toupper($NF),5)}'`
      if [ ! -z "$debug" ] ; then echo "#12 serial=$jserial" >> /tmp/rackdata_debug.tmp ; fi
   fi
   if [ "x${jserial}x" = "xx" ]; then
      ifconfig_ip=`$ifc_loc -a  2>/dev/null \
         | grep inet \
         | grep -v 127.0.0.1 \
         | grep "\ [0-9]*\.[0-9]*\.[0-9]*\.[0-9]*\ " \
         | sed -e "s,^.*inet\ ,,g" \
         | sed -e "s,\ .*$,,g" \
         | head -1 \
         | sed -e "s,\.,\ ,g"`
      ifconfig_ip_hex=`for i in $ifconfig_ip
      do
         printf '%02X' "$i"
      done`
      if [ "x${ifconfig_ip_hex}x" != "xx" ]; then
         jserial="JQBE-${ifconfig_ip_hex}"
      if [ ! -z "$debug" ] ; then echo "#13 serial=$jserial" >> /tmp/rackdata_debug.tmp ; fi
      fi
   fi
   if [ "x${jserial}x" = "xx" ]; then
      # Added check for inode number of /usr/local/mpg.
      jserial="JQJO-`ls -id /usr/local/mpg | awk '{print $1}'`"
      if [ ! -z "$debug" ] ; then echo "#14 serial=$jserial" >> /tmp/rackdata_debug.tmp ; fi
   fi
   if [ "x${jserial}x" = "xx" ]; then
      jserial="NoSerial"
      if [ ! -z "$debug" ] ; then echo "#15 serial=$jserial" >> /tmp/rackdata_debug.tmp ; fi
   fi    
   if [ $jserial ] 
   then 
      memd="Serial"
      printf "%22s = %s\n" "$memd" "$jserial"
   fi
}

rac_aix () {
   #  Collect "rack config" on AIX systems
   #  this only works completely on AIX 5.3 and later

   rac_generic
   ##### AIX specific
   get_tl_level
   get_lru_file_repage

   # Grab the output from
   # vmo -L lru_file_repage
   # and vmo -L
   echo "### vmo -L ###"  > ${nmondir}/vmo-L_commands.mpginc
   echo "" >> ${nmondir}/vmo-L_commands.mpginc
   vmo -L 2>/dev/null >> ${nmondir}/vmo-L_commands.mpginc
   echo "" >> ${nmondir}/vmo-L_commands.mpginc
   echo "" >> ${nmondir}/vmo-L_commands.mpginc
   echo "### vmo -L lru_file_repage ###" >> ${nmondir}/vmo-L_commands.mpginc
   echo "" >> ${nmondir}/vmo-L_commands.mpginc
   vmo -L lru_file_repage 2>/dev/null >> ${nmondir}/vmo-L_commands.mpginc

   isa=`$juname -p 2>/dev/null`
   isad="Instruction set architecture"
   printf "%40s = %s\n" "$isad" "$isa"

   mt_mn=`$juname -M`
   mt_mnd="Machine Type and Model Name"
   printf "%40s = %s\n" "$mt_mnd" "$mt_mn"

   pc_mi=`$juname -u`
   pc_mid="Plant Code and Machine ID"
   printf "%40s = %s\n" "$pc_mid" "$pc_mi"

   ## Memory info

   mem=`lsattr -El sys0 -a realmem -F value`
   memd="Amount of usable physical memory in K"
   printf "%40s = %s\n" "$memd" "$mem"

   if (( OSLV >= 53 )); then
   # if [ "$osv" = "5" ] && [ "$osr" -gt "2" ] ; then
      ## LPAR specific (V5R3)
      # lpnn=`$juname -L`  #  Did not work for older versions of AIX 
      lpnn=`lparstat -i | egrep -i "Partition N" |  awk '{print $NF}' | paste - - | awk '{print $2,$1}'`
      lpnnd="LPAR Number and LPAR Name"
      printf "%40s = %s\n" "$lpnnd" "$lpnn"

      lpcap=`lsattr -El sys0 -a capped -F value`
      lpcapd="Partition is capped"
      printf "%40s = %s\n" "$lpcapd" "$lpcap"

      lpded=`lsattr -El sys0 -a dedicated -F value`
      lpdedd="Partition is dedicated"
      printf "%40s = %s\n" "$lpdedd" "$lpded"

      lpentcap=`lsattr -El sys0 -a ent_capacity -F value`
      lpentcapd="Entitled processor capacity"
      printf "%40s = %s\n" "$lpentcapd" "$lpentcap"

      lpmincap=`lsattr -El sys0 -a min_capacity -F value`
      lpmincapd="Minimum potential processor capacity"
      printf "%40s = %s\n" "$lpmincapd" "$lpmincap"

      lpmaxcap=`lsattr -El sys0 -a max_capacity -F value`
      lpmaxcapd="Maximum potential processor capacity"
      printf "%40s = %s\n" "$lpmaxcapd" "$lpmaxcap"
   fi
   
   # Call to disk_info procedure
   disk_info
   # Collect Volume Group Info 
   lsvg 2>/dev/null > /tmp/rackdata_lv_info4aix.tmp
   if [ -s /tmp/rackdata_lv_info4aix.tmp ]; then
      # List volume group information
      echo "# vginfo"
      echo "vgname,vgstate,vgperm,ppsize,totalpp,lvname,lvtype,lp,pp,pv,lvstate,mtpoint,mirrorp,mirror"
      for jvg in `lsvg 2>/dev/null`
      do
         jvginfo=`lsvg ${jvg} 2>/dev/null | egrep "STATE|GROUP|PERMISS" | awk '{print $3,$6}' | paste - - - | awk '{print $1,$3,$5,$4,$6}'`
         lsvg -l ${jvg} 2>/dev/null | tail -n +3 | sort  > /tmp/rackdata_lsvg-1_vg.tmp
         cat /tmp/rackdata_lsvg-1_vg.tmp | awk '{print $1}'  > /tmp/rackdata_lsvg-l_vg_lv.tmp
         lsvg -m ${jvg} 2>/dev/null | tail -n +2 | sort   > /tmp/rackdata_lsvg-m_vg.tmp
	 # The next two lines emulated no lsvg -m volume_group information
	 # rm /tmp/rackdata_lsvg-m_vg.tmp
	 # touch /tmp/rackdata_lsvg-m_vg.tmp
         join -a 1 /tmp/rackdata_lsvg-l_vg_lv.tmp /tmp/rackdata_lsvg-m_vg.tmp | awk '{j1=0}{if($2 != "None" && $2 != "" )j1++1}{if($3 != "None" && $3 != "" )j1++1}{if($4 != "None" && $4 != "" )j1++1}{print $1,"MP"j1}' > /tmp/rackdata_mp_counts.tmp
         join -a 1 /tmp/rackdata_lsvg-1_vg.tmp /tmp/rackdata_mp_counts.tmp | awk -v jvgi="${jvginfo}" '{print jvgi, $0, "M"($4/$3-1)}' > /tmp/rackdata_vginfo.tmp
         cat /tmp/rackdata_vginfo.tmp | sed -e "s|\ |\,|g" 
      done

      # List physical volume information
      echo "# pvinfo"
      echo "vg_name,pv_name,pv_state,total_pps,free_pps"
      for jvg in `lsvg 2>/dev/null`
      do
         lsvg -p ${jvg} 2>/dev/null| tail -n +3 | sed -e "s/\ \ */\ /" | awk  -v jvn="${jvg}" 'BEGIN{OFS=","}{print jvn, $1, $2, $3, $4}' 
      done
      echo "# dfinfo"
   fi

   # This is for vios servers only.
   if [ -s /usr/ios/cli/ioscli ]
   then
      sysname=`uname -n`          # used in default datafile names
      /usr/ios/cli/ioscli lsmap -all  > ioscli_lsmap-all_${sysname}.mpginc
   fi

}


rac_linux () {
   #####
   # linux rack config info
   #####

   ##? uname -i	gets the machine id #  (?equiv AIX serial # ?)
   # vmstat	
   # ? iostat
   # ? hwconf

   rac_generic

   ##### semi-os specific (not on debian)

   #isa=`$juname -p`
   #isad="Instruction set architecture"
   #printf "%30s = %s\n" "$isad" "$isa"

   ##### os specific

   distinfo=`cat /etc/issue | grep -v "^$" | head -1 `
   distinfod="distinfo"
   if [ "x${distinfo}x" != "xx" ] ; then 
      printf "%22s = %s\n" "$distinfod" "$distinfo"
   fi
   osm=`$juname -m`
   osmd="Machine model"
   # check for mainframe model & type
   if [ "$osm" = "s390x" ] ; then 
      MFtype=`cat /proc/sysinfo | grep 'Type' | awk '{print $2}'`
      MFmodel=`cat /proc/sysinfo | grep 'Model:' | awk '{print $2, $3}'`
      printf "%22s = %s %s %s\n" "$osmd" "$osm" "$MFtype" "$MFmodel"
   else
      printf "%22s = %s\n" "$osmd" "$osm"
   fi

   ## Memory info

   mem=`grep MemTotal /proc/meminfo | awk '{print $2}'`
   memd="Amount physical memory"
   printf "%22s = %s\n" "$memd" "$mem"

## Place linux calls hear
### echo "${jproctype}, Remove or comment later." 
if [ "$jproctype" != "ppc64" ] && [ "$jproctype" != "s390" ] && [ "$jproctype" != "s390x" ] ; then
# dmidecode gathered information.
   if [ -s /usr/sbin/dmidecode ] ; then
      if [ -s ${nmondir}/pntestdmi ] ; then
         DMI="${nmondir}/pntestdmi"
      else
         DMI="/usr/sbin/dmidecode"
      fi
   else
      DMI="${nmondir}/pntestdmi"
   fi 
   jtest=`$DMI`
      echo $jtest | grep "sorry" > /dev/null 
   jgreptest=$?
   if [ ${jgreptest} -eq 0 ] 
   then
      echo $jtest
      echo "### This may be an old server ###"
   fi 
   while
   read s_item
   do
      s_details=`$DMI -s $s_item 2>/dev/null |sed -e "s,\ *$,," | egrep -vi "000000000|Unknown"  | sort | uniq `
      if [ "x${s_details}x" != "xx" ] ; then 
         printf "%22s = %s\n" "$s_item" "$s_details" 
      fi
   done <<- pnmpg
system-manufacturer
system-product-name
system-version
system-serial-number
system-uuid
bios-vendor
bios-version
bios-release-date
chassis-type
processor-family
processor-manufacturer
processor-version
processor-frequency
pnmpg
   # Intentionally not gathering the following:
   # baseboard-manufacturer
   # baseboard-product-name
   # baseboard-version
   # baseboard-serial-number
   # baseboard-asset-tag
   # chassis-manufacturer
   # chassis-version
   # chassis-serial-number
   # chassis-asset-tag

   $DMI --type cache 2>/dev/null    \
      | egrep "Socket Designation:|Installed Size:|Location:"    \
      | awk -F\: '{print $2}'  \
      | sed -e "s,^\ ,," \
      | paste - - - \
      | sed -e "s,\	,\ ," \
      | sed -e "s,\	,\ \=\ ," \
      | awk '{print "    ", $0}'

fi   

if [ "x${DMI}x" != "xx" ] ;then
   nmbr_cpus=`$DMI | grep 'Status: Populated' |sed -e "s,\ *$,," | wc -l | awk '{print $1}' `
   CpuSpeed=`$DMI | grep -i speed: | grep -i "current"  | egrep -vi "unknown|\ ns$"  | sed s,^.*:\ ,, | sort | uniq`
   if [ "x${CpuSpeed}x" != "xx" ]
   then
      CpuSpeedd="CPU Speed"
      printf "%22s = %s\n" "$CpuSpeedd" "$CpuSpeed"
   fi

fi

if [ "x${nmbr_cpus}x" = "xx" ] && [ "$jproctype" = "ppc64" ] ; then
   nmbr_cpus=`grep -i "^processor" /proc/cpuinfo | wc -l | awk '{print $1}'`
fi

if [ "x${nmbr_cpus}x" != "xx" ] && [ ${nmbr_cpus} -gt 0 ] ; then 
   printf "%22s = %s\n" "# of CPUs" "$nmbr_cpus" 
else
   printf "%22s = %s\n" "# of CPUs" "1" 
fi


   ## Mainframe LPAR data
   if [ "$osm" = "s390x" ] ; then 
      lpnn=`cat /proc/sysinfo | grep 'LPAR Number:' | awk '{print $3}'`
      lpnnd="LPAR Number"
      printf "%22s = %sb\n" "$lpnnd" "$lpnn"

      lpchar=`cat /proc/sysinfo | grep 'LPAR Characteristics:' | awk '{print $3}'`
      lpchard="LPAR Characteristics"
      printf "%22s = %s\n" "$lpchard" "$lpchar"

      lpvmtot=`cat /proc/sysinfo | grep 'VM.. CPUs Total:' | awk '{print $4}'`
      lpvmtotd="VM Total CPUs"
      printf "%22s = %s\n" "$lpvmtotd" "$lpvmtot"
 
      lpvmcfg=`cat /proc/sysinfo | grep 'VM.. CPUs Configured:' | awk '{print $4}'`
      lpvmcfgd="VM Configured CPUs"
      printf "%22s = %s\n" "$lpvmcfgd" "$lpvmcfg"
   fi
 
   ## get disk info (number, name, and sizes)
   if [ -s /sbin/fdisk ]; then
      jtest=`/sbin/fdisk -l 2>/dev/null`
   else
      jtest='' 
   fi
   if [ "x${jtest}x" != "xx" ] 
   then 
      echo '# -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~'
   /sbin/fdisk -l 2>/dev/null | egrep "Disk" | sed s,Disk\ i,I,g | sed -e "s,Identifier:,Id is,g"  | paste - -  | sed -e "s/	/\,\ /g"
      echo '# -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~'
   fi

#   i=0
#   for d in `/sbin/fdisk -l 2>/dev/null | grep 'Disk' | sed 's/://g' | awk '{print $2}'` 
#   do
#      diskname[$i]=`echo $d | sed 's/\/dev\///g; s/://g'`
#      disksize[$i]=`/sbin/fdisk -l $d 2>/dev/null | grep 'Disk' | sed 's/\/dev\///g; s/://g; s/,//g' | awk '{print $3, $4}'`
#      disksizedesc[$i]='Size'
#      disp_disksize[$i]=" ${disksizedesc[$i]}: ${disksize[$i]}"
#      let i=$i+1
#      The next line replaces the line above and will set i to 0 if i is not
#      defined.
#      i=`expr "${i-0}" + 1`
#      let i=$i+1
#   done
#   nd=$i
#   ## report disk info
#   echo ; echo "found $nd disk devices :"
#   i=0;
#   while [ $i -lt $nd ]
#   do
#      echo "${diskname[$i]}: ${disp_disksize[$i]}"
#      let i=$i+1
#      let i=$i+1
#      The next line replaces the line above and will set i to 0 if i is not
#      defined.
#      i=`expr "${i-0}" + 1`
#   done
   
   # Call to disk_info procedure
   disk_info  
}

rac_hpux () {
   #####
   # hpux rack config info
   #####
   ## uname -i	gets the machine id #  (?equiv AIX serial # ?)
   ## getconf	gets hpux specific stuff - possibly useful ones:
   ##	MACHINE_MODEL	= "model" command
   # vmstat	
   # ? iostat
   # ? hwconf
   rac_generic
   ##### semi-os specific
   ##### os specific
   osm=`model`	## more complete than uname -m
   osmd="Machine model"
   printf "%20s = %s\n" "$osmd" "$osm"

   osi=`$juname -i`
   osid="Machine ID"
   printf "%20s = %s\n" "$osid" "$osi"
  
   if [ "x${jmachinfo}x" != "xx" ] ; then
 
      mem=`$jmachinfo | grep -i memory | sed s,:,\ :, | cut -d" " -f3-4`
      if [ "x${mem}x" != "xx" ]
      then
         memd="Memory"
         printf "%20s = %s\n" "$memd" "$mem"
      fi

      CpuCount=`$jmachinfo | grep "Number of CPUs" | sed s,^.*\=\ ,,`
      if [ "x${CpuCount}x" != "xx" ]
      then
         CpuCountd="# of CPUs"
         printf "%20s = %s\n" "$CpuCountd" "$CpuCount"
      fi

      CpuSpeed=`$jmachinfo | grep "Clock speed" | sed s,^.*\=\ ,,`
      if [ "x${CpuSpeed}x" != "xx" ]
      then
         CpuSpeedd="CPU Speed"
         printf "%20s = %s\n" "$CpuSpeedd" "$CpuSpeed"
      fi

      CpuInfo=`$jmachinfo | grep "processor" | sed s,^\ *,,`
      if [ "x${CpuInfo}x" != "xx" ]
      then
         CpuInfod="CPU Info"
         # Removed print of CPU Info, but segmented instead
         # printf "%20s = %s\n" "$CpuInfod" "$CpuInfo"
         CpuCountd="CPU Count"
         CpuCount=`echo $CpuInfo |awk '{print $1}'`
         if [ "x${CpuCount}x" != "xprocessorx" ]; then
            printf "%20s = %s\n" "$CpuCountd" "$CpuCount"
            CpuTyped="CPU Type"
            CpuType=`echo $CpuInfo \
               | grep proc \
               | grep ')$' \
               | sed s,\ \(.*,,g \
               | sed s,^\ \*,,g  | cut -d" " -f2-`
            printf "%20s = %s\n" "$CpuTyped" "$CpuType"
            CpuSpeedd="CPU Speed"
            CpuSpeed=`echo $CpuInfo | grep proc | grep ')$' | sed s/\,.*//g | sed s/^.*\(// | awk '{if($2 == "GHz") {print $1 * 1000, "MHZ"} else {print $0}}'`
            printf "%20s = %s\n" "$CpuSpeedd" "$CpuSpeed"
         fi
      fi

      CpuType=`$jmachinfo | grep "processor model:" \
                     | sed s,\ \ *,\ ,g  | sed s,^\ ,, \
                     | cut -d" " -f4-`
      if [ "x${CpuType}x" != "xx" ]
      then
         CpuTyped="CPU Type"
         printf "%20s = %s\n" "$CpuTyped" "$CpuType"
      fi

      #
      # Cores per socket = 2
      # L3 Cache (per core) = 9216 KB
      #

      L3Cache=`$jmachinfo -v \
         | grep L3 \
         | sed s/\,.*// \
         | awk '{print $(NF-1), $NF}'`
      if [ "x${L3Cache}x" != "xx" ]
      then
         PerCore=`$jmachinfo -v \
                      | grep -i "(per core)" \
                      | sed s/\,.*// \
                      | awk '{print $(NF-1), $NF}'`
         # CoresPerSocket if not specified it will be one, unless there is 
	 # a per core statement it will be two.
	 CoresPerSocket=`$jmachinfo -v \
            | grep -i "Cores per socket" \
            | sed s/\,.*// \
            | awk '{print $(NF-1), $NF}'`

         if [ "x${CoresPerSocket}x" != "xx" ]
         then
            CoresPerSocketd="Cores per socket"
	 else
            CoresPerSocketd="Cores per socket"
            CoresPerSocket="1"
         fi
         if [ "x${PerCore}x" != "xx" ]
         then
            L3Cached="L3 Cache (per core)"
            CoresPerSocket="2"
         else
            L3Cached="L3 Cache"
         fi
         printf "%20s = %s\n" "$CoresPerSocketd" "$CoresPerSocket"
         printf "%20s = %s\n" "$L3Cached" "$L3Cache"
      fi

   fi

   # Added in for processor counts on older HP-UX machines that do not have machinfo
   # ioscan -fnk 2>/dev/null |grep -i Processor |wc -l 
   if [ "x${CpuCount}x" = "xx" ]
   then
      CpuCount=`ioscan -fnk 2>/dev/null |grep -i Processor |wc -l | awk '{print $1}'`
      CpuCountd="# of CPUs"
      if [ "${CpuCount}" -eq 0 ]
      then
         CpuCount='?'
      fi
      printf "%20s = %s\n" "$CpuCountd" "$CpuCount"
   fi
  
   # Added in for Memory check on older HP-UX machines that do not have machinfo
   if [ "x${mem}x" = "xx" ]; then
      mem=`/opt/ignite/bin/print_manifest 2>/dev/null |grep -i memory| awk '{print $(NF-1),$NF}'`
      if [ "x${mem}x" != "xx" ]; then
         memd="Memory"
         printf "%20s = %s\n" "$memd" "$mem"
      else
         printf "%20s = %s\n" "$memd" "Unknown"
      fi
   fi
   
   locate_app hpvminfo
   jchk=$?
   if [ $jchk -eq 0 ] ; then 
      HostSerial=`$jloc -V | grep "Host serial number" | awk '{print $NF}'`
      if [ "x${HostSerial}x" != "xx" ]
      then
         HostSeriald="Host serial"
         printf "%20s = %s\n" "$HostSeriald" "$HostSerial"
      fi
      HostServer=`$jloc -V | grep "Host model string" | awk '{print $NF}'`
      if [ "x${HostServer}x" != "xx" ]
      then
         HostServerd="Host server model"
         printf "%20s = %s\n" "$HostServerd" "$HostServer"
      fi
   fi

   
   # Call to disk_info procedure
   disk_info  
}


rac_solaris () {
   #####
   # solaris specific rack config
   #####

   rac_generic

   platform=`$juname -i 2> /dev/null`
   jprtdiag=/usr/platform/`uname -i`/sbin/prtdiag
   jsolaris_sbin=/usr/platform/`uname -i`/sbin/

   osc="`$jprtdiag 2>/dev/null | sed -e "s,\ \ *,\ ,g" | awk -F: '/System Configuration/ {print $2}'`"
   if [ "x${osc}x" != "xx" ] 
   then 
      oscd="System Configuration"
      printf "%23s =%s\n" "$oscd" "$osc"
   fi

   # sysclock=`$jprtdiag 2>/dev/null | awk -F: '/System clock frequency/ {print $2}'`
   # if [ "x${sysclock}x" != "xx" ] 
   # then 
   #    sysclockd="System clock frequency"
   #    printf "%23s =%s\n" "$sysclockd" "$sysclock"
   # fi

   # jclock=`psrinfo -vp \
   jclock=`psrinfo -v \
      | grep -i hz\
      | sed -e "s,The\ ,,g" \
      | sed -e "s/,//g" \
      | sed -e "s,(,,g" \
      | sed -e "s,),,g" | awk '{print $(NF-1),$NF}' | uniq`  
   if [ "x${jclock}x" != "xx" ] 
   then 
      sysclockd="Processor frequency"
      printf "%23s = %s\n" "$sysclockd" "$jclock"
   fi

   mem=`$jprtdiag 2>/dev/null |awk -F: '/Memory size/ {print $2}'`
   if [ "x${mem}x" != "xx" ] 
   then 
      memd="Memory size"
      printf "%23s =%s\n" "$memd" "$mem"
   else  
      # Alternative way to get memory size if first does not work
      ###  prtconf | grep -v "^$"| awk '{print "prtconf.out"NR" =",$0}'
      prtconf -vp 2>/dev/null  | egrep "^Memory size:" \
         | sed -e "s,^\ *,,"  \
         | sed -e "s,\ \ *,\ ,g" \
         | sed -e "s,:,\ =," \
         | sed -e "s,\',,g"  \
         | awk '{jeq=index($0,"=")}{j1st=substr($0,1,jeq-2)}{j2nd=substr($0,jeq+2)}{printf("%23s = %s\n", j1st, j2nd)}'
   fi

   ###  prtconf | grep -v "^$"| awk '{print "prtconf.out"NR" =",$0}'
   prtconf -vp 2>/dev/null  | egrep "^    name:|^    model:" \
      | sed -e "s,^\ *,,"  \
      | sed -e "s,\ \ *,\ ,g" \
      | sed -e "s,:,\ =," \
      | sed -e "s,\',,g"  \
      | awk '{jeq=index($0,"=")}{j1st=substr($0,1,jeq-2)}{j2nd=substr($0,jeq+2)}{printf("%23s = %s\n", j1st, j2nd)}'


   $juname -X | egrep "Machine|NumCPU" \
      | sed -e "s,^\ *,,"  \
      | sed -e "s,\ \ *,\ ,g" \
      | sed -e "s,:,\ =," \
      | sed -e "s,\',,g"  \
      | awk '{jeq=index($0,"=")}{j1st=substr($0,1,jeq-2)}{j2nd=substr($0,jeq+2)}{printf("%23s = %s\n", j1st, j2nd)}'

   if [ -s /usr/sbin/smbios ] ; then
      /usr/sbin/smbios 2>/dev/null > ./smbios.test 
      if [ -s ./smbios.test ] ; then
         /usr/sbin/smbios -t SMB_TYPE_SYSTEM \
            | grep : \
            | egrep "Manu|Prod|Vers|Seri|UUID" \
            | sed -e "s,:,\ =," \
            | sed -e "s,\ \ *,\ ,g"\
            | sed -e "s,^,System,g"\
            | awk '{jeq=index($0,"=")}{j1st=substr($0,1,jeq-2)}{j2nd=substr($0,jeq+2)}{printf("%23s = %s\n", j1st, j2nd)}'

         /usr/sbin/smbios -t SMB_TYPE_BIOS \
            | grep : \
            | egrep "Vend|Vers|Rele" \
            | sed -e "s,:,\ =," \
            | sed -e "s,\ \ *,\ ,g"\
            | sed -e "s,^,Bios,g"\
            | awk '{jeq=index($0,"=")}{j1st=substr($0,1,jeq-2)}{j2nd=substr($0,jeq+2)}{printf("%23s = %s\n", j1st, j2nd)}'

         /usr/sbin/smbios -t SMB_TYPE_CHASSIS \
            | grep : \
            | egrep "Type" \
            | sed -e "s,:,\ =," \
            | sed -e "s,\ \ *,\ ,g"\
            | sed -e "s,^\ ,,g"\
            | awk '{jeq=index($0,"=")}{j1st=substr($0,1,jeq-2)}{j2nd=substr($0,jeq+2)}{printf("%23s = %s\n", j1st, j2nd)}'

         /usr/sbin/smbios -t SMB_TYPE_PROCESSOR \
            | grep : \
            | egrep "Manu|Vers|Curr" \
            | sed -e "s,:,\ =," \
            | sed -e "s,\ \ *,\ ,g"\
            | sed -e "s,^,Processor,g"\
            | awk '{jeq=index($0,"=")}{j1st=substr($0,1,jeq-2)}{j2nd=substr($0,jeq+2)}{printf("%23s = %s\n", j1st, j2nd)}'

         /usr/sbin/smbios -t SMB_TYPE_MEMARRAYMAP \
            | grep : \
            | grep Size \
            | sed -e "s,:,\ =," \
            | sed -e "s,\ \ *,\ ,g"\
            | sed -e "s,^,Total Memory,g"\
            | awk '{jeq=index($0,"=")}{j1st=substr($0,1,jeq-2)}{j2nd=substr($0,jeq+2)}{printf("%23s = %s\n", j1st, j2nd)}'

         /usr/sbin/smbios -t SMB_TYPE_MEMDEVICE \
            | grep : \
            | egrep "Location|Size|Type" \
            | sed -e "s,:,\ =," \
            | sed -e "s,\ \ *,\ ,g"\
            | sed -e "s,^,MemCard,g"\
            | awk '{jeq=index($0,"=")}{j1st=substr($0,1,jeq-2)}{j2nd=substr($0,jeq+2)}{printf("%23s = %s\n", j1st, j2nd)}'
      fi
   fi
   # Added blank line to be consistent with AIX output
   echo "" 
   # ====================
   # This will work for Solaris 8,9,10, and 11
   # ====================
   iostat -E \
      | egrep "Model|Vendor|Size|Soft" \
      | sed s,\ Size:,\
Size:, \
      | sed s,Revision.*Serial\ No:,, \
      | sed s,\<.*\>,, \
      | sed s,Soft.*$,, | paste - - -\
      | sed s,\	,\ ,g \
      | sed -e 's,\ [A-Z][a-z]*:\ ,,g' \
      | sed s,\ \ *,\ ,g \
      | egrep -v "0.00GB|[0-9]XCD|\ st[0-9]|\ JLMS\ " \
      | awk '{print "Device",NR,$0}' | sed s,GB\ $,\ GB, > ./disktest.$sysname.tmp
   #   | awk '{print $1,$2,$3,$(NF-2)}'
   jdiskline=` wc -l ./disktest.$sysname.tmp | awk '{print $1}'`
   echo "found $jdiskline disk devices :"
   cat ./disktest.$sysname.tmp
   rm -f ./disktest.$sysname.tmp

   cat *.err 2>/dev/null | awk '{print "# Error",$0}'
   rm -f *.err *.test
   # jtest=`psrinfo -vp` 
   jtest=`psrinfo -v` 
   if [ "x${jtest}x" != "xx" ] 
   then 
      echo '# -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~'
      # psrinfo -vp | awk '{print "#",$0}'
      psrinfo -v | awk '{print "#",$0}'
      echo '# -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~'
   fi
   which zfs | grep zfs$
   jchk=$?
   if [ ${jchk} -eq 0 ]; then
      # jtest=`zfs list`
      jtest=`zfs list 2>/dev/null`
      if [ "x${jtest}x" != "xx" ]
      then
         echo '# -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~'
         echo '# zfs list'
         # zfs list | awk '{print "#",$0}'
         zfs list | awk '{print "#",$0}'
         echo '# -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~'
      fi
      # jtest=`zpool list`
      jtest=`zpool list 2>/dev/null`
      if [ "x${jtest}x" != "xx" ]
      then
         echo '# -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~'
         echo '# zpool list'
         # zpool list | awk '{print "#",$0}'
         zpool list | awk '{print "#",$0}'
         echo '# -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~'
         # jtest=`zpool list | egrep -vi "NAME|no pool" | awk '{print $1}'`
         jtest=`zpool list | egrep -vi "NAME|no pool" | awk '{print $1}' 2>/dev/null`
         if [ "x${jtest}x" != "xx" ]
         then
            for jpool in $jtest
            do
            echo '# -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~'
               echo "# zpool status $jpool"
               zpool status $jpool | awk '{print "#",$0}'
            echo '# -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~'
            done
         fi
      fi
   fi
   
   # Call to disk_info procedure
   disk_info  
}

rac_sco () {
   #####
   # sco specific rack config
   #####

   rac_generic

   platform=`$juname -i 2> /dev/null`
   jprtdiag=/usr/platform/`uname -i`/sbin/prtdiag
   jsolaris_sbin=/usr/platform/`uname -i`/sbin/

   osc="`$jprtdiag 2>/dev/null | sed -e "s,\ \ *,\ ,g" | awk -F: '/System Configuration/ {print $2}'`"
   if [ "x${osc}x" != "xx" ] 
   then 
      oscd="System Configuration"
      printf "%23s =%s\n" "$oscd" "$osc"
   fi

   jclock=`psrinfo -v | tee psrinfo-v.mpginc \
      | grep -i hz\
      | sed -e "s,The\ ,,g" \
      | sed -e "s/,//g" \
      | sed -e "s,(,,g" \
      | sed -e "s,),,g" | awk '{print $(NF-1),$NF}' | uniq`  
   if [ "x${jclock}x" != "xx" ] 
   then 
      sysclockd="Processor frequency"
      printf "%23s = %s\n" "$sysclockd" "$jclock"
   fi

   mem=`$jprtdiag 2>/dev/null |awk -F: '/Memory size/ {print $2}'`
   if [ "x${mem}x" != "xx" ] 
   then 
      memd="Memory size"
      printf "%23s =%s\n" "$memd" "$mem"
   else  
      # Alternative way to get memory size if first does not work
      ###  prtconf | grep -v "^$"| awk '{print "prtconf.out"NR" =",$0}'
      prtconf -vp 2>/dev/null  | egrep "^Memory size:" \
         | sed -e "s,^\ *,,"  \
         | sed -e "s,\ \ *,\ ,g" \
         | sed -e "s,:,\ =," \
         | sed -e "s,\',,g"  \
         | awk '{jeq=index($0,"=")}{j1st=substr($0,1,jeq-2)}{j2nd=substr($0,jeq+2)}{printf("%23s = %s\n", j1st, j2nd)}'
   fi

   prtconf -vp 2>/dev/null  | egrep "^    name:|^    model:" \
      | sed -e "s,^\ *,,"  \
      | sed -e "s,\ \ *,\ ,g" \
      | sed -e "s,:,\ =," \
      | sed -e "s,\',,g"  \
      | awk '{jeq=index($0,"=")}{j1st=substr($0,1,jeq-2)}{j2nd=substr($0,jeq+2)}{printf("%23s = %s\n", j1st, j2nd)}'


   $juname -X | egrep "Machine|NumCPU" \
      | sed -e "s,^\ *,,"  \
      | sed -e "s,\ \ *,\ ,g" \
      | sed -e "s,:,\ =," \
      | sed -e "s,\',,g"  \
      | awk '{jeq=index($0,"=")}{j1st=substr($0,1,jeq-2)}{j2nd=substr($0,jeq+2)}{printf("%23s = %s\n", j1st, j2nd)}'

   if [ -s /usr/sbin/smbios ] ; then
      /usr/sbin/smbios 2>/dev/null > ./smbios.test 
      if [ -s ./smbios.test ] ; then
         /usr/sbin/smbios -t SMB_TYPE_SYSTEM \
            | grep : \
            | egrep "Manu|Prod|Vers|Seri|UUID" \
            | sed -e "s,:,\ =," \
            | sed -e "s,\ \ *,\ ,g"\
            | sed -e "s,^,System,g"\
            | awk '{jeq=index($0,"=")}{j1st=substr($0,1,jeq-2)}{j2nd=substr($0,jeq+2)}{printf("%23s = %s\n", j1st, j2nd)}'

         /usr/sbin/smbios -t SMB_TYPE_BIOS \
            | grep : \
            | egrep "Vend|Vers|Rele" \
            | sed -e "s,:,\ =," \
            | sed -e "s,\ \ *,\ ,g"\
            | sed -e "s,^,Bios,g"\
            | awk '{jeq=index($0,"=")}{j1st=substr($0,1,jeq-2)}{j2nd=substr($0,jeq+2)}{printf("%23s = %s\n", j1st, j2nd)}'

         /usr/sbin/smbios -t SMB_TYPE_CHASSIS \
            | grep : \
            | egrep "Type" \
            | sed -e "s,:,\ =," \
            | sed -e "s,\ \ *,\ ,g"\
            | sed -e "s,^\ ,,g"\
            | awk '{jeq=index($0,"=")}{j1st=substr($0,1,jeq-2)}{j2nd=substr($0,jeq+2)}{printf("%23s = %s\n", j1st, j2nd)}'

         /usr/sbin/smbios -t SMB_TYPE_PROCESSOR \
            | grep : \
            | egrep "Manu|Vers|Curr" \
            | sed -e "s,:,\ =," \
            | sed -e "s,\ \ *,\ ,g"\
            | sed -e "s,^,Processor,g"\
            | awk '{jeq=index($0,"=")}{j1st=substr($0,1,jeq-2)}{j2nd=substr($0,jeq+2)}{printf("%23s = %s\n", j1st, j2nd)}'

         /usr/sbin/smbios -t SMB_TYPE_MEMARRAYMAP \
            | grep : \
            | grep Size \
            | sed -e "s,:,\ =," \
            | sed -e "s,\ \ *,\ ,g"\
            | sed -e "s,^,Total Memory,g"\
            | awk '{jeq=index($0,"=")}{j1st=substr($0,1,jeq-2)}{j2nd=substr($0,jeq+2)}{printf("%23s = %s\n", j1st, j2nd)}'

         /usr/sbin/smbios -t SMB_TYPE_MEMDEVICE \
            | grep : \
            | egrep "Location|Size|Type" \
            | sed -e "s,:,\ =," \
            | sed -e "s,\ \ *,\ ,g"\
            | sed -e "s,^,MemCard,g"\
            | awk '{jeq=index($0,"=")}{j1st=substr($0,1,jeq-2)}{j2nd=substr($0,jeq+2)}{printf("%23s = %s\n", j1st, j2nd)}'
      fi
   fi
   # Added blank line to be consistent with AIX output
   echo "" 
   # ====================
   # This will work for Solaris 8,9,10, and 11
   # ====================
   iostat -E \
      | egrep "Model|Vendor|Size|Soft" \
      | sed s,\ Size:,\
Size:, \
      | sed s,Revision.*Serial\ No:,, \
      | sed s,\<.*\>,, \
      | sed s,Soft.*$,, | paste - - -\
      | sed s,\	,\ ,g \
      | sed -e 's,\ [A-Z][a-z]*:\ ,,g' \
      | sed s,\ \ *,\ ,g \
      | egrep -v "0.00GB|[0-9]XCD|\ st[0-9]|\ JLMS\ " \
      | awk '{print "Device",NR,$0}' | sed s,GB\ $,\ GB, > ./disktest.$sysname.tmp
   #   | awk '{print $1,$2,$3,$(NF-2)}'
   jdiskline=` wc -l ./disktest.$sysname.tmp | awk '{print $1}'`
   echo "found $jdiskline disk devices :"
   cat ./disktest.$sysname.tmp
   rm -f ./disktest.$sysname.tmp

   cat *.err 2>/dev/null | awk '{print "# Error",$0}'
   rm -f *.err *.test
   # jtest=`psrinfo -vp` 
   jtest=`psrinfo -v` 
   if [ "x${jtest}x" != "xx" ] 
   then 
      echo '# -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~'
      # psrinfo -vp | awk '{print "#",$0}'
      psrinfo -v | awk '{print "#",$0}'
      echo '# -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~'
   fi
   which zfs | grep zfs$
   jchk=$?
   if [ ${jchk} -eq 0 ]; then
      # jtest=`zfs list`
      jtest=`zfs list 2>/dev/null`
      if [ "x${jtest}x" != "xx" ]
      then
         echo '# -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~'
         echo '# zfs list'
         # zfs list | awk '{print "#",$0}'
         zfs list | awk '{print "#",$0}'
         echo '# -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~'
      fi
      # jtest=`zpool list`
      jtest=`zpool list 2>/dev/null`
      if [ "x${jtest}x" != "xx" ]
      then
         echo '# -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~'
         echo '# zpool list'
         # zpool list | awk '{print "#",$0}'
         zpool list | awk '{print "#",$0}'
         echo '# -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~'
         # jtest=`zpool list | egrep -vi "NAME|no pool" | awk '{print $1}'`
         jtest=`zpool list | egrep -vi "NAME|no pool" | awk '{print $1}' 2>/dev/null`
         if [ "x${jtest}x" != "xx" ]
         then
            for jpool in $jtest
            do
            echo '# -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~'
               echo "# zpool status $jpool"
               zpool status $jpool | awk '{print "#",$0}'
            echo '# -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~'
            done
         fi
      fi
   fi
   
   # Call to disk_info procedure
   disk_info  
}

df_data () {
   # generate historical file of DF data for  any OS)
   # printf "file system info: $dateformat\n"
   # 
   # df -Pk works for AIX.  The -P ensures the data is on one line 
   if [ "$OStype" = "AIX" ] ; then
      df -Pk \
         | sed s,Mounted\ on,Mounted_on,g \
         | sed s,\ \ *,\ ,g \
         | sed s/\ /,/g \
         | awk '{print jdate","$0}' jdate=$jdate \
         | egrep -v ",swap,|\/opt\/|,0,0,0,0|,-,-,-,-|\/media\/|/dev/fd|\/etc/mnttab|\/proc|\/cdrom|\/dev\/cd[0-9]"
   # 
   # df -Pk works on most versions of Linux, and ensures data is on one line. 
   # However, some versions of Linux do not have the -P option.  In that case,
   # the sript will use df -k.
   elif [ "$OStype" = "Linux" ] ; then
      jtest=`df -Pk`
      if [ "x${jtest}x" != "xx" ] ; then
         df -Pk \
            | sed s,Mounted\ on,Mounted_on,g \
            | sed s,\ \ *,\ ,g \
            | sed s/\ /,/g \
            | awk '{print jdate","$0}' jdate=$jdate \
            | egrep -v "\/platform\/|,swap,|\/opt\/|,0,0,0,0|,-,-,-,-|\/media\/|/dev/fd|\/etc/mnttab|\/proc|\/cdrom|\/dev\/cd[0-9]"
      else
         jtest=`df -k`
         if [ "x${jtest}x" != "xx" ] ; then
            df -k \
               | sed s,Mounted\ on,Mounted_on,g \
               | sed s,\ \ *,\ ,g \
               | sed s/\ /,/g \
               | awk '{print jdate","$0}' jdate=$jdate \
               | egrep -v "\/platform\/|,swap,|\/opt\/|,0,0,0,0|,-,-,-,-|\/media\/|/dev/fd|\/etc/mnttab|\/proc|\/cdrom|\/dev\/cd[0-9]"
         else
            echo "*** Error *** The command (df -k) did not get an output."
         fi
      fi
   # 
   # bdf appears to work for HP-UX, the awk line corrects for breaks in data.   
   elif [ "$OStype" = "HP-UX" ] ; then
      bdf \
         | awk '{if (NF==1) {line=$0;getline;sub(" *"," ");print line$0} else {print}}' \
         | sed s,Mounted\ on,Mounted_on,g \
         | sed s,\ \ *,\ ,g \
         | sed s/\ /,/g \
         | awk '{print jdate","$0}' jdate=$jdate \
         | egrep -v "\/platform\/|,swap,|\/opt\/|,0,0,0,0|,-,-,-,-|\/media\/|/dev/fd|\/etc/mnttab|\/proc|\/cdrom|\/dev\/cd[0-9]"
   # 
   # df -k works Solaris.  The data is on one line. 
   elif [ "$OStype" = "SunOS" ] ; then
      df -k \
         | sed s,Mounted\ on,Mounted_on,g \
         | sed s,\ \ *,\ ,g \
         | sed s/\ /,/g \
         | awk '{print jdate","$0}' jdate=$jdate \
         | egrep -v "\/platform\/|,swap,|\/opt\/|,0,0,0,0|,-,-,-,-|\/media\/|/dev/fd|\/etc/mnttab|\/proc|\/cdrom|\/dev\/cd[0-9]"
   elif [ "$OStype" = "SCO_SV" ] ; then
      df -k \
         | sed s,Mounted\ on,Mounted_on,g \
         | sed s,\ \ *,\ ,g \
         | sed s/\ /,/g \
         | awk '{print jdate","$0}' jdate=$jdate \
         | egrep -v "\/platform\/|,swap,|\/opt\/|,0,0,0,0|,-,-,-,-|\/media\/|/dev/fd|\/etc/mnttab|\/proc|\/cdrom|\/dev\/cd[0-9]"
   else
      # 
      # This is for systems unable to test.  IE Irix? 
      printf "*** Error *** file system info: ($OStype) is unknown OS\n"
   fi
}

### utility functions and default values :
invocdir=`dirname $0`	    #doesn't work with linux - need path as parm??
sysname=`$juname -n`          # used in default nmon datafile names

usage () {
   echo "USAGE: $0 [options]
   where options may be any of:
   -D                  debug
   -v                  verbose
   -c <configfile>     specify configfile (full path)
   "
}

while getopts "Dvc:" opt; do
  case $opt in
        D)	debug="T" ;;
	v)	verbose="T" ;;
	c)	opt_configfile=$OPTARG ;;
	?)	usage;  exit ;;
  esac
done

shift `expr $OPTIND - 1`

if [ ! -z "$opt_configfile" ] ; then configfile=$opt_configfile; fi
##### quick+dirty configfile - unsafe!
if [ ! -z "$configfile" ] ; then
   . $configfile
fi

if [ ! -d "$nmondir" ] ; then
   if [ ! -z "$datafile" ] ; then
      nmondir=`dirname $datafile` 
   else
      nmondir=$invocdir 
   fi
fi

if [ ! -z "$debug" ] ; then echo "configfile=$configfile" >> /tmp/rackdata_debug.tmp ; fi
if [ ! -z "$debug" ] ; then echo "sysname=$sysname" >> /tmp/rackdata_debug.tmp ; fi

outfile="${nmondir}/rackdata.$sysname"
touch $outfile

if [ ! -z "$debug" ] ; then echo "outfile=$outfile" >> /tmp/rackdata_debug.tmp ; fi

echo "rackdata for $sysname" > $outfile
grep "\$Id\:" $0 | awk '{print $1,$3,$4,$5,$6}' >> $outfile
# Added in to specify version of nmon in use
mpgnmonfile=`ls -al nmon 2>/dev/null | awk '{print $NF}'` 
if [ -s /usr/bin/topas_nmon ]
then
   echo "#" >> $outfile
   strings -a /usr/bin/topas_nmon  | grep 'topas\.c,'  | awk '{print "# Using /usr/bin/topas/nmon Version",$2,$(NF-1),$NF}' >> $outfile
   echo "#" >> $outfile
else
   find nmon -type l 2>/dev/null  >/dev/null
   jchk=$?
   if [ $jchk -eq 0 ]
   then
      echo "#" >> $outfile
      echo $mpgnmonfile `strings -a $mpgnmonfile | grep ^nmon\ [0-9]` | awk '{if (NF == 3)print "# Using",$1,"Version",$3}' >> $outfile
      echo "#" >> $outfile
   else
      OStype=`$juname`
      if [ "$OStype" != "Linux" ] && [ "$OStype" != "AIX" ] ; then
         echo "#" >> $outfile
         echo "# Using sar to collect performance data on $OStype" >> $outfile
         echo "#" >> $outfile
      fi  
   fi
fi 

OStype=`$juname`
if [ ! -z "$debug" ] ; then echo "OStype=$OStype" >> /tmp/rackdata_debug.tmp ; fi
if [ "$OStype" = "AIX" ] ; then
   rac_aix >> $outfile 
elif [ "$OStype" = "Linux" ] ; then
   rac_linux >> $outfile 
elif [ "$OStype" = "HP-UX" ] ; then
   rac_hpux >> $outfile 
elif [ "$OStype" = "SunOS" ] ; then
   rac_solaris >> $outfile 
elif [ "$OStype" = "SCO_SV" ] ; then
   rac_sco >> $outfile 
else
   echo "unknown system type : $OStype" >> $outfile 
   rac_generic >> $outfile 
fi

dffile="${nmondir}/df.$sysname"

#& rackdata history (rdhist.hostname) documents any changes that occur
#& in the top part of the rackdata.hostname file.
#&% rdhist.hold is used to retain last rackdata output for comparison.
#&% rdhist.tmp is a temporary file

rdhistfile="${nmondir}/rdhist.$sysname"
rdhisthold="${nmondir}/rdhist.hold"
rdhisttemp="${nmondir}/rdhist.tmp"

cat $outfile | grep -v "^#" > $rdhisttemp

if [ ! -s $rdhistfile ] ; then
   touch $rdhisthold 
   touch $rdhistfile 
fi

if [ -s $rdhisthold ]; then
  cmp -s $rdhisttemp $rdhisthold 2>&1 > /dev/null
  jcmp=$?
else
  jcmp=1
fi

if [ $jcmp -ne 0 ] ;then
   cat $outfile | awk '{print JDate", "$0}' JDate=$jdate >> $rdhistfile 
   cat $outfile | grep -v "^#" > $rdhisthold
fi

dffile="${nmondir}/df.$sysname"

if [ ! -z "$debug" ] ; then 
   echo "df file=$dffile" >> /tmp/rackdata_debug.tmp 
   cat /tmp/rackdata_debug.tmp 
   cp /tmp/rackdata_debug.tmp rackdata_debug.tmp
   chown root:system *
   tar -cvf ${outfile}.tar $outfile *.mpginc rackdata_debug.tmp 2>&1 > /dev/null
   gzip -f ${outfile}.tar  
   chown root:system *
   echo "Debug file is ${outfile}.tar.gz" 
else
   rm -f /tmp/rackdata_debug.tmp
   rm -f rackdata_debug.tmp
   rm -f ${outfile}.tar.gz 
fi

df_data >> $dffile
cat $dffile >> $outfile

# If pnadcp.sh exists, next check if running 
if [ -s ./pnadcp.sh ]; then
   # If pnadcp.sh is not already running start it.
   ps -ef | grep pnadcp.sh | grep -v grep  > /tmp/pnadcp_running.tmp
   # If /tmp/pnadcp_running.tmp is empty, start pnadcp.sh in background
   if [ ! -s /tmp/pnadcp_running.tmp ]; then
      ./pnadcp.sh & 
   fi
fi


