#!/bin/ksh
#
# $Id: transfer_current.sh,v 1.3 2012/02/14 04:33:34 root Exp root $
#
# The following code is Confidential and is covered by the installation license
# Copyright 2004-2012 by Midrange Performance Group
#
#&& This transfer script can aid in gathering only the most current update of 
#&& nmon/sar data to a common location.
#&& The config files that are used for the full data transfer script 
#&& (transfer_script.sh) can be use with this script with no changes.
#
#& Example configuration files are included.
#
#&% To execute this script, substitute a config file depending on the type of transfer
#&% ./transfer_current.sh -c ./transfer_scp.config 
#&% ./transfer_current.sh -c ./transfer_rcp.config 
#&% ./transfer_current.sh -c ./transfer_ftp.config 
#&% ./transfer_current.sh -c ./transfer_cp.config 
#&% These config files need to be modified to the specific needs of the customer
#
#&@ Requires user modified configuration file for normal operation.
#

function usage {
echo "usage: $0 [options]
  where options may be any of:
	-c configfile (or options can be included manually on the command line)
	-m transfer_method (Choices are ftp,scp,rcp,cp)
	-s transfer_target_system (required for ftp,scp,rcp)
	-d transfer_target_dir
	-u transfer_userid (required for ftp,scp,rcp)
	-p transfer_password (Only required for ftp)
	-O transfer_options
"
}

function check_targ_dir {
 if [ -z "$transfer_target_dir" ] ; then
   echo " No target directory set - please run config script or set on commandline"
   exit
 fi
}

function check_targ_sys {
 if [ -z "$transfer_target_system" ] ; then
   echo " No target system set - please run config script or set on commandline"
   exit
 fi
}

function check_targ_uid {
 if [ -z "$transfer_userid" ] ; then
   echo " No target userid set - please run config script or set on commandline"
   exit
 fi
}

function check_targ_pwd {
 if [ -z "$transfer_password" ] ; then
   echo " No password set - please run config script or set on commandline"
   exit
 fi
}

function check_parms {
 if [ -z "$transfer_method" ] ; then
#   echo " No transfer method set - please run config script or set on commandline"
   echo " No transfer method set - please set in config file or on commandline"
   exit
 fi
 if [ ! -f "$data_current" ] ; then
   echo " Unable to find data_current: $data_current"
   exit
 fi
 case $transfer_method in
	ftp)	check_targ_dir; check_targ_sys; check_targ_uid; check_targ_pwd;;
	rcp)	check_targ_dir; check_targ_sys; check_targ_uid;;
	scp)	check_targ_dir; check_targ_sys; check_targ_uid;;
	cp)	check_targ_dir;;
	*)	echo "unknown transfer method: $transfer_method"
		echo "please run config script or set on commandline"
		echo "exiting..." ; exit;
		;;
esac
}

#MAIN#
# Ensure transfer options are set.  If not, show usage message.
if [ "xx" = "x${1}x" ] ; then 
   usage
   exit 1 
fi

while getopts "D:v:o:c:p:u:s:d:m:O:" opt ; do
  case $opt in
        v)      verbose="T" ;;
        D)      debug="T" ;;
        l)      opt_data_current=$OPTARG ;;
        c)      opt_configfile=$OPTARG ;;
        p)      opt_password=$OPTARG ;;
        u)      opt_userid=$OPTARG ;;
        s)      opt_target_system=$OPTARG ;;
        d)      opt_target_dir=$OPTARG ;;
        m)      opt_transfer_method=$OPTARG ;;
        O)      opt_transfer_options=$OPTARG ;;
        ?)      usage;  exit ;;
  esac
done
shift `expr $OPTIND - 1`


if [ ! -z "$opt_configfile" ] ; then configfile=$opt_configfile; fi
##### quick+dirty configfile - unsafe!
if [ ! -z "$configfile" ] ; then
   . $configfile
fi

## this allows just setting sysname in the configfile
def_data_current="${invocdir}/mpgdata2_current.$sysname"    # default current data_current name
if [ -z "$data_current" ] ; then data_current="$def_data_current" ; fi

if [ ! -z "$debug" ] ; then
   verbose="T";
   echo " -- options after configfile processing: -- ";
   echo "data_current:$data_current"
   echo "transfer_method:$transfer_method"
   echo "target_system:$transfer_target_system"
   echo "target_dir:$transfer_target_dir"
   echo "transfer_userid:$transfer_userid"
   echo "transfer_password:$transfer_password"
   echo "transfer_options:$transfer_options"
fi

# commandline options override config file
if [ ! -z "$opt_data_current" ] ; then data_current=$opt_data_current; fi
if [ ! -z "$opt_transfer_method" ] ; then transfer_method=$opt_transfer_method; fi
if [ ! -z "$opt_target_system" ] ; then transfer_target_system=$opt_target_system; fi
if [ ! -z "$opt_target_dir" ] ; then transfer_target_dir=$opt_target_dir; fi
if [ ! -z "$opt_userid" ] ; then transfer_userid=$opt_userid; fi
if [ ! -z "$opt_password" ] ; then transfer_password=$opt_password; fi
if [ ! -z "$opt_transfer_options" ] ; then transfer_options=$opt_transfer_options; fi

if [ ! -z "$debug" ] ; then
   echo " ----- options after commandline option processing:";
   echo "data_current:$data_current"
   echo "transfer_method:$transfer_method"
   echo "target_system:$transfer_target_system"
   echo "target_dir:$transfer_target_dir"
   echo "transfer_userid:$transfer_userid"
   echo "transfer_password:$transfer_password"
   echo "transfer_options:$transfer_options"
fi

check_parms

## create transfer command

case $transfer_method in
	ftp)	
	      	command="ftp -i -n <<-EOF
		open $transfer_target_system
		user $transfer_userid $transfer_password
		cd $transfer_target_dir
                bin 
		mput $data_current
		quit
EOF
"
	;;
	rcp)	command="rcp $transfer_options $data_current ${transfer_userid}@${transfer_target_system}:$transfer_target_dir" ;;
	scp)	command="scp $transfer_options $data_current ${transfer_userid}@${transfer_target_system}:$transfer_target_dir" ;;
	cp)	command="cp $transfer_options $data_current $transfer_target_dir" ;;
	*)	echo "unknown transfer method: $tm - exiting"; exit;;
esac

if [ ! -z "$debug" ] ; then
   echo "command=$command"
fi

eval "$command"
rc=$?
if [ "$rc" != "0" ] ; then 
  echo "ERROR in data transfer: error code: $rc"
fi

