#!/bin/ksh
#
# $Id: transfer_script.sh,v 1.13 2012/02/14 04:33:34 root Exp root $
#
# The following code is Confidential and is covered by the installation license
# Copyright 2004-2012 by Midrange Performance Group
#
#&& Transfer script can aid in gathering nmon/sar data to a common location.
#&& cp is local or a mounted system, 
#&& ftp requires password, 
#&& scp requires passwordless key based authentication, 
#&& rcp requires administrative setup of /etc/rhost files  
#
#& Example configuration files are included.
#
#&% To execute this script, substitute a config file depending on the type of transfer
#&% ./transfer_script.sh -c ./transfer_scp.config 
#&% ./transfer_script.sh -c ./transfer_rcp.config 
#&% ./transfer_script.sh -c ./transfer_ftp.config 
#&% ./transfer_script.sh -c ./transfer_cp.config 
#&% These config files need to be modified to the specific needs of the customer
#
#&@ Requires user modified configuration file for normal operation.
#
## Jack Miller 4/18/2009
#       Updated shell to ksh, not sh compliant.
#
## Jack Miller 10/12/2009
#       Added configuration examples, and additional help

function usage {
echo "usage: $0 [options]
  where options may be any of:
	-c configfile (or options can be included manually on the command line)
	-m transfer_method (Choices are ftp,scp,rcp,cp)
	-s transfer_target_system (required for ftp,scp,rcp)
	-d transfer_target_dir
	-o transfer_data_file
	-l transfer_data_current
	-x transfer_data_file_external
	-r transfer_rackfile
	-u transfer_userid (required for ftp,scp,rcp)
	-p transfer_password (Only required for ftp)
	-O transfer_options
	-P permissions on files (640 or 644 recommended) 
"
}

function check_targ_dir {
 if [ -z "$transfer_target_dir" ] ; then
   echo " No target directory set - please run config script or set on commandline"
   exit
 fi
}

function check_targ_sys {
 if [ -z "$transfer_target_system" ] ; then
   echo " No target system set - please run config script or set on commandline"
   exit
 fi
}

function check_targ_uid {
 if [ -z "$transfer_userid" ] ; then
   echo " No target userid set - please run config script or set on commandline"
   exit
 fi
}

function check_targ_pwd {
 if [ -z "$transfer_password" ] ; then
   echo " No password set - please run config script or set on commandline"
   exit
 fi
}

function check_parms {
 if [ -z "$transfer_method" ] ; then
#   echo " No transfer method set - please run config script or set on commandline"
   echo " No transfer method set - please set in config file or on commandline"
   exit
 fi
 if [ ! -f "$datafile" ] ; then
   echo " Unable to find datafile: $datafile"
   exit
 fi
 if [ ! -f "$rackfile" ] ; then
   echo " Unable to find rackfile: $rackfile"
   exit
 fi
 case $transfer_method in
	ftp)	check_targ_dir; check_targ_sys; check_targ_uid; check_targ_pwd;;
	rcp)	check_targ_dir; check_targ_sys; check_targ_uid;;
	scp)	check_targ_dir; check_targ_sys; check_targ_uid;;
	cp)	check_targ_dir;;
	*)	echo "unknown transfer method: $transfer_method"
		echo "please run config script or set on commandline"
		echo "exiting..." ; exit;
		;;
esac
}

#MAIN#
# Ensure transfer options are set.  If not, show usage message.
if [ "xx" = "x${1}x" ] ; then 
   usage
   exit 1 
fi

while getopts "D:v:o:c:p:u:s:d:m:O:r:l:x:P:" opt ; do
  case $opt in
        v)      verbose="T" ;;
        D)      debug="T" ;;
        o)      opt_datafile=$OPTARG ;;
        l)      opt_data_current=$OPTARG ;;
        x)      opt_datafile_external=$OPTARG ;;
        c)      opt_configfile=$OPTARG ;;
	r)	opt_rackfile=$OPTARG ;;
        p)      opt_password=$OPTARG ;;
        u)      opt_userid=$OPTARG ;;
        s)      opt_target_system=$OPTARG ;;
        d)      opt_target_dir=$OPTARG ;;
        m)      opt_transfer_method=$OPTARG ;;
        O)      opt_transfer_options=$OPTARG ;;
        P)      opt_permsfiles=$OPTARG ;;
        ?)      usage;  exit ;;
  esac
done
shift `expr $OPTIND - 1`


if [ ! -z "$opt_configfile" ] ; then configfile=$opt_configfile; fi
##### quick+dirty configfile - unsafe!
if [ ! -z "$configfile" ] ; then
   . $configfile
fi

## this allows just setting datafile in the configfile
def_datafile="${invocdir}/mpgdata2.$sysname"    # default datafile name
if [ -z "$datafile" ] ; then datafile="$def_datafile" ; fi

## this allows just setting datafile_external in the configfile
if [ -s ${invocdir}/mpgdata3.$sysname ] ; then  
   def_datafile_external="${invocdir}/mpgdata3.$sysname"    # default datafile_external name
else
   def_datafile_external="" # if no default datafile_external name set blank
fi
if [ -z "$datafile_external" ] ; then datafile_external="$def_datafile_external" ; fi

## this allows just setting data_current in the configfile
def_data_current="${invocdir}/mpgdata2_current.$sysname"    # default datafile name
if [ -z "$data_current" ] ; then datafile="$def_data_current" ; fi

if [ ! -z "$debug" ] ; then
   verbose="T";
   echo " -- options after configfile processing: -- ";
   echo "permsfiles:$permsfiles"
   echo "datafile:$datafile"
   echo "data_current:$data_current"
   echo "rackfile:$rackfile"
   echo "transfer_method:$transfer_method"
   echo "target_system:$transfer_target_system"
   echo "target_dir:$transfer_target_dir"
   echo "transfer_userid:$transfer_userid"
   echo "transfer_password:$transfer_password"
   echo "transfer_options:$transfer_options"
fi

# commandline options override config file
if [ ! -z "$opt_permsfiles" ] ; then permsfiles=$opt_permsfiles; fi
if [ ! -z "$opt_datafile" ] ; then datafile=$opt_datafile; fi
if [ ! -z "$opt_datafile_external" ] ; then datafile_external=$opt_datafile_external; fi
if [ ! -z "$opt_data_current" ] ; then data_current=$opt_data_current; fi
if [ ! -z "$opt_transfer_method" ] ; then transfer_method=$opt_transfer_method; fi
if [ ! -z "$opt_target_system" ] ; then transfer_target_system=$opt_target_system; fi
if [ ! -z "$opt_target_dir" ] ; then transfer_target_dir=$opt_target_dir; fi
if [ ! -z "$opt_userid" ] ; then transfer_userid=$opt_userid; fi
if [ ! -z "$opt_password" ] ; then transfer_password=$opt_password; fi
if [ ! -z "$opt_transfer_options" ] ; then transfer_options=$opt_transfer_options; fi

if [ ! -z "$debug" ] ; then
   echo " ----- options after commandline option processing:";
   echo "permsfiles:$permsfiles"
   echo "datafile:$datafile"
   echo "datafile_external:$datafile_external"
   echo "data_current:$data_current"
   echo "rackfile:$rackfile"
   echo "transfer_method:$transfer_method"
   echo "target_system:$transfer_target_system"
   echo "target_dir:$transfer_target_dir"
   echo "transfer_userid:$transfer_userid"
   echo "transfer_password:$transfer_password"
   echo "transfer_options:$transfer_options"
fi

check_parms

## chmod the files that are transferred.
## If variable permsfiles set chmod to the files before they are transferred.
if [ ! -z "$permsfiles" ] ; then
   chmod ${permsfiles} $transfer_options $datafile $datafile_external $data_current $rackfile  2>/dev/null
fi

## create transfer command

case $transfer_method in
	ftp)	
	      	command="ftp -i -n <<-EOF
		open $transfer_target_system
		user $transfer_userid $transfer_password
		cd $transfer_target_dir
                bin 
		mput $datafile $datafile_external $data_current
		put $rackfile
		quit
EOF
"
	;;
	rcp)	command="rcp $transfer_options $datafile $datafile_external $data_current $rackfile ${transfer_userid}@${transfer_target_system}:$transfer_target_dir" ;;
	scp)	command="scp $transfer_options $datafile $datafile_external $data_current $rackfile ${transfer_userid}@${transfer_target_system}:$transfer_target_dir" ;;
	cp)	command="cp $transfer_options $datafile $datafile_external $data_current $rackfile $transfer_target_dir" ;;
	*)	echo "unknown transfer method: $tm - exiting"; exit;;
esac

if [ ! -z "$debug" ] ; then
   echo "command=$command"
fi

eval "$command"
rc=$?
if [ "$rc" != "0" ] ; then 
   echo "ERROR in data transfer: error code: $rc"
fi

